/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define _color_c_

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "screen.h"
#include "house.h"

/* Cursor controls */
#define C_UP    "\E[A"
#define C_DOWN  "\E[B"
#define C_RIGHT "\E[C"
#define C_LEFT  "\E[D"
#define C_HOME  "\E[H"
#define C_CLR   C_HOME"\E[J"

/* List of colors */
const char *COLORLIST[] =
{
  COL_OFF, COL_NONE, COL_BLACK,
  COL_RED, COL_GREEN, COL_YELLOW,
  COL_BLUE, COL_MAGENTA, COL_CYAN,
  COL_WHITE, COL_FG_OFF,

  COL_E_BLACK, COL_E_RED, COL_E_GREEN,
  COL_E_YELLOW, COL_E_BLUE, COL_E_MAGENTA,
  COL_E_CYAN, COL_E_WHITE, COL_FG_OFF,

  COL_BK_BLACK, COL_BK_RED, COL_BK_GREEN,
  COL_BK_YELLOW, COL_BK_BLUE, COL_BK_MAGENTA,
  COL_BK_CYAN, COL_BK_WHITE, COL_OFF,

  C_UP, C_DOWN, C_RIGHT, C_LEFT, C_HOME, C_CLR,
  STYLE_UNDERLINE, STYLE_FLASH, STYLE_REVERSE
};

#define LAST_COLOR 37

#define BUFSPACE	12*1024	/* This should be equal to LARGE_BUFSIZE
				   in structs.h */

#define BUFSIZE BUFSPACE-1


#define START_CHAR '&'

#if 1
static char out_buf[BUFSPACE], insert_text[BUFSPACE];
void proc_color(char *inbuf, int color_lvl)
{
  int has_color = (color_lvl > 1), has_cursor = ((color_lvl == 1) || (color_lvl == 3)),
   current_color = -1;
  register int inpos = 0, outpos = 0;
  int remaining, color = -2;

  if (*inbuf == '\0')
    return;			/* if first char is null */

/* If color level is 1 (sparse), then character will get cursor controls
   only.  If color level is 2, character will get color codes only.  If
   color is complete (3), character will get both.   Color level 0 removes
   all color codes, of course. =) */


  *out_buf = '\0';
  *insert_text = '\0';


  while (inbuf[inpos] != '\0') {
    remaining = strlen(inbuf) - inpos;

    if (remaining > 2) {
      if (inbuf[inpos] == START_CHAR) {
	*insert_text = '\0';
	switch (inbuf[inpos + 1]) {
#if 0
/* stop checking for doubles */
	case START_CHAR:	/* just an ampersand */
	  *insert_text = START_CHAR;
	  insert_text[1] = '\0';
	  inpos += 2;
	  break;
#endif
	case '+':		/* foreground color */
	  switch (inbuf[inpos + 2]) {
	  case 'l':
	    color = 2;
	    break;
	  case 'L':
	    color = 11;
	    break;
	  case 'r':
	    color = 3;
	    break;
	  case 'R':
	    color = 12;
	    break;
	  case 'g':
	    color = 4;
	    break;
	  case 'G':
	    color = 13;
	    break;
	  case 'y':
	    color = 5;
	    break;
	  case 'Y':
	    color = 14;
	    break;
	  case 'b':
	    color = 6;
	    break;
	  case 'B':
	    color = 15;
	    break;
	  case 'm':
	  case 'p':
	    color = 7;
	    break;
	  case 'M':
	  case 'P':
	    color = 16;
	    break;
	  case 'c':
	    color = 8;
	    break;
	  case 'C':
	    color = 17;
	    break;
	  case 'w':
	    color = 9;
	    break;
	  case 'W':
	    color = 18;
	    break;
	  case 'O':
	  case 'o':
	    color = 10;
	    break;
	  case 'U':
	  case 'u':
	    color = 35;
	    break;
	  case 'F':
	  case 'f':
	    color = 36;
	    break;
	  case 'V':
	  case 'v':
	    color = 37;
	    break;
	  case '0':
	    color = 0;
	    break;
	  default:
	    color = 1;		/* no change */
	  }

	  if (color != current_color)
	    if (has_color)
	      strcpy(insert_text, COLORLIST[color]);
	  inpos += 3;
	  current_color = color;
	  break;

	case '=':		/* background color */
	  switch (inbuf[inpos + 2]) {
	  case 'l':
	    color = 20;
	    break;
	  case 'r':
	    color = 21;
	    break;
	  case 'g':
	    color = 22;
	    break;
	  case 'y':
	    color = 23;
	    break;
	  case 'b':
	    color = 24;
	    break;
	  case 'm':
	  case 'p':
	    color = 25;
	    break;
	  case 'c':
	    color = 26;
	    break;
	  case 'w':
	    color = 27;
	    break;
	  case 'o':
	    color = 0;
	    break;
	  default:
	    color = 1;		/* no change */
	  }

	  if (color != current_color)
	    if (has_color)
	      strcpy(insert_text, COLORLIST[color]);
	  inpos += 3;
	  current_color = color;
	  break;

#if 0
	case '@':		/* cursor control */
	  switch (inbuf[inpos + 2]) {
	  case 'u':
	    color = 29;
	    break;
	  case 'd':
	    color = 30;
	    break;
	  case 'r':
	    color = 31;
	    break;
	  case 'l':
	    color = 32;
	    break;
	  case 'h':
	    color = 33;
	    break;
	  case 'c':
	    color = 34;
	    break;
	  default:
	    color = 1;		/* no change */
	  }
#endif
	  if (has_cursor)
	    strcpy(insert_text, COLORLIST[color]);
	  inpos += 3;
	  break;
#if 0
	case 'n':
	  color = 0;
	  if (color != current_color)
	    if (has_cursor)
	      strcpy(insert_text, COLORLIST[color]);
	  inpos += 2;
	  break;
#endif
	default:		/* not a color code, just give them their & back */
	  *insert_text = START_CHAR;
	  insert_text[1] = '\0';
	  inpos += 1;
	}			/* switch */

	if (color_lvl == 0)
	  out_buf[outpos] = '\0';

	/* don't overfill buffer */
	if ((strlen(out_buf) + strlen(insert_text)) < BUFSIZE) {
	  out_buf[outpos] = '\0';	/* so strcat is not confused by whatever out_buf WAS */
	  strcat(out_buf, insert_text);
	  outpos = strlen(out_buf);
	} else
	  mudlog("buffer overflow", CMP, LVL_GOD, FALSE);
      }
      /* if char is '&' (START_CHAR) */
      else {
	if (outpos < BUFSIZE) {
	  out_buf[outpos] = inbuf[inpos];
	  inpos++;
	  outpos++;
	}
      }

    }
    /* if remaining > 2 */
    else {
      if (outpos < BUFSIZE) {
	out_buf[outpos] = inbuf[inpos];
	inpos++;
	outpos++;
      }
    }


  }				/* while */
#if 0
  sprintf(&out_buf[outpos], "\033[0m");
#endif
  out_buf[outpos] = '\0';

  /* printf("outbuf: %s\n",out_buf); *//* for debugging */

  strcpy(inbuf, out_buf);

}
#else
struct ansi_color {
  const char *symbol;		/* Symbol used in the game. eg &+symbol */
  const char *fg_code;		/* ^[[fg_codem; = foreground color code */
  const char *bg_code;		/* Same except it's the background color */
} color_table[] =

{

  {
    "L", "30", "40"
  },				/* * Black            */
  {
    "R", "31", "41"
  },				/* * Red              */
  {
    "G", "32", "42"
  },				/* * Green            */
  {
    "Y", "33", "43"
  },				/* * Yellow           */
  {
    "B", "34", "44"
  },				/* * Blue             */
  {
    "M", "35", "45"
  },				/* * Magenta          */
  {
    "C", "36", "46"
  },				/* * Cyan             */
  {
    "W", "37", "47"
  },				/* * White            */
  {
    NULL, NULL, NULL
  }				/* * End of the table */
};

int find_color_entry(int c)
{
  int i = 0;
  char s;

  s = UPPER(c);

  while ((color_table[i].symbol != NULL) && (*color_table[i].symbol != s))
    i++;

  return i;
}

void proc_color(char *buf, int color_lvl)
{
  bool flg = TRUE, bold = FALSE, blink = FALSE;
  int ibuf = 0;
  int i, j, k, bg = 0;

  ibuf = strlen(buf);

  /* Go through and convert/strip color symbols -Ithor */
  for (i = 0, j = 0; (i < ibuf) && (j < (sizeof(buf2))); i++) {
    if (buf[i] == '&') {
      i++;
      if (i >= ibuf)
	continue;

      switch (buf[i]) {
      case '&':
	buf2[j++] = '&';
	break;

      case 'N':
      case 'n':
	if (flg) {
	  sprintf(&buf2[j], "\033[0m");
	  j += 4;
	}
	break;

      case '+':
      case '-':
	bg = (buf[i] == '-');
	i++;
	if (i >= ibuf)
	  continue;

	bold = bg ? 0 : (isupper(buf[i])) ? 1 : 0;
	blink = !bg ? 0 : (isupper(buf[i])) ? 1 : 0;
	k = find_color_entry(buf[i]);
	if (color_table[k].symbol != NULL) {
	  if (flg) {
	    sprintf(&buf2[j], "\033[%s%s%sm", bold ? "1;" : "", blink ? "5;" : "",
		 (bg ? color_table[k].bg_code : color_table[k].fg_code));
	    j += (5 + (bold ? 2 : 0) + (blink ? 2 : 0));
	  }
	} else {
	  sprintf(&buf2[j], "&%c%c", (bg ? '-' : '+'), buf[i]);
	  j += 3;
	}
	break;

      case '=':
	i++;
	if (i >= ibuf)
	  continue;

	blink = (isupper(buf[i]) ? 1 : 0);
	bg = find_color_entry(buf[i]);
	i++;
	if (i >= ibuf)
	  continue;

	bold = (isupper(buf[i]) ? 1 : 0);
	k = find_color_entry(buf[i]);
	if ((color_table[k].symbol != NULL)
	    && (color_table[bg].symbol != NULL)) {
	  if (flg) {
	    sprintf(&buf2[j], "\033[%s%s%s;%sm", bold ? "1;" : "", blink ? "5;" : "",
		    color_table[bg].bg_code, color_table[k].fg_code);
	    j += (8 + (bold ? 2 : 0) + (blink ? 2 : 0));
	  }
	} else {
	  sprintf(&buf2[j], "&=%c%c", buf[i - 1], buf[i]);
	  j += 4;
	}
	break;

      default:
	sprintf(&buf2[j], "&%c", buf[i]);
	j += 2;
	break;
      }
    } else if (flg && (buf[i] == '\n')) {
      /* Want normal color at EoLN */
      sprintf(&buf2[j], "\033[0m\n");
      j += 5;
    } else {
      buf2[j++] = buf[i];
    }
  }
}
#endif
