/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <dirent.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "db.h"

#include "boards.h"
#include "interpreter.h"
#include "handler.h"

struct board_info_type *board_info;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern struct index_data *obj_index;

void Board_init_boards(void)
{
  int n, o, board_vnum;
  struct dirent **namelist;

  n = scandir(BOARD_DIRECTORY, &namelist, 0, alphasort);
  if (n < 0) {
    log("Funny, no board files found.\n");
    return;
  }
  for (o = 0; o != n; o++) {
    if (strcmp("..", namelist[o]->d_name) &&
	strcmp(".", namelist[o]->d_name)) {
      sscanf(namelist[o]->d_name, "%d", &board_vnum);
      create_new_board(board_vnum);
    }
  }

  /* just before we close, lets look at boards huh? */

  look_at_boards();
}

struct board_info_type *create_new_board(int board_vnum)
{
  extern struct board_info_type *board_info;

  int boards = 0, t_messages = 0, error = 0, t[4];
  char buf[512];
  FILE *fl;
  struct board_info_type *temp_board = NULL, *old_board;
  struct obj_data *obj;

  sprintf(buf, "%s/%d", BOARD_DIRECTORY, board_vnum);

  if (!(fl = fopen(buf, "r"))) {

    /* this is for boards which exist as an object, but don't have a file yet */
    /* Ie, new boards */

    if (!(fl = fopen(buf, "w"))) {
      log("Hm. Error while creating new board file");
    } else {

      obj = read_object(real_object(board_vnum), REAL);

      CREATE(temp_board, struct board_info_type, 1);
      READ_LVL(temp_board) = GET_OBJ_VAL(obj, 0);
      READ_LVL(temp_board) = 30;
      WRITE_LVL(temp_board) = GET_OBJ_VAL(obj, 1);
      REMOVE_LVL(temp_board) = GET_OBJ_VAL(obj, 2);
      BOARD_VNUM(temp_board) = board_vnum;
      BOARD_MNUM(temp_board) = 0;
      BOARD_NEXT(temp_board) = NULL;
      BOARD_MESSAGES(temp_board) = NULL;
      extract_obj(obj);
      fprintf(fl, "Board File\n%d %d %d %d\n", READ_LVL(temp_board),
	      WRITE_LVL(temp_board), REMOVE_LVL(temp_board), BOARD_MNUM(temp_board));
      fclose(fl);

      old_board = board_info;
      board_info = temp_board;
      BOARD_NEXT(temp_board) = old_board;
      return temp_board;

    }
  } else {
    get_line(fl, buf);
    if ((strcasecmp("Board File", buf))) {
    } else {
      boards++;
      CREATE(temp_board, struct board_info_type, 1);
      temp_board->vnum = board_vnum;
      get_line(fl, buf);
      if (sscanf(buf, "%d", t) != 1) {
	error = 1;
      }
      if (!(real_object(board_vnum))) {
	sprintf(buf, "Erasing non-existant board file: %d",
		board_vnum);
	log(buf);
	sprintf(buf, "%s/%d", BOARD_DIRECTORY, board_vnum);
	unlink(buf);
	if (temp_board)
	  FREE(temp_board);
	return NULL;		/* realize why I do this yes? */
      }
      obj = read_object(real_object(board_vnum), REAL);

      READ_LVL(temp_board) = GET_OBJ_VAL(obj, 0);
      WRITE_LVL(temp_board) = GET_OBJ_VAL(obj, 1);
      REMOVE_LVL(temp_board) = GET_OBJ_VAL(obj, 2);
      BOARD_MNUM(temp_board) = t[0];
      BOARD_NEXT(temp_board) = NULL;
      BOARD_MESSAGES(temp_board) = NULL;
      extract_obj(obj);

      if (error) {
	sprintf(buf, "Parse error in board %d - attempting to continue\n",
		BOARD_VNUM(temp_board));
	log(buf);
	/* attempting board ressurection */

	if (!(t[0] > 0 && t[0] < LVL_IMPL)) {
	  READ_LVL(temp_board) = LVL_IMMORT;
	}
	if (!(t[1] > 0 && t[1] < LVL_IMPL)) {
	  WRITE_LVL(temp_board) = LVL_IMMORT;
	}
	if (!(t[2] > 0 && t[2] < LVL_IMPL)) {
	  REMOVE_LVL(temp_board) = LVL_IMPL;
	}
	if (!(t[3] > 0 && t[3] < LVL_IMPL)) {
	  /*  This is an indication of a very unfun board */
	  /* I guess if I ever optimize any of this raggedy code */
	  /* this will actually mean something. Right now, its nada */
	  BOARD_MNUM(temp_board) = -1;
	}
      }
      while (get_line(fl, buf)) {
	/* this is the loop that reads the messages 
	   and also the message read info stuff */
	if (*buf == 'S') {

	} else if (*buf == '#') {
	  if (parse_message(fl, temp_board))
	    t_messages++;
	}
      }
      BOARD_MNUM(temp_board) = t_messages;

      /* add it to the list. */

      old_board = board_info;
      board_info = temp_board;
      BOARD_NEXT(temp_board) = old_board;

    }
    fclose(fl);
  }
  /* hold on. Lets think about this:
     Board messages are added in the folowing way - 1,2,3,4,5...etc
     However, when they're read in, its fifo, 5,4,3,2,1, so between
     each reboot, the existing order is reversed.  hm. */

  reverse_message_order(temp_board);

  return temp_board;
}

void reverse_message_order(struct board_info_type *t_board)
{
  struct board_msg_info *old, *new, *store;

  if (!t_board)
    return;

  old = BOARD_MESSAGES(t_board);

  if (!old || MESG_NEXT(old) == NULL) {
    return;

  }
  store = new = NULL;

  /* yes, there's a better way for doing this, but I think I had some
     problems with second thoughts after accidently deleting stuff */

  while (old) {
    store = new;
    CREATE(new, struct board_msg_info, 1);
    MESG_POSTER(new) = MESG_POSTER(old);
    MESG_TIMESTAMP(new) = MESG_TIMESTAMP(old);
    MESG_SUBJECT(new) = MESG_SUBJECT(old);
    MESG_DATA(new) = MESG_DATA(old);
    MESG_NEXT(new) = store;

    store = old;
    old = MESG_NEXT(old);
    FREE(store);
  }
  BOARD_MESSAGES(t_board) = new;

}

byte parse_message(FILE * fl, struct board_info_type *t_board)
{
  struct board_msg_info *tmsg, *t2msg;
  char subject[81];
  char buf[8192];

  CREATE(tmsg, struct board_msg_info, 1);

  fscanf(fl, "%ld\n", &(MESG_POSTER(tmsg)));
  fscanf(fl, "%ld\n", &(MESG_TIMESTAMP(tmsg)));

  if (!get_line(fl, subject)) {
    FREE(tmsg);
    log("SYSERR: Board could not be fully loaded, due to missing lines.");
    return 0;
  }
  MESG_SUBJECT(tmsg) = str_dup(subject);
  MESG_DATA(tmsg) = fread_string(fl, buf);
  MESG_NEXT(tmsg) = NULL;

  t2msg = BOARD_MESSAGES(t_board);
  BOARD_MESSAGES(t_board) = tmsg;
  MESG_NEXT(tmsg) = t2msg;

  return 1;
}

void look_at_boards()
{
  int counter, messages = 0;
  struct board_info_type *tboard = board_info;
  struct board_msg_info *msg;

  for (counter = 0; tboard; counter++) {
    msg = BOARD_MESSAGES(tboard);
    while (msg) {
      messages++;
      msg = MESG_NEXT(msg);
    }
    tboard = BOARD_NEXT(tboard);
  }

  sprintf(buf, "There are %d boards located; %d messages", counter, messages);
  log(buf);
}

int Board_show_board(int board_vnum, struct char_data *ch)
{
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message;
  char *tmstr;
  int msgcount = 0;
  char buf3[MAX_STRING_LENGTH];
  char name[127];
  /* board locate */
  if (IS_NPC(ch)) {
    send_to_char("Gosh.. now .. if only mobs could read.. you'd be doing good.\r\n", ch);
    return 1;
  }
  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }
  if (!thisboard) {

    sprintf(buf, "Creating new board - board #%d", board_vnum);
    log(buf);
    if (!(thisboard = create_new_board(board_vnum))) {
      log("Couldn't allocate a board!!!");
      return 0;
    }
  }
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char("You try but fail to understand the holy words.\r\n", ch);
    return 1;
  }
  /* send the standard board boilerplate */


  sprintf(buf2, "This is a bulletin board.\r\n"
	  "Usage:READ/REMOVE/RESPOND <messg #>, WRITE<header>.\r\n");
  if (!BOARD_MNUM(thisboard) || !BOARD_MESSAGES(thisboard)) {
    strcat(buf2, "The board is empty.\r\n");
    send_to_char(buf2, ch);
    return 1;
  } else {
    sprintf(buf2, "%sThere %s %d %s on the board.\r\n",
	    buf2
	    ,(BOARD_MNUM(thisboard) == 1) ? "is" : "are",
	BOARD_MNUM(thisboard), (BOARD_MNUM(thisboard) == 1) ? "message" :
	    "messages");
  }
  message = BOARD_MESSAGES(thisboard);


  for (message = BOARD_MESSAGES(thisboard); message; message = MESG_NEXT(message)) {
    tmstr = (char *) asctime(localtime(&MESG_TIMESTAMP(message)));
    *(tmstr + strlen(tmstr) - 1) = '\0';

    sprintf(name, "%s", get_name_by_id(MESG_POSTER(message)));

    sprintf(buf3, "(%s)", CAP(name));

    sprintf(buf, "(&+B%2d&n) : %6.10s %-15s :: %s \r\n",
	    ++msgcount,
	    tmstr,
	    buf3,
	    MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject");
    strcat(buf2, buf);
  }
  page_string(ch->desc, buf2, 1);
  return 1;
}


int Board_display_msg(int board_vnum, struct char_data *ch, int arg)
{
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message;
  char *tmstr;
  int msgcount;
  char name[127];

  if (IS_NPC(ch)) {
    send_to_char("Silly mob - reading is for pcs!\r\n", ch);
    return 1;
  }
  /* guess we'll have to locate the board now in the list */

  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    sprintf(buf, "Creating new board - board #%d", board_vnum);
    log(buf);
    thisboard = create_new_board(board_vnum);
  }
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char("You try but fail to understand the holy words.\r\n", ch);
    return 1;
  }
  if (!BOARD_MESSAGES(thisboard)) {
    send_to_char("The board is empty!\r\n", ch);
    return 1;
  }
  /* now we locate the message. */
  message = BOARD_MESSAGES(thisboard);
  if (arg < 1) {
    send_to_char("You must specify the (positive) number of the message to be read!\r\n", ch);
    return 1;
  }
  for (msgcount = arg; message && msgcount != 1; msgcount--) {
    message = MESG_NEXT(message);
  }
  if (!message) {
    send_to_char("That message exists only in your imagination.\r\n", ch);
    return 1;
  }
  /* before we print out the message, we may as well restore a human
     readable timestamp. */
  tmstr = (char *) asctime(localtime(&MESG_TIMESTAMP(message)));
  *(tmstr + strlen(tmstr) - 1) = '\0';
  sprintf(name, "%s", get_name_by_id(MESG_POSTER(message)));
  sprintf(buf, "Message %d : %6.10s (%s) :: %s\r\n\r\n%s\r\n",
	  arg,
	  tmstr,
	  CAP(name),
	  MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject",
	  MESG_DATA(message) ? MESG_DATA(message) : "Looks like this message is empty.");
  page_string(ch->desc, buf, 1);
  Board_save_board(BOARD_VNUM(thisboard));
  return 1;

}

int Board_display_next_msg(int board_vnum, struct char_data *ch)
{
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message;
  char *tmstr;
  int mem, yesno = 0;
  char name[127];

  if (IS_NPC(ch)) {
    send_to_char("Silly mob - reading is for pcs!\r\n", ch);
    return 1;
  }
  /* guess we'll have to locate the board now in the list */

  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    sprintf(buf, "Creating new board - board #%d", board_vnum);
    log(buf);
    thisboard = create_new_board(board_vnum);
  }
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char("You try but fail to understand the holy words.\r\n", ch);
    return 1;
  }
  if (!BOARD_MESSAGES(thisboard)) {
    send_to_char("The board is empty!\r\n", ch);
    return 1;
  }
  /* now we locate the message. */
  for (message = BOARD_MESSAGES(thisboard); message; message = MESG_NEXT(message)) {
    yesno = mesglookup(message, ch, thisboard);
    if (MESG_SUBJECT(message) && yesno)
      break;
  }
  if (!message) {
    send_to_char("Can't find another unread message.\r\n", ch);
    return 1;
  }
  /* Have message, lets add the fact that this player read the mesg */

  mem = ((MESG_TIMESTAMP(message) % 301 + MESG_POSTER(message) % 301) % 301);

  /* before we print out the message, we may as well restore a human
     readable timestamp. */
  tmstr = (char *) asctime(localtime(&MESG_TIMESTAMP(message)));
  *(tmstr + strlen(tmstr) - 1) = '\0';
  sprintf(name, "%s", get_name_by_id(MESG_POSTER(message)));
  sprintf(buf, "Next unread message: %6.10s (%s) :: %s\r\n\r\n%s\r\n",
	  tmstr,
	  CAP(name),
	  MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject",
	  MESG_DATA(message) ? MESG_DATA(message) : "Looks like this message is empty.");
  page_string(ch->desc, buf, 1);
  Board_save_board(BOARD_VNUM(thisboard));
  return 1;

}

int Board_remove_msg(int board_vnum, struct char_data *ch, int arg)
{
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message, *old_message;
  struct descriptor_data *d;
  int msgcount;

  if (IS_NPC(ch)) {
    send_to_char("Nuts.. looks like you forgot your eraser back in mobland...\r\n", ch);
    return 1;
  }
  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }
  if (!thisboard) {
    send_to_char("Error: Your board could not be found. Please report.\n", ch);
    sprintf(buf, "Error in Board_remove_msg - board #%d", board_vnum);
    log(buf);
    return 0;
  }
  message = BOARD_MESSAGES(thisboard);
  if (arg < 1) {
    send_to_char("You must specify the (positive) number of the message to be read!\r\n", ch);
    return 1;
  }
  old_message = NULL;
  for (msgcount = arg; message && msgcount != 1; msgcount--) {
    old_message = message;
    message = MESG_NEXT(message);
  }
  if (!message) {
    send_to_char("That message exists only in your imagination.\r\n", ch);
    return 1;
  }
  if (GET_IDNUM(ch) != MESG_POSTER(message) &&
      GET_LEVEL(ch) < REMOVE_LVL(thisboard)) {
    send_to_char("You are not holy enough to remove other peoples messages.\r\n", ch);
    return 1;
  }
  /* perform check for mesg in creation */
  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->str == &(MESG_DATA(message))) {
      send_to_char("At least wait until the author is finished before removing it!\r\n", ch);
      return 1;
    }
  /* everything else is peachy, kill the message */
  if (MESG_NEXT(message)) {
    if (old_message) {
      MESG_NEXT(old_message) = MESG_NEXT(message);
    } else {
      BOARD_MESSAGES(thisboard) = MESG_NEXT(message);
    }
  } else {
    if (old_message) {
      MESG_NEXT(old_message) = NULL;
    } else {
      BOARD_MESSAGES(thisboard) = NULL;
    }
  }

  FREE(message);
  BOARD_MNUM(thisboard) = BOARD_MNUM(thisboard) - 1;

  send_to_char("Message removed.\r\n", ch);
  sprintf(buf, "$n just removed message %d.", arg);
  act(buf, FALSE, ch, 0, 0, PROXIMITY);
  Board_save_board(BOARD_VNUM(thisboard));
  return 1;

}

int Board_save_board(int board_vnum)
{
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message;
  struct obj_data *obj;
  FILE *fl;
  int i = 1;

  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }
  if (!thisboard) {
    thisboard = create_new_board(board_vnum);
  }
  /* before we save to the file, lets make sure that our board is updated */

  obj = read_object(real_object(board_vnum), REAL);
  READ_LVL(thisboard) = GET_OBJ_VAL(obj, 0);
  WRITE_LVL(thisboard) = GET_OBJ_VAL(obj, 1);
  REMOVE_LVL(thisboard) = GET_OBJ_VAL(obj, 2);
  extract_obj(obj);

  sprintf(buf, "%s/%d", BOARD_DIRECTORY, board_vnum);

  if (!(fl = fopen(buf, "wb"))) {
    perror("Error writing board");
    return 0;
  }
  /* now we write out the basic stats of the board */

  fprintf(fl, "Board File\n%d\n", BOARD_MNUM(thisboard));

  for (message = BOARD_MESSAGES(thisboard); message; message = MESG_NEXT(message)) {
    fprintf(fl, "#%d\n"
	    "%ld\n"
	    "%ld\n"
	    "%s\n"
	    "%s~\n",
	    i, MESG_POSTER(message), MESG_TIMESTAMP(message),
	    MESG_SUBJECT(message), MESG_DATA(message));
    i++;
  }

  fclose(fl);
  return 1;
}

void Board_write_message(int board_vnum, struct char_data *ch, char *arg)
{
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message, *old_message;

  /* board locate */
  if (IS_NPC(ch)) {
    send_to_char("Orwellian police thwart your attempt at free speech.\r\n", ch);
    return;
  }
  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }
  if (!thisboard) {
    send_to_char("Error: Your board could not be found. Please report.\n", ch);
    sprintf(buf, "Error in Board_display_msg - board #%d", board_vnum);
    log(buf);
    return;
  }
  if (GET_LEVEL(ch) < WRITE_LVL(thisboard)) {
    send_to_char("You are not holy enough to write on this board.\r\n", ch);
    return;
  }
  if (!*arg || !arg)
    sprintf(arg, "No Subject");

  skip_spaces(&arg);
  delete_doubledollar(arg);

  arg[81] = '\0';

  CREATE(message, struct board_msg_info, 1);
  MESG_POSTER(message) = GET_IDNUM(ch);
  MESG_TIMESTAMP(message) = time(0);
  MESG_SUBJECT(message) = str_dup(arg);
  MESG_NEXT(message) = NULL;
  MESG_DATA(message) = NULL;
  BOARD_MNUM(thisboard) = BOARD_MNUM(thisboard) + 1;

  if (BOARD_MESSAGES(thisboard)) {
    old_message = BOARD_MESSAGES(thisboard);
    BOARD_MESSAGES(thisboard) = message;
    MESG_NEXT(message) = old_message;
  } else
    BOARD_MESSAGES(thisboard) = message;

  send_to_char("Write your message.  (/s saves /h for help)\r\n\r\n", ch);
  act("$n starts to write a message.", TRUE, ch, 0, 0, PROXIMITY);
#ifndef MUDFTP
  if (!IS_NPC(ch))
    SET_BIT(PLR_FLAGS(ch), PLR_WRITING);

  ch->desc->str = &(MESG_DATA(message));
  ch->desc->max_str = MAX_MESSAGE_LENGTH;
  ch->desc->mail_to = board_vnum + BOARD_MAGIC;
#else
  string_write(ch->desc, &(MESG_DATA(message)),
	       MAX_MESSAGE_LENGTH, board_vnum + BOARD_MAGIC, NULL);
#endif
  return;

}

void Board_respond_message(int board_vnum, struct char_data *ch, int mnum)
{
  struct board_info_type *thisboard = board_info;
  struct board_msg_info *message, *old_message, *other;
  char number[MAX_STRING_LENGTH];
  int gcount = 0;

  /* board locate */
  while (thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard = BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    send_to_char("Error: Your board could not be found. Please report.\n", ch);
    sprintf(buf, "Error in Board_display_msg - board #%d", board_vnum);
    log(buf);
    return;
  }
  if (GET_LEVEL(ch) < WRITE_LVL(thisboard)) {
    send_to_char("You are not holy enough to write on this board.\r\n", ch);
    return;
  }
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char("You are not holy enough to respond to this board.\r\n", ch);
    return;
  }
  if (mnum < 0 || mnum > BOARD_MNUM(thisboard)) {
    send_to_char("You can only respond to an actual message.\r\n", ch);
    return;
  }
  other = BOARD_MESSAGES(thisboard);

  /*locate message to be repsponded to */
  for (gcount = 0; other && gcount != (mnum - 1); gcount++)
    other = MESG_NEXT(other);

  CREATE(message, struct board_msg_info, 1);
  MESG_POSTER(message) = GET_IDNUM(ch);
  MESG_TIMESTAMP(message) = time(0);
  sprintf(buf, "Re: %s", MESG_SUBJECT(other));
  MESG_SUBJECT(message) = str_dup(buf);
  MESG_NEXT(message) = NULL;
  MESG_DATA(message) = NULL;
  BOARD_MNUM(thisboard) = BOARD_MNUM(thisboard) + 1;


  if (BOARD_MESSAGES(thisboard)) {
    old_message = BOARD_MESSAGES(thisboard);
    BOARD_MESSAGES(thisboard) = message;
    MESG_NEXT(message) = old_message;
  } else
    BOARD_MESSAGES(thisboard) = message;

  send_to_char("Write your message.  (/s saves /h for help)\r\n\r\n", ch);
  act("$n starts to write a message.", TRUE, ch, 0, 0, PROXIMITY);

  if (!IS_NPC(ch))
    SET_BIT(PLR_FLAGS(ch), PLR_WRITING);


  /* don't need number anymore, so we'll reuse it. */
  sprintf(number, "\t------- Quoted message -------\r\n%s\t------- End Quote -------\r\n", MESG_DATA(other));

  MESG_DATA(message) = str_dup(number);

  ch->desc->backstr = str_dup(number);
  SEND_TO_Q(number, ch->desc);
  ch->desc->str = &(MESG_DATA(message));
  ch->desc->max_str = MAX_MESSAGE_LENGTH;
  ch->desc->mail_to = board_vnum + BOARD_MAGIC;
/*  log("yay!");   debugging use */
  return;

}

int mesglookup(struct board_msg_info *message, struct char_data *ch,
	       struct board_info_type *board)
{
  return 0;
}


ACMD(do_respond)
{
  int found = 0, mnum = 0;
  struct obj_data *obj;
  char number[MAX_STRING_LENGTH];

  if (IS_NPC(ch)) {
    send_to_char("As a mob, you never bothered to learn to read or write.\r\n", ch);
    return;
  }
  for (obj = ch->carrying; obj; obj = obj->next_content) {
    if (GET_OBJ_TYPE(obj) == ITEM_BOARD) {
      found = 1;
      break;
    }
  }
  if (!obj) {
    for (obj = world[ch->in_room].contents; obj; obj = obj->next_content) {
      if (GET_OBJ_TYPE(obj) == ITEM_BOARD) {
	found = 1;
	break;
      }
    }
  }
  if (obj) {
    argument = one_argument(argument, number);
    if (!*number)
      send_to_char("Respond to what?\r\n", ch);
    if (isname(number, "board bulletin")) {
      send_to_char("You cannot reply to an entire board!\r\n", ch);
    } else if (!isdigit(*number) || (!(mnum = atoi(number)))) {
      send_to_char("You must reply to a message number.\r\n", ch);
      return;
    }
    Board_respond_message(GET_OBJ_VNUM(obj), ch, mnum);
  }
}
