/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

/*
 * This file attempts to concentrate most of the code which must be changed
 * in order for new classes to be added.  If you're adding a new class,
 * you should go through this entire file from beginning to end and add
 * the appropriate new special cases for your new class.
 */



#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "comm.h"
#include "db.h"
#include "utils.h"
#include "spells.h"
#include "interpreter.h"
#include "handler.h"
#include "skills.h"
#include "sound.h"

void set_saving_throws(struct char_data *ch);
void SetBookRandomSpells(struct obj_data *book, int circle, int num);
extern void outfit_random_mobs(struct char_data *ch);
extern void create_and_give_money(struct char_data *ch, int amount, int type);

#define NUM_DEITY  109
#define NUM_HUMAN  22
#define NUM_HFELF  27
#define NUM_ELF    14
#define NUM_DWARF  11
#define NUM_HALFLING 8
#define NUM_GNOME   8
#define NUM_HFORC   8

int race_gods[][40] =
{
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Human
  {2, 3, 4, 5, 8, 10, 11, 12, 13, 14,	//10
    17, 18, 21, 22, 24, 25, 45, 49, 60, 63,	//20
    15, 16, 0, 0, 0, 0, 0, 0, 0, 0,	//30
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0},	//40
//Half Elf
   {2, 3, 4, 5, 8, 10, 11, 12, 13, 14,
    17, 18, 21, 22, 24, 25, 45, 49, 60, 81,
    83, 84, 85, 88, 89, 63, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Elf
  {2, 81, 83, 8, 10, 85, 12, 84, 88, 89,
   45, 17, 18, 63, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Dwarf
  {74, 5, 25, 75, 76, 78, 37, 13, 14, 80,
   63, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Halfling
  {99, 100, 101, 102, 17, 37, 12, 10, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Gnome
  {97, 17, 95, 37, 12, 92, 25, 91, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
//Half Orc
  {103, 104, 105, 106, 107, 108, 14, 31, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};

const struct deity_data deity_list[] =
{
/*  name    Pantheon    major   minor , allowable alignments */
  {"Nobody", "NONE", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//0
   {"Beory", "Oerth Mother, Nature, Rain", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Boccob", "Magic, Arcane Knowledge", SPHERE_ALL | SPHERE_ASTRAL
   | SPHERE_DIVINATION | SPHERE_CHARM | SPHERE_COMBAT,
   SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_PROTECTION,
   {0, 1, 0, 0, 1, 1, 0, 0, 0}},
  {"Incabulos", "Plagues,Famines,Nightmares",
   SPHERE_ALL | SPHERE_ANIMAL | SPHERE_HEALING | SPHERE_SUMMONING | SPHERE_SUN,
   SPHERE_DIVINATION | SPHERE_NECROMANTIC | SPHERE_PROTECTION,
   {0, 0, 0, 0, 0, 0, 1, 1, 1}},
  {"Istus", "Fate, Destiny",
   SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_NECROMANTIC,
   SPHERE_CHARM | SPHERE_PROTECTION | SPHERE_CREATION,
   {1, 1, 1, 1, 1, 1, 1, 1, 1}},
  {"Nerull", "Death, Darkness, Murder",
   SPHERE_NECROMANTIC | SPHERE_ALL | SPHERE_ASTRAL | SPHERE_COMBAT | SPHERE_GUARDIAN,
   SPHERE_PROTECTION | SPHERE_SUN | SPHERE_CHARM,
   {0, 0, 0, 0, 0, 0, 1, 1, 1}},	//5
   {"Pelor", "Sun, Light, Strength, Healing", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Rao", "Peace, Reason, Serenity", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Celestian", "Stars, Space, Wanderers",
   SPHERE_ASTRAL | SPHERE_DIVINATION | SPHERE_ALL | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_CHARM | SPHERE_GUARDIAN | SPHERE_NECROMANTIC,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},
  {"Cyndor", "Time, Infinity, Continuity", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Ehlonna", "Forests, Woodlands, Flowers",
   SPHERE_PLANT | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_CREATION | SPHERE_ELEMENTAL,
   SPHERE_HEALING | SPHERE_SUN | SPHERE_WEATHER,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},	//10
   {"Erythnul", "Hate, Envy, Malice, Panic",
    SPHERE_HEALING | SPHERE_ALL | SPHERE_COMBAT | SPHERE_ELEMENTAL | SPHERE_SUMMONING,
    SPHERE_NECROMANTIC | SPHERE_PROTECTION,
    {0, 0, 0, 0, 0, 1, 1, 1, 1}},
  {"Fharlanghn", "Horizons, Distance, Travel",
   SPHERE_ALL | SPHERE_ASTRAL | SPHERE_DIVINATION | SPHERE_HEALING | SPHERE_ELEMENTAL,
   SPHERE_SUN | SPHERE_WEATHER | SPHERE_PROTECTION,
   {0, 1, 0, 1, 1, 1, 0, 1, 0}},
  {"Heironeous", "Chivalry, Justice, Honor, War",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_SUN | SPHERE_DIVINATION | SPHERE_NECROMANTIC,
   {1, 1, 0, 1, 0, 0, 0, 0, 0}},
  {"Hextor", "War, Discord, Massacres",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_HEALING | SPHERE_ELEMENTAL | SPHERE_NECROMANTIC,
   SPHERE_SUMMONING | SPHERE_ASTRAL | SPHERE_DIVINATION,
   {0, 0, 0, 1, 0, 0, 1, 1, 0}},
  {"Kord", "Athletics, Sports, Brawling",
   SPHERE_ALL | SPHERE_ELEMENTAL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_NECROMANTIC,
   SPHERE_DIVINATION | SPHERE_PROTECTION | SPHERE_SUMMONING,
   {0, 0, 1, 0, 1, 0, 0, 0, 1}},	//15
   {"Lendor", "Time, Tedium",
    SPHERE_ALL | SPHERE_ASTRAL | SPHERE_DIVINATION | SPHERE_GUARDIAN | SPHERE_HEALING,
    SPHERE_PROTECTION | SPHERE_SUMMONING | SPHERE_CREATION,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Obad-Hai", "Nature, Woodlands, Freedom",
   SPHERE_ANIMAL | SPHERE_ALL | SPHERE_ELEMENTAL | SPHERE_HEALING | SPHERE_PLANT |
   SPHERE_SUN | SPHERE_WEATHER,
   SPHERE_DIVINATION,
   {0, 0, 0, 0, 1, 0, 0, 0, 0}},
  {"Olidammara", "Music, Revels, Rogues, Wine",
   SPHERE_CHARM | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_CREATION | SPHERE_HEALING,
   SPHERE_DIVINATION | SPHERE_PLANT | SPHERE_PROTECTION,
   {0, 1, 1, 0, 1, 1, 0, 0, 0}},
  {"Pholtus", "Light, Resolution, Law", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Procan", "Oceans, Seas, Sailing", 0, 0},	//20
   {"Ralishaz", "Chance, Ill-luck, Misfortune",
    SPHERE_ALL | SPHERE_CHARM | SPHERE_CREATION | SPHERE_DIVINATION | SPHERE_HEALING,
    SPHERE_COMBAT | SPHERE_GUARDIAN | SPHERE_WEATHER,
    {0, 0, 1, 0, 0, 1, 0, 0, 1}},
  {"St-Cuthbert", "Common Sense, Honesty, Zeal",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_CHARM | SPHERE_NECROMANTIC | SPHERE_SUMMONING,
   {1, 0, 0, 1, 0, 0, 0, 0, 0}},
  {"Tharizdun", "Eternal Darkness, Decay", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Tritherion", "Individuality, Liberty, Retribution",
   SPHERE_SUMMONING | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_DIVINATION | SPHERE_HEALING,
   SPHERE_PROTECTION | SPHERE_ELEMENTAL | SPHERE_CREATION,
   {0, 0, 1, 0, 0, 1, 0, 0, 0}},
  {"Ulaa", "Hills, Mountains, Gemstones",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_COMBAT | SPHERE_DIVINATION,
   SPHERE_CREATION | SPHERE_HEALING | SPHERE_PROTECTION,
   {1, 1, 0, 1, 1, 0, 0, 0, 0}},	//25
   {"Wee Jas", "Magic, Death",
    SPHERE_NECROMANTIC | SPHERE_ASTRAL | SPHERE_CHARM | SPHERE_DIVINATION | SPHERE_ELEMENTAL,
    SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_SUMMONING,
    {1, 0, 0, 1, 0, 0, 1, 0, 0}},
  {"Zilchus", "Money, Influence, Business", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Akwamom", "Storms, Life at Sea", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Allitur", "Ethics, Propriety", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Atroa", "Spring, Renewal, East Wind", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//30
   {"Beltar", "Malice, Caves, Pits",
    SPHERE_NECROMANTIC | SPHERE_ALL | SPHERE_COMBAT | SPHERE_ELEMENTAL | SPHERE_GUARDIAN,
    SPHERE_DIVINATION | SPHERE_PROTECTION | SPHERE_HEALING,
    {0, 0, 0, 0, 0, 0, 0, 1, 1}},
  {"Berei", "Home, Family, Agriculture", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Bleredd", "Metal, Mines, Smiths", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Bralm", "Insects, Industriousness", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Dalt", "Portals and  Enclosures", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//35
   {"Delleb", "Reason, Intellect", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Fortubo", "Stone, Metals, Mountains",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_COMBAT | SPHERE_GUARDIAN | SPHERE_HEALING,
 SPHERE_DIVINATION | SPHERE_CHARM | SPHERE_PROTECTION | SPHERE_SUMMONING,
   {1, 1, 0, 1, 1, 0, 0, 0, 0}},
  {"Geshtai", "Lakes, Rivers, Wells, Streams", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Jascar", "Hills, Mountains, Slopes", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Joramy", "Fire, Volcanoes, Wrath", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//40
   {"Kurell", "Jealousy, Revenge, Thievery", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Landron", "Order, winds, seasons", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Lirr", "Prose, Poetry, Literacy, Art", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Llerg", "Beasts, Strength", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Lydia", "Music, Knowledge, Daylight",
   SPHERE_DIVINATION | SPHERE_ALL | SPHERE_ASTRAL | SPHERE_SUN | SPHERE_GUARDIAN | SPHERE_CREATION,
   SPHERE_HEALING | SPHERE_CHARM | SPHERE_WEATHER,
   {1, 1, 1, 0, 0, 0, 0, 0, 0}},	//45
   {"Myhriss", "Love, Romance, Beauty", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Norebo", "Luck, Gambling, Risks", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Om", "Patterns and totality of action", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Osprem", "Water Voyages",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_CHARM | SPHERE_HEALING |
   SPHERE_WEATHER,
   SPHERE_PROTECTION | SPHERE_SUN,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},
  {"Phaulkon", "Air, Winds, Clouds", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//50
   {"Phyton", "Beauty, Nature", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Pyremius", "Fire, Poison, Murder", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Raxivort", "Xvarts, Rats, Bats", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Sotillion", "Summer, Ease, South Wind", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Syrul", "Lies, Deceit, Treachery", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//55
   {"Telchur", "Winter, Cold, North Wind", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Velnius", "Sky, Weather", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Wenta", "Autumn, Harvest, West Wind", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Xan Yae", "Twilight, Shadows, Stealth", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Xerbo", "Sea, Sailing, Money, Business",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_DIVINATION | SPHERE_HEALING,
   SPHERE_GUARDIAN | SPHERE_WEATHER | SPHERE_PROTECTION,
   {0, 1, 0, 1, 1, 0, 0, 1, 0}},	//60
   {"Zodal", "Mercy, Hope, Benevolence", 0, 0,
    {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Chitza-Atlan", "The Dead, The Underworld", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Iuz", "Opression, Deceit, Pain",
   SPHERE_ALL | SPHERE_ASTRAL | SPHERE_COMBAT | SPHERE_ELEMENTAL | SPHERE_HEALING,
   SPHERE_SUN | SPHERE_GUARDIAN | SPHERE_NECROMANTIC,
   {0, 0, 0, 0, 0, 0, 1, 1, 1}},
  {"Mayaheine", "Protection, Justice, Valor", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},
  {"Merikka", "Farming, Agriculture", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//65
   {"Rudd", "Good Luck, Chance, Skill", 0, 0},
  {"Stern Alia", "Law of Pontylver", 0, 0},
  {"Tlazoteotl", "Oerth Mother", 0, 0},
  {"Vecna", "Dark Secrets, Black Magic", 0, 0},
  {"Wastri", "Bigotry, Amphibians", 0, 0},	//70
   {"Xilonen", "Corn, Birth", 0, 0},
  {"Zagyg", "Humor, Eccentricity, Arcane Lore", 0, 0},
  {"Zuoken", "Physical and Mental Mastery", 0, 0},
  {"Moradin", "Creator; smithing, crafts, war",
   SPHERE_ELEMENTAL | SPHERE_COMBAT | SPHERE_CREATION | SPHERE_GUARDIAN | SPHERE_HEALING,
   SPHERE_DIVINATION | SPHERE_PROTECTION | SPHERE_NECROMANTIC,
   {1, 0, 0, 1, 0, 0, 0, 0, 0}},
  {"Berronar Truesilver", "Safety, truth, home, healing",
   SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_NECROMANTIC | SPHERE_PROTECTION,
   SPHERE_DIVINATION | SPHERE_SUMMONING | SPHERE_CHARM,
   {1, 0, 0, 0, 0, 0, 0, 0, 0}},	//75
   {"Clangeddin Silverbeard", "Battle, war",
    SPHERE_COMBAT | SPHERE_ALL | SPHERE_CREATION | SPHERE_GUARDIAN | SPHERE_PROTECTION,
    SPHERE_ELEMENTAL | SPHERE_HEALING | SPHERE_SUMMONING,
    {1, 0, 0, 1, 0, 0, 1, 0, 0}},
  {"Dugmaren Brightmantle", "Knowledge, creativity", 0, 0},
  {"Dumathoin", "Mining, exploration",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_CREATION | SPHERE_DIVINATION | SPHERE_GUARDIAN,
   SPHERE_COMBAT | SPHERE_HEALING | SPHERE_PROTECTION,
   {0, 1, 0, 1, 1, 1, 0, 1, 0}},
  {"Muamman Duathal", "Expatriates, urban dwarves, travellers, exiles", 0, 0},
  {"Vergadain", "Wealth, luck",
   SPHERE_DIVINATION | SPHERE_ALL | SPHERE_CHARM | SPHERE_CREATION | SPHERE_PROTECTION,
   SPHERE_GUARDIAN | SPHERE_HEALING,
   {0, 1, 0, 1, 1, 1, 0, 0, 0}},	//80
   {"Corellon Larethian", "Creator; magic, arts and crafts, war",
    SPHERE_ALL | SPHERE_CHARM | SPHERE_COMBAT | SPHERE_GUARDIAN | SPHERE_PROTECTION,
    SPHERE_ANIMAL | SPHERE_DIVINATION | SPHERE_NECROMANTIC,
    {1, 1, 1, 0, 0, 1, 0, 0, 0}},
  {"Sehanine the Moonbow", "Mysticism, dreams, far journeys, death and transcendence", 0, 0},
  {"Aerdie Faeyna", "Air, weather, birds",
   SPHERE_ELEMENTAL | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_HEALING | SPHERE_WEATHER,
   SPHERE_CREATION | SPHERE_DIVINATION | SPHERE_GUARDIAN,
   {0, 1, 1, 0, 1, 1, 0, 0, 0}},
  {"Hanali Celanil", "Romance, love, beauty",
   SPHERE_CHARM | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_DIVINATION | SPHERE_HEALING,
   SPHERE_PLANT | SPHERE_PROTECTION | SPHERE_NECROMANTIC,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},
  {"Erevan Ilesere", "Trickster, mischief, rogues",
   SPHERE_ALL | SPHERE_ANIMAL | SPHERE_CHARM | SPHERE_CREATION | SPHERE_HEALING,
   SPHERE_ELEMENTAL | SPHERE_DIVINATION | SPHERE_PROTECTION,
   {0, 1, 1, 0, 1, 1, 0, 1, 1}},	//85
   {"Fenmarel Mestarine", "Uncivilized elves, drow haters", 0, 0},
  {"Rillifane", "Freedom, Wood Elf Creator, Voadkin", 0, 0},
  {"Labelas Enoreth", "Time, longevity",
   SPHERE_ALL | SPHERE_ASTRAL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_DIVINATION,
   SPHERE_PROTECTION | SPHERE_PLANT | SPHERE_ANIMAL,
   {0, 1, 1, 0, 0, 0, 0, 0, 0}},
  {"Solonor Thelandira", "Archery, hunting",
   SPHERE_ANIMAL | SPHERE_ALL | SPHERE_HEALING | SPHERE_PLANT | SPHERE_WEATHER,
   SPHERE_CHARM | SPHERE_DIVINATION | SPHERE_PROTECTION,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},
  {"Lolth", "The Demon Spidress", 0, 0,
   {0, 0, 0, 0, 0, 0, 0, 0, 0}},	//90
   {"Garl Glittergold", "Creator; protection, humor, gems, smithing",
    SPHERE_ALL | SPHERE_ASTRAL | SPHERE_CHARM | SPHERE_ELEMENTAL | SPHERE_HEALING |
    SPHERE_NECROMANTIC | SPHERE_PROTECTION,
    SPHERE_ANIMAL | SPHERE_DIVINATION | SPHERE_GUARDIAN,
    {1, 1, 0, 0, 0, 0, 0, 0, 0}},
  {"Baervan Wildwanderer", "Forests, nature, travel",
 SPHERE_ALL | SPHERE_ANIMAL | SPHERE_HEALING | SPHERE_PLANT | SPHERE_SUN,
   SPHERE_CREATION | SPHERE_ELEMENTAL | SPHERE_WEATHER,
   {1, 1, 1, 1, 1, 1, 0, 0, 0}},
  {"Baravar Cloakshadow", "Illusions, protection, deception, hatred of goblinoids", 0, 0},
  {"Flandal Steelskin", "Mining, smithing, fitness", 0, 0},
  {"Gaerdal Ironhand", "Protection, vigilance, combat",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_DIVINATION | SPHERE_CREATION | SPHERE_ASTRAL,
   {1, 0, 0, 1, 0, 0, 0, 0, 0}},	//95
   {"Segojan Earthcaller", "Earth, nature", 0, 0},
  {"Urdlen", "Evil, fear, madness", 0, 0,
   {0, 0, 0, 0, 0, 0, 1, 1, 1}},
  {"Yondalla", "Creator; protection, fertility", 0, 0},
  {"Arvoreen the Defender", "Protection, vigilance, war",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_ELEMENTAL | SPHERE_NECROMANTIC,
   {1, 1, 1, 1, 1, 0, 0, 0, 0}},
  {"Brandobaris", "Stealth, thieves, adventure",
   SPHERE_ALL | SPHERE_CHARM | SPHERE_PROTECTION | SPHERE_GUARDIAN | SPHERE_ANIMAL,
   SPHERE_CREATION | SPHERE_DIVINATION | SPHERE_HEALING,
   {0, 1, 0, 0, 1, 1, 0, 0, 0}},	//100
   {"Cyrrollalee", "Friendship, trust, home",
    SPHERE_ALL | SPHERE_CHARM | SPHERE_DIVINATION | SPHERE_GUARDIAN | SPHERE_HEALING
    | SPHERE_PROTECTION,
    SPHERE_ANIMAL | SPHERE_PLANT,
    {1, 1, 1, 1, 0, 0, 0, 0, 0}},
  {"Sheela Peryoryl", "Nature, agriculture, weather",
   SPHERE_PLANT | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_ELEMENTAL | SPHERE_HEALING | SPHERE_SUN
   | SPHERE_WEATHER,
   SPHERE_CHARM,
   {0, 0, 0, 0, 1, 0, 0, 0, 0}},
  {"Gruumsh One-Eye", "Orcs, Blood, Conquest, Law, Revenge",
   SPHERE_ALL | SPHERE_ELEMENTAL | SPHERE_HEALING | SPHERE_SUMMONING,
   SPHERE_DIVINATION | SPHERE_NECROMANTIC | SPHERE_PROTECTION,
   {0, 0, 0, 0, 0, 0, 1, 1, 0}},
  {"Baghtru the Mighty", "Strength, Might, Rage",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_ELEMENTAL | SPHERE_HEALING | SPHERE_PROTECTION,
   SPHERE_SUMMONING | SPHERE_GUARDIAN | SPHERE_SUN,
   {0, 0, 0, 0, 0, 0, 1, 1, 1,}},
  {"Ilneval, One-Eye's Hand", "Warriors, Swordsmanship, Obedience, Leadership",
   SPHERE_COMBAT | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_PROTECTION | SPHERE_SUMMONING,
   SPHERE_SUN | SPHERE_CHARM | SPHERE_HEALING,
   {0, 0, 0, 0, 0, 0, 1, 1, 1}},	//105
   {"Luthic, The Cave Mother", "Caves, Fertility, Servitude, Medicine",
    SPHERE_HEALING | SPHERE_ALL | SPHERE_GUARDIAN | SPHERE_NECROMANTIC | SPHERE_PROTECTION,
    SPHERE_SUMMONING | SPHERE_SUN | SPHERE_CHARM,
    {0, 0, 0, 1, 1, 1, 1, 1, 1}},
  {"Shargas, The Night Lord", "Night, Assassins, Thievery, Subterfuge",
   SPHERE_SUN | SPHERE_ALL | SPHERE_COMBAT | SPHERE_DIVINATION | SPHERE_GUARDIAN,
   SPHERE_PROTECTION | SPHERE_SUMMONING | SPHERE_HEALING,
   {0, 0, 0, 0, 0, 0, 1, 1, 1,}},
  {"Yutrus White Hand", "Death, Undead, Disease, Silence",
   SPHERE_NECROMANTIC | SPHERE_ALL | SPHERE_ANIMAL | SPHERE_HEALING | SPHERE_SUN
   | SPHERE_COMBAT,
   SPHERE_CHARM | SPHERE_PLANT | SPHERE_PROTECTION,
   {0, 0, 0, 0, 0, 0, 0, 1, 0}}
};

/* Names first */

const char *class_abbrevs[NUM_CLASSES + 1] =
{
  "Cl ",
  "Dr ",
  "Fi ",
  "Pa ",
  "Ra ",
  "Mu ",
  "Il ",
  "Th ",
  "Ba ",
  "CM ",
  "FM ",
  "FT ",
  "MT ",
  "FMT",
  "CR ",
  "CF ",
  "IT ",
  "CFM",
  "CT ",
  "FI ",
  "CI ",
  "\n"
};

char *class_types[NUM_CLASSES + 1] =
{
  "Cleric",
  "Druid",
  "Fighter",
  "Paladin",
  "Ranger",
  "Magic-User",
  "Illusionist",
  "Thief",
  "Bard",
  "Cleric/Magic-User",
  "Fighter/Magic-User",
  "Fighter/Thief",
  "Magic-User/Thief",
  "Fighter/Magic-User/Thief",
  "Cleric/Ranger",
  "Cleric/Fighter",
  "Illusionist/Thief",
  "Cleric/Fighter/Magic-User",
  "Cleric/Thief",
  "Fighter/Illusionist",
  "Cleric/Illusionist",
  "\n"
};


char *class_display[NUM_CLASSES] =
{
  "a) Cleric\r\n",
  "b) Druid\r\n",
  "c) Fighter\r\n",
  "d) Paladin\r\n",
  "e) Ranger\r\n",
  "f) Magic-User\r\n",
  "g) Illusionist\r\n",
  "h) Thief\r\n",
  "i) Bard\r\n",
  "j) Cleric/Magic-User\r\n",
  "k) Fighter/Magic-User\r\n",
  "l) Fighter/Thief\r\n",
  "m) Magic-User/Thief\r\n",
  "n) Fighter/Magic-User/Thief\r\n",
  "o) Cleric/Ranger\r\n",
  "p) Cleric/Fighter\r\n",
  "q) Illusionist/Thief\r\n",
  "r) Cleric/Fighter/Magic-User\r\n",
  "s) Cleric/Thief\r\n",
  "t) Fighter/Illusionist\r\n",
  "u) Cleric/Illusionist\r\n",
};
/*IH 05-Oct-98 FIX: should be NUM_RACES not NUM_PC_RACES */
/* It should?!? Why? faf 9/25/99 */
int class_ok_race[NUM_PC_RACES][NUM_CLASSES] =
{
  /* Cl Dr Fi Pa Ra Mu Il Th Ba CM FM FT MT FMT CR CF IT CFM CT FI CI */
  /* Human */
  {Y, Y, Y, Y, Y, Y, Y, Y, Y, N, N, N, N, N, N, N, N, N, N, N, N},
  /* H-Elf */
  {Y, Y, Y, N, Y, Y, N, Y, Y, Y, Y, Y, Y, Y, Y, Y, N, Y, N, N, N},
  /* Elf   */
  {Y, N, Y, N, N, Y, N, Y, N, N, Y, Y, Y, Y, N, N, N, N, N, N, N},
  /* Dwarf */
  {Y, N, Y, N, N, N, N, Y, N, N, N, Y, N, N, N, Y, N, N, N, N, N},
  /* Halfl */
  {Y, N, Y, N, N, N, N, Y, N, N, N, Y, N, N, N, N, N, N, N, N, N},
  /* Gnome */
  {Y, N, Y, N, N, N, Y, Y, N, N, N, Y, N, N, N, N, Y, N, Y, Y, Y},
  /* H-Orc */
  {Y, N, Y, N, N, N, N, Y, N, N, N, N, N, N, N, N, N, N, N, N, N},
  /* Barb  */
  {Y, N, Y, N, Y, Y, N, Y, N, N, Y, Y, Y, N, Y, Y, N, N, Y, N, N},
  /* Drow  */
  {Y, N, Y, N, N, Y, N, Y, N, N, Y, Y, Y, Y, N, N, N, N, N, N, N},
  /* Duerg */
  {Y, N, Y, N, N, N, N, Y, N, N, N, Y, N, N, N, Y, N, N, N, N, N},
  /* Ogre  */
  {N, N, Y, N, N, Y, N, Y, N, N, N, Y, N, N, N, N, N, N, N, N, N},
  /* Troll */
  {N, N, Y, N, N, N, N, Y, N, N, N, Y, N, N, N, N, N, N, N, N, N},
  /* Illit */
  {N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N},
  /* Orc   */
  {Y, N, Y, N, N, N, N, Y, N, N, N, N, N, N, N, N, N, N, N, N, N},
  /* Gobli */
  {Y, N, Y, N, N, N, N, Y, N, N, N, N, N, N, N, N, N, N, N, N, N}
};

char *alignment_display[NUM_ALIGNMENT] =
{
  "0) None\r\n",
  "1) Lawful/Good\r\n",
  "2) Neutral/Good\r\n",
  "3) Chaotic/Good\r\n",
  "4) Lawful/Neutral\r\n",
  "5) True Neutral\r\n",
  "6) Chaotic/Neutral\r\n",
  "7) Lawful/Evil\r\n",
  "8) Neutral/Evil\r\n",
  "9) Chaotic/Evil\r\n",
};

int alignment_ok_class[NUM_CLASSES][NUM_ALIGNMENT] =
{

  /*    N LG NG CG LN NN CN LE NE CE */
  /* Cl */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* Dr */
  {N, N, N, N, N, Y, N, N, N, N},
  /* Fi */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* Pa */
  {N, Y, N, N, N, N, N, N, N, N},
  /* Ra */
  {N, Y, Y, Y, N, N, N, N, N, N},
  /* Mu */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* Il */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* Th */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* Ba */
  {N, Y, N, N, Y, N, N, Y, N, N},
  /* CM */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* FM */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* FT */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* MT */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* FMT */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* CR */
  {N, Y, Y, Y, N, N, N, N, N, N},
  /* CF */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* IT */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* CFM */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* CT */
  {N, N, Y, Y, N, Y, Y, N, Y, Y},
  /* FI */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y},
  /* CI */
  {N, Y, Y, Y, Y, Y, Y, Y, Y, Y}
};

/*
 * The code to interpret a class letter -- used in interpreter.c when a
 * new character is selecting a class and by 'set class' in act.wizard.c.
 */

int parse_class(struct char_data *ch, char arg)
{
  int class = CLASS_UNDEFINED;

  switch (LOWER(arg)) {
  case 'a':
    class = CLASS_CLERIC;
    break;
  case 'b':
    class = CLASS_DRUID;
    break;
  case 'c':
    class = CLASS_FIGHTER;
    break;
  case 'd':
    class = CLASS_PALADIN;
    break;
  case 'e':
    class = CLASS_RANGER;
    break;
  case 'f':
    class = CLASS_MAGIC_USER;
    break;
  case 'g':
    class = CLASS_ILLUSIONIST;
    break;
  case 'h':
    class = CLASS_THIEF;
    break;
  case 'i':
    class = CLASS_BARD;
    break;
  case 'j':
    class = CLASS_MULTI_C_M;
    break;
  case 'k':
    class = CLASS_MULTI_F_M;
    break;
  case 'l':
    class = CLASS_MULTI_F_T;
    break;
  case 'm':
    class = CLASS_MULTI_M_T;
    break;
  case 'n':
    class = CLASS_MULTI_F_M_T;
    break;
  case 'o':
    class = CLASS_MULTI_C_R;
    break;
  case 'p':
    class = CLASS_MULTI_C_F;
    break;
  case 'q':
    class = CLASS_MULTI_I_T;
    break;
  case 'r':
    class = CLASS_MULTI_C_F_M;
    break;
  case 's':
    class = CLASS_MULTI_C_T;
    break;
  case 't':
    class = CLASS_MULTI_F_I;
    break;
  case 'u':
    class = CLASS_MULTI_C_I;
    break;
  default:
    class = CLASS_UNDEFINED;
    break;
  }

  return (class);
}


int parse_alignment(struct char_data *ch, char arg)
{
  int alignment = ALIGN_NONE;

  switch (arg) {
  case '1':
    alignment = ALIGN_LAWFUL_GOOD;
    break;
  case '2':
    alignment = ALIGN_NEUTRAL_GOOD;
    break;
  case '3':
    alignment = ALIGN_CHAOTIC_GOOD;
    break;
  case '4':
    alignment = ALIGN_LAWFUL_NEUTRAL;
    break;
  case '5':
    alignment = ALIGN_NEUTRAL_NEUTRAL;
    break;
  case '6':
    alignment = ALIGN_CHAOTIC_NEUTRAL;
    break;
  case '7':
    alignment = ALIGN_LAWFUL_EVIL;
    break;
  case '8':
    alignment = ALIGN_NEUTRAL_EVIL;
    break;
  case '9':
    alignment = ALIGN_CHAOTIC_EVIL;
    break;
  default:
    alignment = ALIGN_NONE;
    break;
  }

  if (alignment >= 1 && alignment <= NUM_ALIGNMENT)
    if (!alignment_ok_class[(int) GET_CLASS(ch)][alignment])
      alignment = ALIGN_NONE;

  return (alignment);
}


/*
 * bitvectors (i.e., powers of two) for each class, mainly for use in
 * do_who and do_users.  Add new classes at the end so that all classes
 * use sequential powers of two (1 << 0, 1 << 1, 1 << 2, 1 << 3, 1 << 4,
 * 1 << 5, etc.
 */

long find_class_bitvector(char arg)
{
  arg = LOWER(arg);

  switch (arg) {
  case 'a':
    return (1 << 0);
    break;
  case 'b':
    return (1 << 1);
    break;
  case 'c':
    return (1 << 2);
    break;
  case 'd':
    return (1 << 3);
    break;
  case 'e':
    return (1 << 4);
    break;
  case 'f':
    return (1 << 5);
    break;
  case 'g':
    return (1 << 6);
    break;
  case 'h':
    return (1 << 7);
    break;
  case 'i':
    return (1 << 8);
    break;
  case 'j':
    return (1 << 9);
    break;
  case 'k':
    return (1 << 10);
    break;
  case 'l':
    return (1 << 11);
    break;
  case 'm':
    return (1 << 12);
    break;
  case 'n':
    return (1 << 13);
    break;
  case 'o':
    return (1 << 14);
    break;
  case 'p':
    return (1 << 15);
    break;
  case 'q':
    return (1 << 16);
    break;
  case 'r':
    return (1 << 17);
    break;
  case 's':
    return (1 << 18);
    break;
  case 't':
    return (1 << 19);
    break;
  case 'u':
    return (1 << 20);
    break;
  default:
    return 0;
    break;
  }
}


/* THAC0 for classes and levels.  (To Hit Armor Class 0) */

/* [class], [level] (all) */
const int thaco[NUM_CLASSES][LVL_IMPL + 1] =
{

/* CLERIC */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 18, 18, 18, 16, 16, 16, 14, 14, 14, 12, 12, 12, 10, 10,
 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 1, 1, 1, 1},
  /* 20             25             30                               */
/* DRUID */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 18, 18, 18, 16, 16, 16, 14, 14, 14, 12, 12, 12, 10, 10,
 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 1, 1, 1, 1},
  /* 20             25             30                               */
/* FIGHTER */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* PALADIN */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* RANGER */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* MAGE */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 19, 19, 19, 18, 18, 18, 17, 17, 17, 16, 16, 16, 15, 15,
 15, 14, 14, 14, 13, 13, 13, 12, 12, 12, 11, 11, 11, 10, 10, 10, 9},
  /* 20                  25                  30             */
/* ILLUSIONIST */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 19, 19, 19, 18, 18, 18, 17, 17, 17, 16, 16, 16, 15, 15,
 15, 14, 14, 14, 13, 13, 13, 12, 12, 12, 11, 11, 11, 10, 10, 10, 9},
  /* 20                  25                  30             */
/* THIEF */
  /* 0                   5                  10                  15          */
{100, 20, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15, 14, 13, 13, 12, 12,
 11, 11, 10, 10, 9, 9, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3},
  /* 20              25             30                              */
/* BARD */
  /* 0                   5                  10                  15          */
{100, 20, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15, 14, 13, 13, 12, 12,
 11, 11, 10, 10, 9, 9, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3},
  /* 20              25             30                              */
/* CLERIC/MAGIC-USER */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 18, 18, 18, 16, 16, 16, 14, 14, 14, 12, 12, 12, 10, 10,
 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 1, 1, 1, 1},
  /* 20             25             30                               */
/* FIGHTER/MAGIC-USER */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* FIGHTER/THIEF */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* MAGIC-USER/THIEF */
  /* 0                   5                  10                  15          */
{100, 20, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15, 14, 13, 13, 12, 12,
 11, 11, 10, 10, 9, 9, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3},
  /* 20              25             30                              */
/* FIGHTER/MAGIC-USER/THIEF */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* CLERIC/RANGER */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* CLERIC/FIGHTER */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* ILLUSIONIST/THIEF */
  /* 0                   5                  10                  15          */
{100, 20, 20, 19, 19, 18, 18, 17, 17, 16, 16, 15, 15, 14, 13, 13, 12, 12,
 11, 11, 10, 10, 9, 9, 8, 8, 7, 7, 6, 6, 5, 5, 4, 4, 3},
  /* 20              25             30                              */
/* CLERIC/FIGHTER/MAGE */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* CLERIC/THIEF */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 18, 18, 18, 16, 16, 16, 14, 14, 14, 12, 12, 12, 10, 10,
 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 1, 1, 1, 1},
  /* 20             25             30                               */
/* FIGHTER/ILLUSIONIST */
  /* 0                   5                  10              15      */
  {100, 20, 19, 18, 17, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3,
   2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
  /* 20             25             30                               */
/* CLERIC/ILLUSIONIST */
  /* 0                   5                  10                  15          */
{100, 20, 20, 20, 18, 18, 18, 16, 16, 16, 14, 14, 14, 12, 12, 12, 10, 10,
 10, 8, 8, 8, 6, 6, 6, 4, 4, 4, 2, 2, 2, 1, 1, 1, 1}
  /* 20             25             30                               */
};




/* Some initializations for characters, including initial skills */
void do_start(struct char_data *ch)
{
  int i = 0;
  struct obj_data *obj;
  void advance_level(struct char_data *ch);

  GET_LEVEL(ch) = 1;
  GET_HIGH_LEVEL(ch) = 1;
  GET_TOT_LEVEL(ch) = 1;
  GET_EXP(ch) = 1;
  GET_AC(ch) = 10;

  set_title(ch, NULL);
  ch->points.max_hit = 0;

  for (i = 0; i < 25; i++)
    ch->player_specials->saved.known_rooms[i] = 0;

  if (IS_MAGE(ch)) {
    GET_MAGE_SCHOOL(ch) = 0;
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ALTERATION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CONJURATION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_SUMMONING);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ENCHANTMENT);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CHARM);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ILLUSION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_PHANTASM);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_LESSER_DIVIN);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_GREATER_DIVIN);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
    SET_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
  }
  switch (GET_CLASS(ch)) {

  case CLASS_CLERIC:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_DRUID:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    GET_PRIEST_SPHERE_MAJOR(ch) = 0;
    GET_PRIEST_SPHERE_MINOR(ch) = 0;
    break;
  case CLASS_FIGHTER:
    SET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD, 1);
    SET_SKILL(ch, SKILL_STYLE_ONE_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_MISSILE, 1);
    SET_SKILL(ch, SKILL_KICK, 10);
    break;
  case CLASS_PALADIN:
    SET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD, 1);
    SET_SKILL(ch, SKILL_STYLE_ONE_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_MISSILE, 1);
    break;
  case CLASS_RANGER:
    SET_SKILL(ch, SKILL_STYLE_WEAPON_SHIELD, 1);
    SET_SKILL(ch, SKILL_STYLE_ONE_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_HANDED_WEAPON, 1);
    SET_SKILL(ch, SKILL_STYLE_TWO_WEAPON, 2);
    SET_SKILL(ch, SKILL_STYLE_MISSILE, 1);
    break;
  case CLASS_MAGIC_USER:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_ILLUSIONIST:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
    break;
  case CLASS_THIEF:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    break;
  case CLASS_BARD:
    GET_THIEF_SKILL(ch) = 20;
    SET_SKILL(ch, SKILL_STEAL, 10);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 20);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 5);
    break;
  case CLASS_MULTI_C_M:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_MULTI_F_M:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_MULTI_F_T:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    break;
  case CLASS_MULTI_M_T:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    break;
  case CLASS_MULTI_F_M_T:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    break;
  case CLASS_MULTI_C_R:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_MULTI_C_F:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_MULTI_I_T:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);

    break;
  case CLASS_MULTI_C_F_M:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    break;
  case CLASS_MULTI_C_T:
    GET_THIEF_SKILL(ch) = 60;
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    SET_SKILL(ch, SKILL_SNEAK, 10);
    SET_SKILL(ch, SKILL_HIDE, 5);
    SET_SKILL(ch, SKILL_STEAL, 15);
    SET_SKILL(ch, SKILL_BACKSTAB, 100);
    SET_SKILL(ch, SKILL_PICK_LOCK, 10);
    SET_SKILL(ch, SKILL_LISTEN, 15);
    SET_SKILL(ch, SKILL_FIND_TRAP, 5);
    SET_SKILL(ch, SKILL_CLIMB_WALL, 60);
    SET_SKILL(ch, SKILL_READ_LANGUAGE, 0);
    break;
  case CLASS_MULTI_F_I:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
    break;
  case CLASS_MULTI_C_I:
    SET_SKILL(ch, SKILL_READING_WRITING, 1);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
    REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
    break;
  }

  switch (GET_RACE(ch)) {
  case RACE_HALFLING:
  case RACE_GNOME:
  case RACE_DWARF:
  case RACE_P_DUERGAR:
  case RACE_P_GOBLIN:
  case RACE_P_ORC:
    GET_SIZE(ch) = SIZE_SMALL;
    break;
  case RACE_HUMAN:
  case RACE_HALFORC:
  case RACE_P_ILLITHID:
  case RACE_BARBARIAN:
    GET_SIZE(ch) = SIZE_MEDIUM;
    break;
  case RACE_ELF:
  case RACE_HALFELF:
  case RACE_P_DROW:
    GET_SIZE(ch) = SIZE_SM;
    break;
  case RACE_OGRE:
  case RACE_TROLL:
    GET_SIZE(ch) = SIZE_LARGE;
    break;
  default:
    GET_SIZE(ch) = SIZE_MEDIUM;
    break;
  }

  advance_level(ch);

  GET_HIT(ch) = GET_MAX_HIT(ch);
  GET_MAX_VIT(ch) = 5 * GET_CON(ch);
  GET_VIT(ch) = GET_MAX_VIT(ch);

  GET_COND(ch, THIRST) = 24;
  GET_COND(ch, FULL) = 24;
  GET_COND(ch, DRUNK) = 0;

  ch->player.time.played = 0;
  ch->player.time.logon = time(0);

  /* start goodies, lets try cash for now */
  outfit_random_mobs(ch);
  create_and_give_money(ch, number(5, 15), MONEY_GOLD);
  obj = read_object(1101, VIRTUAL);	/* small bag */
  if (obj)
    obj_to_char(obj, ch);

  /* spellbooks? */
  if (GET_CLASS(ch) == CLASS_MAGIC_USER || GET_CLASS(ch) == CLASS_ILLUSIONIST
  || GET_CLASS(ch) == CLASS_MULTI_C_M || GET_CLASS(ch) == CLASS_MULTI_M_T
  || GET_CLASS(ch) == CLASS_MULTI_F_M || GET_CLASS(ch) == CLASS_MULTI_F_I
      || GET_CLASS(ch) == CLASS_MULTI_F_M_T || GET_CLASS(ch) == CLASS_MULTI_C_F_M) {
    obj = read_object(1402, VIRTUAL);	/* traveling spellbook */
    if (obj) {
      SetBookRandomSpells(obj, 1, number(1, 3));
      obj_to_char(obj, ch);
    }
  }
}



/*
 * This function controls the change to maxhp for
 * each class every time they gain a level.
 */
void proficiency_set(struct char_data *ch);
void set_saving_throws(struct char_data *ch);
void thief_skill_set(struct char_data *ch);
void fighter_skill_set(struct char_data *ch);

void advance_level(struct char_data *ch)
{
  int add_hp = 0, i;

  extern struct con_app_type con_app[];

  switch (GET_CLASS(ch)) {

  case CLASS_MAGIC_USER:
  case CLASS_ILLUSIONIST:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 4;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 10) {
      add_hp += number(2, 4);
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else {
      add_hp += 1;
    }
    break;
  case CLASS_RANGER:
  case CLASS_FIGHTER:
  case CLASS_PALADIN:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 10;
      add_hp += con_app[GET_CON(ch)].hitp;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += number(3, 10);
      add_hp += con_app[GET_CON(ch)].hitp;
    } else {
      add_hp += 3;
    }
    break;
  case CLASS_CLERIC:
  case CLASS_DRUID:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 8;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += number(3, 8);
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_THIEF:
  case CLASS_BARD:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 6;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 10) {
      add_hp += number(2, 6);
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_C_M:
  case CLASS_MULTI_C_I:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 6;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 8) + number(2, 4)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (2 + number(2, 4)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_C_F:
  case CLASS_MULTI_C_R:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 9;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 8) + number(3, 10)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else {
      add_hp += 3;
    }
    break;
  case CLASS_MULTI_I_T:
  case CLASS_MULTI_M_T:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 5;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 10) {
      add_hp += (number(2, 4) + number(2, 6)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_F_T:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 8;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 10) + number(2, 6)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (3 + number(2, 6)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
    } else {
      add_hp += 3;
    }
    break;
  case CLASS_MULTI_F_M:
  case CLASS_MULTI_F_I:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 7;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 10) + number(2, 4)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
      add_hp += con_app[GET_CON(ch)].hitp / 2;
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (3 + number(2, 4)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_F_M_T:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 7;
      add_hp += 2 * MIN(con_app[GET_CON(ch)].hitp, 2) / 3;
      add_hp += con_app[GET_CON(ch)].hitp / 3;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 10) + number(2, 6) + number(2, 4)) / 3;
      add_hp += 2 * MIN(con_app[GET_CON(ch)].hitp, 2) / 3;
      add_hp += con_app[GET_CON(ch)].hitp / 3;
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (3 + number(2, 6) + number(2, 4)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_C_F_M:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 8;
      add_hp += 2 * MIN(con_app[GET_CON(ch)].hitp, 2) / 3;
      add_hp += con_app[GET_CON(ch)].hitp / 3;
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 10) + number(3, 8) + number(2, 4)) / 3;
      add_hp += 2 * MIN(con_app[GET_CON(ch)].hitp, 2) / 3;
      add_hp += con_app[GET_CON(ch)].hitp / 3;
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (3 + 2 + number(2, 4)) / 2;
      add_hp += 2 * MIN(con_app[GET_CON(ch)].hitp, 2) / 3;
    } else {
      add_hp += 2;
    }
    break;
  case CLASS_MULTI_C_T:
    if (GET_LEVEL(ch) == 1) {
      add_hp += 7;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) <= 9) {
      add_hp += (number(3, 8) + number(2, 6)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2);
    } else if (GET_LEVEL(ch) == 10) {
      add_hp += (2 + number(2, 6)) / 2;
      add_hp += MIN(con_app[GET_CON(ch)].hitp, 2) / 2;
    } else {
      add_hp += 2;
    }
    break;

  }

  ch->points.max_hit += MAX(1, add_hp);

  GET_HIT(ch) = GET_MAX_HIT(ch);

  if (GET_LEVEL(ch) >= LVL_IMMORT) {
    for (i = 0; i < 3; i++)
      GET_COND(ch, i) = -1;
    SET_BIT(PRF_FLAGS(ch), PRF_HOLYLIGHT);

    if (GET_LEVEL(ch) == LVL_IMPL) {
      ch->real_abils.str_add = 0;
      ch->real_abils.str = 25;
      ch->real_abils.dex = 25;
      ch->real_abils.con = 25;
      ch->real_abils.wis = 25;
      ch->real_abils.intel = 25;
      ch->real_abils.cha = 25;
    }
    if (GET_LEVEL(ch) == LVL_GRGOD) {
      ch->real_abils.str_add = 0;
      ch->real_abils.str = 24;
      ch->real_abils.dex = 24;
      ch->real_abils.con = 24;
      ch->real_abils.wis = 24;
      ch->real_abils.intel = 24;
      ch->real_abils.cha = 24;
    }
    if (GET_LEVEL(ch) == LVL_GOD) {
      ch->real_abils.str_add = 0;
      ch->real_abils.str = 23;
      ch->real_abils.dex = 23;
      ch->real_abils.con = 23;
      ch->real_abils.wis = 23;
      ch->real_abils.intel = 23;
      ch->real_abils.cha = 23;
    }
    if (GET_LEVEL(ch) == LVL_IMMORT) {
      ch->real_abils.str_add = 0;
      ch->real_abils.str = 20;
      ch->real_abils.dex = 20;
      ch->real_abils.con = 20;
      ch->real_abils.wis = 20;
      ch->real_abils.intel = 20;
      ch->real_abils.cha = 20;
    }
    ch->aff_abils = ch->real_abils;
  }
  /* activate page_string by default */
  SET_BIT(PRF_FLAGS(ch), PRF_PAGING);

  /* logs on */
  if (GET_LEVEL(ch) > LVL_IMPL)
    SET_BIT(PRF_FLAGS(ch), PRF_LOG1);

  set_saving_throws(ch);
  proficiency_set(ch);
  thief_skill_set(ch);
  GET_HIGH_LEVEL(ch) = GET_LEVEL(ch);

  save_char(ch, NOWHERE);

  sprintf(buf, "%s advanced to level %d", GET_NAME(ch), GET_LEVEL(ch));
  mudlog(buf, BRF, MAX(LVL_IMMORT, GET_INVIS_LEV(ch)), TRUE);
}


/*
 * This simply calculates the backstab multiplier based on a character's
 * level.  This used to be an array, but was changed to be a function so
 * that it would be easier to add more levels to your MUD.  This doesn't
 * really create a big performance hit because it's not used very often.
 */
int backstab_mult(int level)
{
  if (level <= 0)
    return 1;
  else if (level <= 4)
    return 2;
  else if (level <= 8)
    return 3;
  else if (level <= 12)
    return 4;
  else if (level < LVL_IMMORT)
    return 5;
  else
    return 20;
}


/*
 * invalid_class is used by handler.c to determine if a piece of equipment is
 * usable by a particular class, based on the ITEM_ANTI_{class} bitvectors.
 */
int invalid_class(struct char_data *ch, struct obj_data *obj)
{

  if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_ALL)
    return 0;

  switch (GET_CLASS(ch)) {
  case CLASS_CLERIC:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC)
      return 0;
    break;
  case CLASS_DRUID:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_DRUID)
      return 0;
    break;
  case CLASS_FIGHTER:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER)
      return 0;
    break;
  case CLASS_PALADIN:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_PALADIN)
      return 0;
    break;
  case CLASS_RANGER:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_RANGER)
      return 0;
    break;
  case CLASS_MAGIC_USER:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER)
      return 0;
    break;
  case CLASS_ILLUSIONIST:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_ILLUSIONIST)
      return 0;
    break;
  case CLASS_THIEF:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF)
      return 0;
    break;
  case CLASS_BARD:
    if (GET_OBJ_ANTI(obj) == ITEM_ALLOW_BARD)
      return 0;
    break;
  case CLASS_MULTI_C_M:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER))
      return 0;
    break;
  case CLASS_MULTI_F_M:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER))
      return 0;
    break;
  case CLASS_MULTI_F_T:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF))
      return 0;
    break;
  case CLASS_MULTI_M_T:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER))
      return 0;
    break;
  case CLASS_MULTI_F_M_T:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF))
      return 0;
    break;
  case CLASS_MULTI_C_R:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_RANGER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC))
      return 0;
    break;
  case CLASS_MULTI_C_F:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC))
      return 0;
    break;
  case CLASS_MULTI_I_T:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_ILLUSIONIST) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF))
      return 0;
    break;
  case CLASS_MULTI_C_F_M:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_MAGIC_USER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC))
      return 0;
    break;
  case CLASS_MULTI_C_T:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_THIEF))
      return 0;
    break;
  case CLASS_MULTI_F_I:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_FIGHTER) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_ILLUSIONIST))
      return 0;
    break;
  case CLASS_MULTI_C_I:
    if ((GET_OBJ_ANTI(obj) == ITEM_ALLOW_ILLUSIONIST) ||
	(GET_OBJ_ANTI(obj) == ITEM_ALLOW_CLERIC))
      return 0;
    break;
  }
  return 1;
}



/*
 * SPELLS .  This area defines which spells are assigned to
 * which classes, and the minimum level the character must be to use
 * the spell or skill.
 */
void init_spell_levels(void)
{
  int j;
  int cls_mage = (1 << CLASS_MAGIC_USER);
  int cls_cleric = (1 << CLASS_CLERIC);
  int cls_thief = (1 << CLASS_THIEF);
  int cls_warrior = (1 << CLASS_FIGHTER);

  // Assign a spell/skill to a a whole group of classes (0 is all)
  // For instance, { SKILL_SECOND_ATTACK, cls_mage | cls_cleric, 14 },
  // will give mages and clerics the SECOND_ATTACK skill at level 14.
  // More convenient than individual spell_level()s.  Use 0 to give
  // a skill to all the classes.
  //   -dkoepke

  // in my base patch, cls_mage, etc. are unused and that leads to
  // annyoing warnings, so here I'll use them...
  j = (cls_mage - cls_mage) + (cls_cleric - cls_cleric) + (cls_thief - cls_thief) +
      (cls_warrior - cls_warrior);

  /* PRIEST SPELL */
  spell_level(SPELL_BLESS, CLASS_CLERIC, 1, 0);
  //spell_level(SPELL_COMMAND, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_CREATE_WATER, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_CURE_LIGHT, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_CAUSE_LIGHT, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_DETECT_EVIL, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_DETECT_GOOD, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_DETECT_MAGIC, CLASS_CLERIC, 1, 0);
  //spell_level(SPELL_LIGHT, CLASS_CLERIC, 1, 0);
  //spell_level(SPELL_DARKNESS, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_PROT_FROM_EVIL, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_PROT_FROM_GOOD, CLASS_CLERIC, 1, 0);
  /*
     spell_level(SPELL_PURIFY_FOOD_WATER, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_CONTAM_FOOD_WATER, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_REMOVE_FEAR, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_FEAR, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_COMBINE, CLASS_CLERIC, 1, 0);
   */
  spell_level(SPELL_SANCTUARY, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_DETECT_POISON, CLASS_CLERIC, 1, 0);
  /*
     spell_level(SPELL_ENDURE_COLD, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_ANIMAL_FRIENDSHIP, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_ENDURE_HEAT, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_DETECT_SNARES, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_ENTANGLE, CLASS_CLERIC, 1, 0);
   */
  spell_level(SPELL_FAERIE_FIRE, CLASS_CLERIC, 1, 0);
  spell_level(SPELL_INVIS_ANIMALS, CLASS_CLERIC, 1, 0);
  /*
     spell_level(SPELL_LOCATE_ANIMAL, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_PASS_WITHOUT_TRACE, CLASS_CLERIC, 1, 0);
   */
  spell_level(SPELL_INVIS_UNDEAD, CLASS_CLERIC, 1, 0);
  /*
     spell_level(SPELL_MAGICAL_STONE, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_SHILLELAGH, CLASS_CLERIC, 1, 0);
     spell_level(SPELL_LOCATE_PLANTS, CLASS_CLERIC, 1, 0);

     spell_level(SPELL_RESIST_COLD, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_AUGURY, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_CHANT, CLASS_CLERIC, 2, 0);
   */
  spell_level(SPELL_DETECT_CHARM, CLASS_CLERIC, 2, 0);
  spell_level(SPELL_UNDETECTABLE_CHARM, CLASS_CLERIC, 2, 0);
  /*
     spell_level(SPELL_FIND_TRAPS, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_HOLD_PERSON, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_KNOW_ALIGN, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_RESIST_FIRE, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_SILENCE_15, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_SLOW_POISON, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_SNAKE_CHARM, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_SPEAK_ANIMALS, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_SPIRIT_HAMMER, CLASS_CLERIC, 2, 0);
   */
  spell_level(SPELL_AID, CLASS_CLERIC, 2, 0);
  //spell_level(SPELL_DUST_DEVIL, CLASS_CLERIC, 2, 0);

  spell_level(SPELL_BARKSKIN, CLASS_CLERIC, 2, 0);
  spell_level(SPELL_CHARM_MAMMAL, CLASS_CLERIC, 2, 0);
  /*
     spell_level(SPELL_FIRE_TRAP, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_HEAT_METAL, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_CHILL_METAL, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_OBSCUREMENT, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_PRODUCE_FLAME, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_TRIP, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_WARP_WOOD, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_ENTHRALL, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_FLAME_BLADE, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_GOODBERRY, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_BADBERRY, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_MESSENGER, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_WITHDRAW, CLASS_CLERIC, 2, 0);
     spell_level(SPELL_WYVERN_WATCH, CLASS_CLERIC, 2, 0);
   */

  /*
     spell_level(SPELL_ANIMATE_DEAD, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_CONTINUAL_LIGHT, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_CONTINUAL_DARKNESS, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_CREATE_FOOD, CLASS_CLERIC, 3, 0);
   */
  spell_level(SPELL_CURE_BLINDNESS, CLASS_CLERIC, 3, 0);
  spell_level(SPELL_CURE_DEAFNESS, CLASS_CLERIC, 3, 0);
  spell_level(SPELL_CAUSE_BLINDNESS, CLASS_CLERIC, 3, 0);
  spell_level(SPELL_CAUSE_DEAFNESS, CLASS_CLERIC, 3, 0);
  /*
     spell_level(SPELL_CURE_DISEASE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_CAUSE_DISEASE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_DISPEL_MAGIC, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_FEIGN_DEATH, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_GLYPH_WARDING, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_LOCATE_OBJECT, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_OBSCURE_OBJECT, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_PRAYER, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_REMOVE_CURSE, CLASS_CLERIC, 3, 0);
   */
  spell_level(SPELL_CURSE, CLASS_CLERIC, 3, 0);
  //spell_level(SPELL_SPEAK_WITH_DEAD, CLASS_CLERIC, 3, 0);
  spell_level(SPELL_CURE_SERIOUS, CLASS_CLERIC, 3, 0);
  spell_level(SPELL_CAUSE_SERIOUS, CLASS_CLERIC, 3, 0);
  /*
     spell_level(SPELL_CALL_LIGHTNING, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_HOLD_ANIMAL, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_PLANT_GROWTH, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_PROT_FIRE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_PYROTECHNICS, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_SNARE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_STONE_SHAPE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_SUMMON_INSECTS, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_TREE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_WATER_BREATHING, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_AIR_BREATHING, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_FLAME_WALK, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_MAGICAL_VESTMENT, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_MELD_INTO_STONE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_NEG_PLANE_PROT, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_REMOVE_PARA, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_SPIKE_GROWTH, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_STARSHINE, CLASS_CLERIC, 3, 0);
     spell_level(SPELL_WATER_WALK, CLASS_CLERIC, 3, 0);

     spell_level(SPELL_DETECT_LIE, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_UNDET_LIE, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_DIVINATION, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_LOWER_WATER, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_RAISE_WATER, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_NEUT_POISON, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_POISON, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_PROT_EVIL_10, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_PROT_GOOD_10, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_SPEAK_PLANTS, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_STICKS_SNAKES, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_SNAKES_STICKS, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_TONGUES, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_REV_TONGUES, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_ANIMAL_SUMM1, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_CALL_WOOD_BEINGS, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_CONT_TEMP_10, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_HALLUC_FOREST, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_HOLD_PLANT, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_PLANT_DOOR, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_PROD_FIRE, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_QUENCH_FIRE, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_PROT_LIGHTNING, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_REPEL_INSECTS, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_ABJURE, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_CLOAK_BRAVERY, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_FREE_ACTION, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_GIANT_INSECT, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_SMALL_INSECT, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_IMBUE_SPELL_ABILITY, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_REFLECTING_POOL, CLASS_CLERIC, 4, 0);
     spell_level(SPELL_SPELL_IMMUNITY, CLASS_CLERIC, 4, 0);

     spell_level(SPELL_ATONEMENT, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_COMMUNE, CLASS_CLERIC, 5, 0);
   */
  spell_level(SPELL_CURE_CRITIC, CLASS_CLERIC, 5, 0);
  spell_level(SPELL_CAUSE_CRITIC, CLASS_CLERIC, 5, 0);
  /*
     spell_level(SPELL_DISPEL_EVIL, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_DISPEL_GOOD, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_FLAME_STRIKE, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_INSECT_PLAGUE, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_PLANE_SHIFT, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_QUEST, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_RAISE_DEAD, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_SLAY_LIVING, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_TRUE_SEEING, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_FALSE_SEEING, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_ANIMAL_GROWTH, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_REDUCE_ANIMAL, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_ANIM_SUMM2, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_ANTI_PLANT_SHELL, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_COMMUNE_NATURE, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_CONTROL_WINDS, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_PASS_PLANT, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_ROCK_MUD, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_MUD_ROCK, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_WALL_FIRE, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_AIR_WALK, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_MAGIC_FONT, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_MOONBEAM, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_RAINBOW, CLASS_CLERIC, 5, 0);
     spell_level(SPELL_SPIKE_STONES, CLASS_CLERIC, 5, 0);

     spell_level(SPELL_AERIAL_SERVANT, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_ANIMATE_OBJECT, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_BLADE_BARRIER, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_CONJURE_ANIMALS, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_FIND_PATH, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_LOSE_PATH, CLASS_CLERIC, 6, 0);
   */
  spell_level(SPELL_HEAL, CLASS_CLERIC, 6, 0);
  spell_level(SPELL_HARM, CLASS_CLERIC, 6, 0);
  /*
     spell_level(SPELL_PART_WATER, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_SPEAK_MONSTERS, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_STONE_TELL, CLASS_CLERIC, 6, 0);
   */
  spell_level(SPELL_WORD_RECALL, CLASS_CLERIC, 6, 0);
  /*
     spell_level(SPELL_ANIM_SUMM3, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_ANTI_ANIM_SHELL, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_CONJ_FIRE_ELEMENTAL, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_DISMISS_FIRE_ELEMENTAL, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_FIRE_SEEDS, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_TRANS_VIA_PLANTS, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_TURN_WOOD, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_WALL_THORNS, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_WEATHER_SUMMONING, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_FORBIDDANCE, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_HEROS_FEAST, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_LIVEOAK, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_WATER_TO_DUST, CLASS_CLERIC, 6, 0);
     spell_level(SPELL_DUST_TO_WATER, CLASS_CLERIC, 6, 0);

     spell_level(SPELL_ASTRAL_SPELL, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CONTROL_WEATHER, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_EARTHQUAKE, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_HOLYWORD, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_UNHOLYWORD, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_REGENERATE, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_WITHER, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_RESTORATION, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_ENERGY_DRAIN, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_RESURECTION, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_DESTRUCTION, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_SYMBOL, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_WIND_WALK, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_ANIMATE_ROCK, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CHARIOT_SUSTARRE, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CONFUSION, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CONJ_EARTH_ELEMENTAL, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_DISMISS_EARTH_ELEMENTAL, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CREEPING_DOOM, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_FIRE_STORM, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_FIRE_QUENCH, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_REINCARNATE, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_TRANS_METAL_WOOD, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_CHANGESTAFF, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_EXACTION, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_GATE, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_SUNRAY, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_SUCCOR, CLASS_CLERIC, 7, 0);
     spell_level(SPELL_WOOD_TO_METAL, CLASS_CLERIC, 7, 0);
   */


  /* MAGE SPELL */
  /*
     spell_level(SPELL_AFFECT_NORMAL_FIRES, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_ALARM, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_ARMOR, CLASS_MAGIC_USER, 1, 0);
  //spell_level(SPELL_AUDIBLE_GLAMER, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_BURNING_HANDS, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_CANTRIP, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_CHANGE_SELF, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_CHILL_TOUCH, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_COLOR_SPRAY, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_COMPREHEND_LANGUAGES, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_OBSCURE_LANGUAGES, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_DANCING_LIGHTS, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_DETECT_UNDEAD, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_DETECT_MAGIC, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_ENLARGE, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_REDUCE, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_ERASE, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_FEATHER_FALL, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_FIND_FAMILIAR, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_FRIENDS, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_GAZE_REFLECTION, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_GREASE, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_HOLD_PORTAL, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_HYPNOTISM, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_IDENTIFY, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_JUMP, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_MAGIC_MISSILE, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_MENDING, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_MESSAGE, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_MOUNT, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_NYSTUL_MAGICAL_AURA, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_PHANTASMAL_FORCE, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_PROT_FROM_EVIL, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_PROT_FROM_GOOD, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_READ_MAGIC, CLASS_MAGIC_USER, 1, 0);
   */
  spell_level(SPELL_SHIELD, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_SHOCKING_GRASP, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_SLEEP, CLASS_MAGIC_USER, 1, 0);
  spell_level(SPELL_SPIDER_CLIMB, CLASS_MAGIC_USER, 1, 0);
  /*
     spell_level(SPELL_SPOOK, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_TAUNT, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_TENSER_FLOATING_DISC, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_UNSEEN_SERVANT, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_VENTRILOQUISM, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_WALL_OF_FOG, CLASS_MAGIC_USER, 1, 0);
     spell_level(SPELL_WIZARD_MARK, CLASS_MAGIC_USER, 1, 0);
   */

  /*
     spell_level(SPELL_ALTER_SELF, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_BIND, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_BLINDNESS, CLASS_MAGIC_USER, 2, 0);
  spell_level(SPELL_BLUR, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_DARKNESS, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_CONTINUAL_LIGHT, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_DEAFNESS, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_DEEPPOCKETS, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_DETECT_EVIL, CLASS_MAGIC_USER, 2, 0);
  spell_level(SPELL_DETECT_GOOD, CLASS_MAGIC_USER, 2, 0);
  spell_level(SPELL_DETECT_INVISIBILITY, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_ESP, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_FLAMING_SPHERE, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_FOG_CLOUD, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_FOOLS_GOLD, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_FORGET, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_GLITTERDUST, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_HYPNOTIC_PATTERN, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_IMPROVED_PHANTASMAL_FORCE, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_INVISIBILITY, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_IRRITATION, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_KNOCK, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_KNOW_ALIGN, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_LEOMUND_TRAP, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_LEVITATE, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_LOCATE_OBJECT, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_OBSCURE_OBJECT, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_MAGIC_MOUTH, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_MELF_ACID_ARROW, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_MIRROR_IMAGE, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_MISDIRECTION, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_PROTECTION_FROM_CANTRIPS, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_PYROTECHNICS, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_RAY_OF_ENFEEBLEMENT, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_ROPE_TRICK, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_SCARE, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_SHATTER, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_SPECTRAL_HAND, CLASS_MAGIC_USER, 2, 0);
   */
  spell_level(SPELL_STINKING_CLOUD, CLASS_MAGIC_USER, 2, 0);
  spell_level(SPELL_STRENGTH, CLASS_MAGIC_USER, 2, 0);
  /*
     spell_level(SPELL_SUMMON_SWARM, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_TASHA_UNCONTROLLABLE_HIDEOUS_LAUGHTER, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_WEB, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_WHISPERING_WIND, CLASS_MAGIC_USER, 2, 0);
     spell_level(SPELL_WIZARD_LOCK, CLASS_MAGIC_USER, 2, 0);
   */

  spell_level(SPELL_BLINK, CLASS_MAGIC_USER, 3, 0);
  //spell_level(SPELL_CLAIRAUDIENCE, CLASS_MAGIC_USER, 3, 0);
  spell_level(SPELL_CLAIRVOYANCE, CLASS_MAGIC_USER, 3, 0);
  /*
     spell_level(SPELL_DELUDE, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_DISPEL_MAGIC, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_EXPLOSIVE_RUNES, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_FEIGN_DEATH, CLASS_MAGIC_USER, 3, 0);
   */
  spell_level(SPELL_FIREBALL, CLASS_MAGIC_USER, 3, 0);
  //spell_level(SPELL_FLAME_ARROW, CLASS_MAGIC_USER, 3, 0);
  spell_level(SPELL_FLY, CLASS_MAGIC_USER, 3, 0);
  //spell_level(SPELL_GUST_OF_WIND, CLASS_MAGIC_USER, 3, 0);
  spell_level(SPELL_HASTE, CLASS_MAGIC_USER, 3, 0);
  /*
     spell_level(SPELL_HOLD_PERSON, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_HOLD_UNDEAD, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_ILLUSIONARY_SCRIPT, CLASS_MAGIC_USER, 3, 0);
   */
  spell_level(SPELL_INFRAVISION, CLASS_MAGIC_USER, 3, 0);
  spell_level(SPELL_INVISIBILITY_10, CLASS_MAGIC_USER, 3, 0);
  /*
     spell_level(SPELL_ITEM, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_LEOMUND_TINY_HUT, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_LIGHTNING_BOLT, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_MELF_MINUTE_METEORS, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_MONSTER_SUMMONING_I, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_NON_DETECTION, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_PHANTOM_STEED, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_PROT_EVIL_10, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_PROT_GOOD_10, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_PROTECTION_FROM_NORMAL_MISSILES, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_SECRET_PAGE, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_SEPIA_SNAKE_SIGIL, CLASS_MAGIC_USER, 3, 0);
   */
  spell_level(SPELL_SLOW, CLASS_MAGIC_USER, 3, 0);
  /*
     spell_level(SPELL_SPECTRAL_FORCE, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_SUGGESTION, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_TONGUES, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_REV_TONGUES, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_VAMPIRIC_TOUCH, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_WATER_BREATHING, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_AIR_BREATHING, CLASS_MAGIC_USER, 3, 0);
     spell_level(SPELL_WIND_WALL, CLASS_MAGIC_USER, 3, 0);
   */
  spell_level(SPELL_WRAITHFORM, CLASS_MAGIC_USER, 3, 0);

  /*
     spell_level(SPELL_CHARM_MONSTER, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_CONFUSION, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_CONTAGION, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_DETECT_SCRYING, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_DIG, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_DIMENSION_DOOR, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_EMOTION, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_ENCHANTED_WEAPON, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_ENERVATION, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_EVARD_BLACK_TENTACLES, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_EXTENSION_I, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_FIRE_CHARM, CLASS_MAGIC_USER, 4, 0);
   */
  spell_level(SPELL_FIRE_SHIELD, CLASS_MAGIC_USER, 4, 0);
  /*
     spell_level(SPELL_FIRE_TRAP, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_FUMBLE, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_HALLUCINATORY_TERRAIN, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_ICE_STORM, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_ILLUSIONARY_WALL, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_IMPROVED_INVISIBILITY, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_LEOMUND_SECURE_SHELTER, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_MAGIC_MIRROR, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_MASSMORPH, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_MINOR_CREATION, CLASS_MAGIC_USER, 4, 0);
   */
  spell_level(SPELL_MINOR_GLOBE_OF_INVULNERABILITY, CLASS_MAGIC_USER, 4, 0);
  /*
     spell_level(SPELL_MONSTER_SUMMONING_II, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_OTILUKE_RESILIENT_SPHERE, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_PHANTASMAL_KILLER, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_PLANT_GROWTH, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_POLYMORPH_OTHER, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_POLYMORPH_SELF, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_RAINBOW_PATTERN, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_RARY_MNEMONIC_ENHANCER, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_REMOVE_CURSE, CLASS_MAGIC_USER, 4, 0);
   */
  spell_level(SPELL_CURSE, CLASS_MAGIC_USER, 4, 0);
  /*
     spell_level(SPELL_SHADOW_MONSTERS, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_SHOUT, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_SOLID_FOG, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_STONESKIN, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_VACANCY, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_WALL_OF_FIRE, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_WALL_OF_ICE, CLASS_MAGIC_USER, 4, 0);
     spell_level(SPELL_WIZARD_EYE, CLASS_MAGIC_USER, 4, 0);
   */

  /*
     spell_level(SPELL_ADVANCED_ILLUSION, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_AIRY_WATER, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_ANIMAL_GROWTH, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_REDUCE_ANIMAL, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_ANIMATE_DEAD, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_AVOIDANCE, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_BIGBY_INTERPOSING_HAND, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_CHAOS, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_CLOUDKILL, CLASS_MAGIC_USER, 5, 0);
   */
  spell_level(SPELL_CONE_OF_COLD, CLASS_MAGIC_USER, 5, 0);
  /*
     spell_level(SPELL_CONJURE_ELEMENTAL, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_CONTACT_OTHER_PLANE, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_DEMI_SHADOW_MONSTERS, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_DISMISSAL, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_DISTANCE_DISTORTION, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_DOMINATION, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_DREAM, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_EXTENSION_II, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_FABRICATE, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_FALSE_VISION, CLASS_MAGIC_USER, 5, 0);
   */
  spell_level(SPELL_FEEBLEMIND, CLASS_MAGIC_USER, 5, 0);
  /*
     spell_level(SPELL_HOLD_MONSTER, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_LEOMUND_LAMENTABLE_BELABORMENT, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_LEOMUND_SECRET_CHEST, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_MAGIC_JAR, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_MAJOR_CREATION, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_MONSTER_SUMMONING_III, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_MORDENKAINEN_FAITHFUL_HOUND, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_PASSWALL, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_SEEMING, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_SENDING, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_SHADOW_DOOR, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_SHADOW_MAGIC, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_STONE_SHAPE, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_SUMMON_SHADOW, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_TELEKINESIS, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_TELEPORT, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_TRANSMUTE_ROCK_TO_MUD, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_TRANSMUTE_MUD_TO_ROCK, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_WALL_OF_FORCE, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_WALL_OF_IRON, CLASS_MAGIC_USER, 5, 0);
     spell_level(SPELL_WALL_OF_STONE, CLASS_MAGIC_USER, 5, 0);
   */

  /*
     spell_level(SPELL_ANTI_MAGIC_SHELL, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_BIGBY_FORCEFUL_HAND, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_CHAIN_LIGHTNING, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_CONJURE_ANIMALS, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_CONTINGENCY, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_CONTROL_WEATHER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_DEATH_FOG, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_DEATH_SPELL, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_DEMI_SHADOW_MAGIC, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_DISINTEGRATE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_ENCHANT_AN_ITEM, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_ENSNAREMENT, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_EXTENSION_III, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_EYEBITE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_GEAS, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_GLASSEE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_GLOBE_OF_INVULNERABILITY, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_GUARDS_AND_WARDS, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_INVISIBLE_STALKER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_LEGEND_LORE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_LOWER_WATER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_RAISE_WATER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MASS_SUGGESTION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MIRAGE_ARCANA, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MISLEAD, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MONSTER_SUMMONING_IV, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MORDENKAINEN_LUCUBRATION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_MOVE_EARTH, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_OTILUKE_FREEZING_SPHERE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_PART_WATER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_PERMANENT_ILLUSION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_PROGRAMMED_ILLUSION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_PROJECT_IMAGE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_REINCARNATION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_REPULSION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_SHADES, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_STONE_TO_FLESH, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_FLESH_TO_STONE, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_TENSER_TRANSFORMATION, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_WATER_TO_DUST, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_DUST_TO_WATER, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_TRUE_SEEING, CLASS_MAGIC_USER, 6, 0);
     spell_level(SPELL_VEIL, CLASS_MAGIC_USER, 6, 0);
   */

  /*
     spell_level(SPELL_BANISHMENT, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_BIGBY_GRASPING_HAND, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_CHARM_PLANTS, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_CONTROL_UNDEAD, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_DELAYED_BLAST_FIREBALL, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_DRAWMIJ_INSTANT_SUMMONS, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_DUO_DIMENSION, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_FINGER_OF_DEATH, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_FORCECAGE, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_LIMITED_WISH, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_MASS_INVISIBILITY, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_MONSTER_SUMMONING_V, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_MORDENKAINEN_MAGNIFICENT_MANSION, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_MORDENKAINEN_SWORD, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_PHASE_DOOR, CLASS_MAGIC_USER, 7, 0);
   */
  spell_level(SPELL_POWER_WORD_STUN, CLASS_MAGIC_USER, 7, 0);
  /*
     spell_level(SPELL_PRISMATIC_SPRAY, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_REVERSE_GRAVITY, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_SEQUESTER, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_SHADOW_WALK, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_SIMULACRUM, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_SPELL_TURNING, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_STATUE, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_TELEPORT_WITHOUT_ERROR, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_VANISH, CLASS_MAGIC_USER, 7, 0);
     spell_level(SPELL_VISION, CLASS_MAGIC_USER, 7, 0);
   */

  /*
     spell_level(SPELL_ANTIPATHY_SYMPATHY, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_BIGBY_CLENCHED_FIST, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_BINDING, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_CLONE, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_DEMAND, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_GLASSTEEL, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_INCENDIARY_CLOUD, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_MASS_CHARM, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_MAZE, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_MIND_BLANK, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_MONSTER_SUMMONING_VI, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_OTILUKE_TELEKINETIC_SPHERE, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_OTTO_IRRESISTIBLE_DANCE, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_PERMANENCY, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_POLYMORPH_ANY_OBJECT, CLASS_MAGIC_USER, 8, 0);
   */
  spell_level(SPELL_POWER_WORD_BLIND, CLASS_MAGIC_USER, 8, 0);
  /*
     spell_level(SPELL_PRISMATIC_WALL, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_SCREEN, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_SERTEN_SPELL_IMMUNITY, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_SINK, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_SYMBOL, CLASS_MAGIC_USER, 8, 0);
     spell_level(SPELL_TRAP_THE_SOUL, CLASS_MAGIC_USER, 8, 0);
   */

  /*
     spell_level(SPELL_ASTRAL_SPELL, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_BIGBY_CRUSHING_HAND, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_CRYSTALBRITTLE, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_ENERGY_DRAIN, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_FORESIGHT, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_GATE, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_IMPRISONMENT, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_METEOR_SWARM, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_MONSTER_SUMMONING_VII, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_MORDENKAINEN_DISJUNCTION, CLASS_MAGIC_USER, 9, 0);
   */
  spell_level(SPELL_POWER_WORD_KILL, CLASS_MAGIC_USER, 9, 0);
  /*
     spell_level(SPELL_PRISMATIC_SPHERE, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_SHAPE_CHANGE, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_SUCCOR, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_TEMPORAL_STASIS, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_TIME_STOP, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_WEIRD, CLASS_MAGIC_USER, 9, 0);
     spell_level(SPELL_WISH, CLASS_MAGIC_USER, 9, 0);
   */
}


/*
 * This is the exp given to implementors -- it must always be greater
 * than the exp required for immortality, plus at least 20,000 or so.
 */
#define EXP_MAX  10000000

/* Function to return the exp required for each class/level */
int level_exp(int class, int level)
{
  if (level > LVL_IMPL || level < 0) {
    log("SYSERR: Requesting exp for invalid level!");
    return 0;
  }
  /*
   * Gods have exp close to EXP_MAX.  This statement should never have to
   * changed, regardless of how many mortal or immortal levels exist.
   */
  if (level > LVL_IMMORT) {
    return EXP_MAX - ((LVL_IMPL - level) * 1000);
  }
  /* Exp required for normal mortals is below */

  switch (class) {

  case CLASS_MAGIC_USER:
  case CLASS_ILLUSIONIST:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 2500;
      break;
    case 3:
      return 5000;
      break;
    case 4:
      return 10000;
      break;
    case 5:
      return 20000;
      break;
    case 6:
      return 40000;
      break;
    case 7:
      return 60000;
      break;
    case 8:
      return 90000;
      break;
    case 9:
      return 135000;
      break;
    case 10:
      return 250000;
      break;
    case 11:
      return 375000;
      break;
    case 12:
      return 750000;
      break;
    case 13:
      return 1125000;
      break;
    case 14:
      return 1500000;
      break;
    case 15:
      return 1875000;
      break;
    case 16:
      return 2250000;
      break;
    case 17:
      return 2625000;
      break;
    case 18:
      return 3000000;
      break;
    case 19:
      return 3375000;
      break;
    case 20:
      return 3750000;
      break;
    case 21:
      return 4000000;
      break;
    case 22:
      return 4300000;
      break;
    case 23:
      return 4600000;
      break;
    case 24:
      return 4900000;
      break;
    case 25:
      return 5200000;
      break;
    case 26:
      return 5500000;
      break;
    case 27:
      return 5950000;
      break;
    case 28:
      return 6400000;
      break;
    case 29:
      return 6850000;
      break;
    case 30:
      return 7400000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 8000000;
      break;
    }
    break;

  case CLASS_CLERIC:
  case CLASS_DRUID:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 1500;
      break;
    case 3:
      return 3000;
      break;
    case 4:
      return 6000;
      break;
    case 5:
      return 13000;
      break;
    case 6:
      return 27500;
      break;
    case 7:
      return 55000;
      break;
    case 8:
      return 110000;
      break;
    case 9:
      return 225000;
      break;
    case 10:
      return 450000;
      break;
    case 11:
      return 675000;
      break;
    case 12:
      return 900000;
      break;
    case 13:
      return 1125000;
      break;
    case 14:
      return 1350000;
      break;
    case 15:
      return 1575000;
      break;
    case 16:
      return 1800000;
      break;
    case 17:
      return 2025000;
      break;
    case 18:
      return 2250000;
      break;
    case 19:
      return 2475000;
      break;
    case 20:
      return 2700000;
      break;
    case 21:
      return 2950000;
      break;
    case 22:
      return 3200000;
      break;
    case 23:
      return 3450000;
      break;
    case 24:
      return 3700000;
      break;
    case 25:
      return 3950000;
      break;
    case 26:
      return 4200000;
      break;
    case 27:
      return 4450000;
      break;
    case 28:
      return 4700000;
      break;
    case 29:
      return 4950000;
      break;
    case 30:
      return 5200000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 7000000;
      break;
    }
    break;

  case CLASS_THIEF:
  case CLASS_BARD:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 1250;
      break;
    case 3:
      return 2500;
      break;
    case 4:
      return 5000;
      break;
    case 5:
      return 10000;
      break;
    case 6:
      return 20000;
      break;
    case 7:
      return 30000;
      break;
    case 8:
      return 70000;
      break;
    case 9:
      return 110000;
      break;
    case 10:
      return 160000;
      break;
    case 11:
      return 220000;
      break;
    case 12:
      return 440000;
      break;
    case 13:
      return 660000;
      break;
    case 14:
      return 880000;
      break;
    case 15:
      return 1100000;
      break;
    case 16:
      return 1320000;
      break;
    case 17:
      return 1540000;
      break;
    case 18:
      return 1760000;
      break;
    case 19:
      return 1980000;
      break;
    case 20:
      return 2200000;
      break;
    case 21:
      return 2420000;
      break;
    case 22:
      return 2640000;
      break;
    case 23:
      return 2860000;
      break;
    case 24:
      return 3080000;
      break;
    case 25:
      return 3300000;
      break;
    case 26:
      return 3520000;
      break;
    case 27:
      return 3740000;
      break;
    case 28:
      return 3960000;
      break;
    case 29:
      return 4180000;
      break;
    case 30:
      return 4400000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 7000000;
      break;
    }
    break;

  case CLASS_FIGHTER:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 2000;
      break;
    case 3:
      return 4000;
      break;
    case 4:
      return 8000;
      break;
    case 5:
      return 16000;
      break;
    case 6:
      return 32000;
      break;
    case 7:
      return 64000;
      break;
    case 8:
      return 125000;
      break;
    case 9:
      return 250000;
      break;
    case 10:
      return 500000;
      break;
    case 11:
      return 750000;
      break;
    case 12:
      return 1000000;
      break;
    case 13:
      return 1250000;
      break;
    case 14:
      return 1500000;
      break;
    case 15:
      return 1750000;
      break;
    case 16:
      return 2000000;
      break;
    case 17:
      return 2250000;
      break;
    case 18:
      return 2500000;
      break;
    case 19:
      return 2750000;
      break;
    case 20:
      return 3000000;
      break;
    case 21:
      return 3300000;
      break;
    case 22:
      return 3600000;
      break;
    case 23:
      return 3900000;
      break;
    case 24:
      return 4200000;
      break;
    case 25:
      return 4500000;
      break;
    case 26:
      return 4800000;
      break;
    case 27:
      return 5100000;
      break;
    case 28:
      return 5400000;
      break;
    case 29:
      return 5700000;
      break;
    case 30:
      return 6100000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 8000000;
      break;
    }
    break;

  case CLASS_RANGER:
  case CLASS_PALADIN:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 2250;
      break;
    case 3:
      return 4500;
      break;
    case 4:
      return 9000;
      break;
    case 5:
      return 18000;
      break;
    case 6:
      return 36000;
      break;
    case 7:
      return 75000;
      break;
    case 8:
      return 150000;
      break;
    case 9:
      return 300000;
      break;
    case 10:
      return 600000;
      break;
    case 11:
      return 900000;
      break;
    case 12:
      return 1200000;
      break;
    case 13:
      return 1500000;
      break;
    case 14:
      return 1800000;
      break;
    case 15:
      return 2100000;
      break;
    case 16:
      return 2400000;
      break;
    case 17:
      return 2700000;
      break;
    case 18:
      return 3000000;
      break;
    case 19:
      return 3300000;
      break;
    case 20:
      return 3600000;
      break;
    case 21:
      return 3900000;
      break;
    case 22:
      return 4200000;
      break;
    case 23:
      return 4500000;
      break;
    case 24:
      return 4800000;
      break;
    case 25:
      return 5150000;
      break;
    case 26:
      return 5500000;
      break;
    case 27:
      return 5950000;
      break;
    case 28:
      return 6400000;
      break;
    case 29:
      return 6850000;
      break;
    case 30:
      return 7400000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 8000000;
      break;
    }
    break;

  case CLASS_MULTI_C_M:
  case CLASS_MULTI_F_M:
  case CLASS_MULTI_F_T:
  case CLASS_MULTI_M_T:
  case CLASS_MULTI_C_R:
  case CLASS_MULTI_C_F:
  case CLASS_MULTI_I_T:
  case CLASS_MULTI_C_T:
  case CLASS_MULTI_F_I:
  case CLASS_MULTI_C_I:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 2500;
      break;
    case 3:
      return 5000;
      break;
    case 4:
      return 10000;
      break;
    case 5:
      return 20000;
      break;
    case 6:
      return 40000;
      break;
    case 7:
      return 60000;
      break;
    case 8:
      return 140000;
      break;
    case 9:
      return 220000;
      break;
    case 10:
      return 320000;
      break;
    case 11:
      return 440000;
      break;
    case 12:
      return 880000;
      break;
    case 13:
      return 1320000;
      break;
    case 14:
      return 1760000;
      break;
    case 15:
      return 2200000;
      break;
    case 16:
      return 3000000;
      break;
    case 17:
      return 4000000;
      break;
    case 18:
      return 5000000;
      break;
    case 19:
      return 6000000;
      break;
    case 20:
      return 7000000;
      break;
    case 21:
      return 7300000;
      break;
    case 22:
      return 7600000;
      break;
    case 23:
      return 8200000;
      break;
    case 24:
      return 8800000;
      break;
    case 25:
      return 9400000;
      break;
    case 26:
      return 10200000;
      break;
    case 27:
      return 11000000;
      break;
    case 28:
      return 11800000;
      break;
    case 29:
      return 12600000;
      break;
    case 30:
      return 13300000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 7000000;
      break;
    }
    break;

  case CLASS_MULTI_C_F_M:
  case CLASS_MULTI_F_M_T:
    switch (level) {
    case 0:
      return 0;
      break;
    case 1:
      return 1;
      break;
    case 2:
      return 6250;
      break;
    case 3:
      return 7500;
      break;
    case 4:
      return 15000;
      break;
    case 5:
      return 30000;
      break;
    case 6:
      return 60000;
      break;
    case 7:
      return 90000;
      break;
    case 8:
      return 210000;
      break;
    case 9:
      return 330000;
      break;
    case 10:
      return 320000;
      break;
    case 11:
      return 660000;
      break;
    case 12:
      return 1320000;
      break;
    case 13:
      return 1980000;
      break;
    case 14:
      return 2640000;
      break;
    case 15:
      return 3300000;
      break;
    case 16:
      return 4500000;
      break;
    case 17:
      return 6000000;
      break;
    case 18:
      return 7500000;
      break;
    case 19:
      return 9000000;
      break;
    case 20:
      return 10500000;
      break;
    case 21:
      return 10950000;
      break;
    case 22:
      return 11400000;
      break;
    case 23:
      return 12300000;
      break;
    case 24:
      return 13200000;
      break;
    case 25:
      return 14100000;
      break;
    case 26:
      return 15300000;
      break;
    case 27:
      return 16500000;
      break;
    case 28:
      return 17700000;
      break;
    case 29:
      return 18900000;
      break;
    case 30:
      return 19950000;
      break;
      /* add new levels here */
    case LVL_IMMORT:
      return 7000000;
      break;
    }
    break;
  }

  /*
   * This statement should never be reached if the exp tables in this function
   * are set up properly.  If you see exp of 123456 then the tables above are
   * incomplete -- so, complete them!
   */
  log("SYSERR: XP tables not set up correctly in class.c!");
  return 123456;
}


void set_saving_throws(struct char_data *ch)
{
  extern struct con_app_type con_app[];
  extern const int save_wizard[][7];
  extern const int save_warrior[][7];
  extern const int save_priest[][7];
  extern const int save_rogue[][7];

  int lvl, i, Class;

  if (IS_MOB(ch)) {
    lvl = GET_LEVEL(ch);
    if (GET_HPPLUS(ch) > 0)
      lvl += (int) (GET_HPPLUS(ch) / 4);

    if (GET_CLASS(ch) == CLASS_UNDEFINED) {
      Class = CLASS_FIGHTER;
    } else {
      Class = GET_CLASS(ch);
    }

  } else {
    lvl = GET_LEVEL(ch);
    Class = GET_CLASS(ch);
  }

  switch (Class) {

  case CLASS_MAGIC_USER:
  case CLASS_ILLUSIONIST:
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	GET_SAVE(ch, 0) = save_wizard[i][2];
	GET_SAVE(ch, 1) = save_wizard[i][3];
	GET_SAVE(ch, 2) = save_wizard[i][4];
	GET_SAVE(ch, 3) = save_wizard[i][5];
	GET_SAVE(ch, 4) = save_wizard[i][6];
	break;
      }
    }
    break;
  case CLASS_RANGER:
  case CLASS_FIGHTER:
  case CLASS_PALADIN:
    for (i = 0; i < 11; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	GET_SAVE(ch, 0) = save_warrior[i][2];
	GET_SAVE(ch, 1) = save_warrior[i][3];
	GET_SAVE(ch, 2) = save_warrior[i][4];
	GET_SAVE(ch, 3) = save_warrior[i][5];
	GET_SAVE(ch, 4) = save_warrior[i][6];
	break;
      }
    }
    break;

  case CLASS_CLERIC:
  case CLASS_DRUID:
    for (i = 0; i < 8; i++) {
      if (lvl >= save_priest[i][0] && lvl <= save_priest[i][1]) {
	GET_SAVE(ch, 0) = save_priest[i][2];
	GET_SAVE(ch, 1) = save_priest[i][3];
	GET_SAVE(ch, 2) = save_priest[i][4];
	GET_SAVE(ch, 3) = save_priest[i][5];
	GET_SAVE(ch, 4) = save_priest[i][6];
	break;
      }
    }
    break;

  case CLASS_THIEF:
  case CLASS_BARD:
    for (i = 0; i < 7; i++) {
      if (lvl >= save_rogue[i][0] && lvl <= save_rogue[i][1]) {
	GET_SAVE(ch, 0) = save_rogue[i][2];
	GET_SAVE(ch, 1) = save_rogue[i][3];
	GET_SAVE(ch, 2) = save_rogue[i][4];
	GET_SAVE(ch, 3) = save_rogue[i][5];
	GET_SAVE(ch, 4) = save_rogue[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_C_M:
  case CLASS_MULTI_C_I:
    for (i = 0; i < 8; i++) {
      if (lvl >= save_priest[i][0] && lvl <= save_priest[i][1]) {
	GET_SAVE(ch, 0) = save_priest[i][2];
	GET_SAVE(ch, 1) = save_priest[i][3];
	GET_SAVE(ch, 2) = save_priest[i][4];
	GET_SAVE(ch, 3) = save_priest[i][5];
	GET_SAVE(ch, 4) = save_priest[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	if (GET_SAVE(ch, 0) > save_wizard[i][2])
	  GET_SAVE(ch, 0) = save_wizard[i][2];
	if (GET_SAVE(ch, 1) > save_wizard[i][3])
	  GET_SAVE(ch, 1) = save_wizard[i][3];
	if (GET_SAVE(ch, 1) > save_wizard[i][4])
	  GET_SAVE(ch, 1) = save_wizard[i][4];
	if (GET_SAVE(ch, 1) > save_wizard[i][5])
	  GET_SAVE(ch, 1) = save_wizard[i][5];
	if (GET_SAVE(ch, 1) > save_wizard[i][6])
	  GET_SAVE(ch, 1) = save_wizard[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_C_F:
  case CLASS_MULTI_C_R:
    for (i = 0; i < 8; i++) {
      if (lvl >= save_priest[i][0] && lvl <= save_priest[i][1]) {
	GET_SAVE(ch, 0) = save_priest[i][2];
	GET_SAVE(ch, 1) = save_priest[i][3];
	GET_SAVE(ch, 2) = save_priest[i][4];
	GET_SAVE(ch, 3) = save_priest[i][5];
	GET_SAVE(ch, 4) = save_priest[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 11; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	if (GET_SAVE(ch, 0) > save_warrior[i][2])
	  GET_SAVE(ch, 0) = save_warrior[i][2];
	if (GET_SAVE(ch, 1) > save_warrior[i][3])
	  GET_SAVE(ch, 1) = save_warrior[i][3];
	if (GET_SAVE(ch, 1) > save_warrior[i][4])
	  GET_SAVE(ch, 1) = save_warrior[i][4];
	if (GET_SAVE(ch, 1) > save_warrior[i][5])
	  GET_SAVE(ch, 1) = save_warrior[i][5];
	if (GET_SAVE(ch, 1) > save_warrior[i][6])
	  GET_SAVE(ch, 1) = save_warrior[i][6];
	break;
      }
    }
    break;


  case CLASS_MULTI_I_T:
  case CLASS_MULTI_M_T:
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	GET_SAVE(ch, 0) = save_wizard[i][2];
	GET_SAVE(ch, 1) = save_wizard[i][3];
	GET_SAVE(ch, 2) = save_wizard[i][4];
	GET_SAVE(ch, 3) = save_wizard[i][5];
	GET_SAVE(ch, 4) = save_wizard[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 7; i++) {
      if (lvl >= save_rogue[i][0] && lvl <= save_rogue[i][1]) {
	if (GET_SAVE(ch, 0) > save_rogue[i][2])
	  GET_SAVE(ch, 0) = save_rogue[i][2];
	if (GET_SAVE(ch, 1) > save_rogue[i][3])
	  GET_SAVE(ch, 1) = save_rogue[i][3];
	if (GET_SAVE(ch, 1) > save_rogue[i][4])
	  GET_SAVE(ch, 1) = save_rogue[i][4];
	if (GET_SAVE(ch, 1) > save_rogue[i][5])
	  GET_SAVE(ch, 1) = save_rogue[i][5];
	if (GET_SAVE(ch, 1) > save_rogue[i][6])
	  GET_SAVE(ch, 1) = save_rogue[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_F_T:
    for (i = 0; i < 11; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	GET_SAVE(ch, 0) = save_warrior[i][2];
	GET_SAVE(ch, 1) = save_warrior[i][3];
	GET_SAVE(ch, 2) = save_warrior[i][4];
	GET_SAVE(ch, 3) = save_warrior[i][5];
	GET_SAVE(ch, 4) = save_warrior[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 7; i++) {
      if (lvl >= save_rogue[i][0] && lvl <= save_rogue[i][1]) {
	if (GET_SAVE(ch, 0) > save_rogue[i][2])
	  GET_SAVE(ch, 0) = save_rogue[i][2];
	if (GET_SAVE(ch, 1) > save_rogue[i][3])
	  GET_SAVE(ch, 1) = save_rogue[i][3];
	if (GET_SAVE(ch, 1) > save_rogue[i][4])
	  GET_SAVE(ch, 1) = save_rogue[i][4];
	if (GET_SAVE(ch, 1) > save_rogue[i][5])
	  GET_SAVE(ch, 1) = save_rogue[i][5];
	if (GET_SAVE(ch, 1) > save_rogue[i][6])
	  GET_SAVE(ch, 1) = save_rogue[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_F_M:
  case CLASS_MULTI_F_I:
    for (i = 0; i < 11; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	GET_SAVE(ch, 0) = save_warrior[i][2];
	GET_SAVE(ch, 1) = save_warrior[i][3];
	GET_SAVE(ch, 2) = save_warrior[i][4];
	GET_SAVE(ch, 3) = save_warrior[i][5];
	GET_SAVE(ch, 4) = save_warrior[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	if (GET_SAVE(ch, 0) > save_wizard[i][2])
	  GET_SAVE(ch, 0) = save_wizard[i][2];
	if (GET_SAVE(ch, 1) > save_wizard[i][3])
	  GET_SAVE(ch, 1) = save_wizard[i][3];
	if (GET_SAVE(ch, 1) > save_wizard[i][4])
	  GET_SAVE(ch, 1) = save_wizard[i][4];
	if (GET_SAVE(ch, 1) > save_wizard[i][5])
	  GET_SAVE(ch, 1) = save_wizard[i][5];
	if (GET_SAVE(ch, 1) > save_wizard[i][6])
	  GET_SAVE(ch, 1) = save_wizard[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_C_F_M:
    for (i = 0; i < 8; i++) {
      if (lvl >= save_priest[i][0] && lvl <= save_priest[i][1]) {
	GET_SAVE(ch, 0) = save_priest[i][2];
	GET_SAVE(ch, 1) = save_priest[i][3];
	GET_SAVE(ch, 2) = save_priest[i][4];
	GET_SAVE(ch, 3) = save_priest[i][5];
	GET_SAVE(ch, 4) = save_priest[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	if (GET_SAVE(ch, 0) > save_wizard[i][2])
	  GET_SAVE(ch, 0) = save_wizard[i][2];
	if (GET_SAVE(ch, 1) > save_wizard[i][3])
	  GET_SAVE(ch, 1) = save_wizard[i][3];
	if (GET_SAVE(ch, 1) > save_wizard[i][4])
	  GET_SAVE(ch, 1) = save_wizard[i][4];
	if (GET_SAVE(ch, 1) > save_wizard[i][5])
	  GET_SAVE(ch, 1) = save_wizard[i][5];
	if (GET_SAVE(ch, 1) > save_wizard[i][6])
	  GET_SAVE(ch, 1) = save_wizard[i][6];
	break;
      }
    }
    for (i = 0; i < 7; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	if (GET_SAVE(ch, 0) > save_warrior[i][2])
	  GET_SAVE(ch, 0) = save_warrior[i][2];
	if (GET_SAVE(ch, 1) > save_warrior[i][3])
	  GET_SAVE(ch, 1) = save_warrior[i][3];
	if (GET_SAVE(ch, 1) > save_warrior[i][4])
	  GET_SAVE(ch, 1) = save_warrior[i][4];
	if (GET_SAVE(ch, 1) > save_warrior[i][5])
	  GET_SAVE(ch, 1) = save_warrior[i][5];
	if (GET_SAVE(ch, 1) > save_warrior[i][6])
	  GET_SAVE(ch, 1) = save_warrior[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_F_M_T:
    for (i = 0; i < 11; i++) {
      if (lvl >= save_warrior[i][0] && lvl <= save_warrior[i][1]) {
	GET_SAVE(ch, 0) = save_warrior[i][2];
	GET_SAVE(ch, 1) = save_warrior[i][3];
	GET_SAVE(ch, 2) = save_warrior[i][4];
	GET_SAVE(ch, 3) = save_warrior[i][5];
	GET_SAVE(ch, 4) = save_warrior[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 6; i++) {
      if (lvl >= save_wizard[i][0] && lvl <= save_wizard[i][1]) {
	if (GET_SAVE(ch, 0) > save_wizard[i][2])
	  GET_SAVE(ch, 0) = save_wizard[i][2];
	if (GET_SAVE(ch, 1) > save_wizard[i][3])
	  GET_SAVE(ch, 1) = save_wizard[i][3];
	if (GET_SAVE(ch, 1) > save_wizard[i][4])
	  GET_SAVE(ch, 1) = save_wizard[i][4];
	if (GET_SAVE(ch, 1) > save_wizard[i][5])
	  GET_SAVE(ch, 1) = save_wizard[i][5];
	if (GET_SAVE(ch, 1) > save_wizard[i][6])
	  GET_SAVE(ch, 1) = save_wizard[i][6];
	break;
      }
    }
    for (i = 0; i < 7; i++) {
      if (lvl >= save_rogue[i][0] && lvl <= save_rogue[i][1]) {
	if (GET_SAVE(ch, 0) > save_rogue[i][2])
	  GET_SAVE(ch, 0) = save_rogue[i][2];
	if (GET_SAVE(ch, 1) > save_rogue[i][3])
	  GET_SAVE(ch, 1) = save_rogue[i][3];
	if (GET_SAVE(ch, 1) > save_rogue[i][4])
	  GET_SAVE(ch, 1) = save_rogue[i][4];
	if (GET_SAVE(ch, 1) > save_rogue[i][5])
	  GET_SAVE(ch, 1) = save_rogue[i][5];
	if (GET_SAVE(ch, 1) > save_rogue[i][6])
	  GET_SAVE(ch, 1) = save_rogue[i][6];
	break;
      }
    }
    break;

  case CLASS_MULTI_C_T:
    for (i = 0; i < 6; i++) {
      if (lvl >= save_priest[i][0] && lvl <= save_priest[i][1]) {
	GET_SAVE(ch, 0) = save_priest[i][2];
	GET_SAVE(ch, 1) = save_priest[i][3];
	GET_SAVE(ch, 2) = save_priest[i][4];
	GET_SAVE(ch, 3) = save_priest[i][5];
	GET_SAVE(ch, 4) = save_priest[i][6];
	break;
      }
    }
    break;
    for (i = 0; i < 7; i++) {
      if (lvl >= save_rogue[i][0] && lvl <= save_rogue[i][1]) {
	if (GET_SAVE(ch, 0) > save_rogue[i][2])
	  GET_SAVE(ch, 0) = save_rogue[i][2];
	if (GET_SAVE(ch, 1) > save_rogue[i][3])
	  GET_SAVE(ch, 1) = save_rogue[i][3];
	if (GET_SAVE(ch, 1) > save_rogue[i][4])
	  GET_SAVE(ch, 1) = save_rogue[i][4];
	if (GET_SAVE(ch, 1) > save_rogue[i][5])
	  GET_SAVE(ch, 1) = save_rogue[i][5];
	if (GET_SAVE(ch, 1) > save_rogue[i][6])
	  GET_SAVE(ch, 1) = save_rogue[i][6];
	break;
      }
    }
    break;

  }

  if (GET_RACE(ch) == RACE_DWARF || GET_RACE(ch) == RACE_GNOME || GET_RACE(ch) == RACE_HALFLING) {
    GET_SAVE(ch, 1) -= con_app[GET_CON(ch)].special;
    GET_SAVE(ch, 4) -= con_app[GET_CON(ch)].special;
  }
  if (GET_CLASS(ch) == CLASS_PALADIN) {
    GET_SAVE(ch, 0) -= 2;
    GET_SAVE(ch, 1) -= 2;
    GET_SAVE(ch, 2) -= 2;
    GET_SAVE(ch, 3) -= 2;
    GET_SAVE(ch, 4) -= 2;
  }
}

void proficiency_set(struct char_data *ch)
{

  if (GET_LEVEL(ch) == 1 && GET_HIGH_LEVEL(ch) == 1) {
    switch (GET_CLASS(ch)) {

    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_C_I:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_I_T:
    case CLASS_MULTI_C_T:
      GET_NON_WEAPON_PROF(ch) = 4;
      GET_WEAPON_PROF(ch) = 2;
      break;
    case CLASS_FIGHTER:
    case CLASS_PALADIN:
    case CLASS_RANGER:
    case CLASS_MULTI_F_T:
      GET_NON_WEAPON_PROF(ch) = 3;
      GET_WEAPON_PROF(ch) = 4;
      break;
    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
      GET_NON_WEAPON_PROF(ch) = 4;
      GET_WEAPON_PROF(ch) = 1;
      break;
    case CLASS_THIEF:
    case CLASS_BARD:
      GET_NON_WEAPON_PROF(ch) = 3;
      GET_WEAPON_PROF(ch) = 2;
      break;
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_F_I:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_C_F:
    case CLASS_MULTI_C_F_M:
      GET_WEAPON_PROF(ch) = 4;
      GET_NON_WEAPON_PROF(ch) = 4;
      break;

    }
    return;
  }
  if ((GET_LEVEL(ch) % 6 == 0) && GET_LEVEL(ch) > GET_HIGH_LEVEL(ch)) {
    switch (GET_CLASS(ch)) {
    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
      GET_WEAPON_PROF(ch)++;
      break;
    }
  }
  if ((GET_LEVEL(ch) % 3 == 0) && GET_LEVEL(ch) > GET_HIGH_LEVEL(ch)) {
    switch (GET_CLASS(ch)) {
    case CLASS_FIGHTER:
    case CLASS_PALADIN:
    case CLASS_RANGER:
    case CLASS_MULTI_F_T:
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_F_I:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_C_F:
    case CLASS_MULTI_C_F_M:
      GET_WEAPON_PROF(ch)++;
      GET_NON_WEAPON_PROF(ch)++;
      break;

    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_C_I:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_I_T:
    case CLASS_MULTI_C_T:
      GET_NON_WEAPON_PROF(ch)++;
      break;
    }
  }
  if ((GET_LEVEL(ch) % 4 == 0) && GET_LEVEL(ch) > GET_HIGH_LEVEL(ch)) {
    switch (GET_CLASS(ch)) {
    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_C_I:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_I_T:
    case CLASS_MULTI_C_T:
      GET_WEAPON_PROF(ch)++;
      break;
    case CLASS_THIEF:
    case CLASS_BARD:
      GET_WEAPON_PROF(ch)++;
      GET_NON_WEAPON_PROF(ch)++;
      break;
    }
    return;
  }
}


void thief_skill_set(struct char_data *ch)
{

  if ((GET_LEVEL(ch) > 1) && GET_LEVEL(ch) > GET_HIGH_LEVEL(ch)) {
    switch (GET_CLASS(ch)) {
    case CLASS_THIEF:
    case CLASS_MULTI_F_T:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_I_T:
    case CLASS_MULTI_C_T:
      GET_THIEF_SKILL(ch) += 30;
      break;
    case CLASS_BARD:
      GET_THIEF_SKILL(ch) += 15;
      break;
    }
  }
}

ACMD(do_specialize)
{
  extern char *specialist_list[];
  int i, special = 0;

  one_argument(argument, arg);

  if (IS_MAGE_SPECIALIST(ch)) {
    send_to_char("You are already a specialist!\r\n", ch);
    return;
  }
  if (GET_CLASS(ch) != CLASS_MAGIC_USER) {
    send_to_char("Only true mage may become specialist!\r\n", ch);
    return;
  }
  if (GET_LEVEL(ch) > 1) {
    send_to_char("You are too high level to specialize, you should have tought about it sooner!\r\n", ch);
    return;
  }
  if (!*arg) {
    send_to_char("&+YList of mage speciality.&N\r\n", ch);
    for (i = 0; i < NUM_SPECIALIST; i++) {
      sprintf(buf, "%s\r\n", specialist_list[i]);
      send_to_char(buf, ch);
    }
    return;
  }
  for (i = 0; i < NUM_SPECIALIST; i++) {
    if (!str_cmp(arg, specialist_list[i]))
      special = i + 1;
  }
  if (special) {
    switch (special) {
    case ABJURER:
      if (GET_RACE(ch) != RACE_HUMAN) {
	send_to_char("Only human may become Abjurer!\r\n", ch);
	return;
      }
      if (GET_WIS(ch) < 15) {
	send_to_char("You are not wise enough to become an abjurer!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ALTERATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ILLUSION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_PHANTASM);
      break;
    case CONJURER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_HALFELF &&
	  GET_RACE(ch) != RACE_P_DROW) {
	send_to_char("Only human, half-elf or drow may become Conjurer!\r\n", ch);
	return;
      }
      if (GET_CON(ch) < 15) {
	send_to_char("Your constitution is not high enough to become an conjurer!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_GREATER_DIVIN);
      break;
    case DIVINER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_ELF &&
	  GET_RACE(ch) != RACE_HALFELF && GET_RACE(ch) != RACE_P_ORC &&
	  GET_RACE(ch) != RACE_P_GOBLIN) {
	send_to_char("Only human, elf, orc, goblin or half-elf may become Diviner!\r\n", ch);
	return;
      }
      if (GET_WIS(ch) < 16) {
	send_to_char("You are not wise enough to become a diviner!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_SUMMONING);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CONJURATION);
      break;
    case ENCHANTER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_ELF &&
	  GET_RACE(ch) != RACE_P_DROW && GET_RACE(ch) != RACE_P_OGRE &&
	  GET_RACE(ch) != RACE_P_ORC && GET_RACE(ch) != RACE_HALFELF) {
	send_to_char("Only human, elf, drow, ogre, orc or half-elf may become Enchanter!\r\n", ch);
	return;
      }
      if (GET_CHA(ch) < 16) {
	send_to_char("You dont have enough charisma to become an enchanter!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_EVOCATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_INVOCATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
      break;
    case INVOKER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_P_DROW) {
	send_to_char("Only human or drow may become Invoker!\r\n", ch);
	return;
      }
      if (GET_CON(ch) < 16) {
	send_to_char("Your constitution is not high enough to become an invoker!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_SUMMONING);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CONJURATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ENCHANTMENT);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CHARM);
      break;
    case NECROMANCER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_P_DROW) {
	send_to_char("Only human or drow may become Necromancer!\r\n", ch);
	return;
      }
      if (GET_WIS(ch) < 16) {
	send_to_char("You are not wise enough to become a necromancer!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ENCHANTMENT);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_CHARM);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ILLUSION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_PHANTASM);
      break;
    case TRANSMUTER:
      if (GET_RACE(ch) != RACE_HUMAN && GET_RACE(ch) != RACE_P_DROW &&
	  GET_RACE(ch) != RACE_HALFELF) {
	send_to_char("Only human drow or half-elf may become Transmuter!\r\n", ch);
	return;
      }
      if (GET_DEX(ch) < 15) {
	send_to_char("You lack the dexterity to become a transmuter!\r\n", ch);
	return;
      }
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_ABJURATION);
      REMOVE_BIT(GET_MAGE_SCHOOL(ch), SCHOOL_NECROMANCY);
      break;
    default:
      send_to_char("Specialize to what!\r\n", ch);
      return;
      break;
    }
    IS_MAGE_SPECIALIST(ch) = special;
    sprintf(buf, "You are now a %s!\r\n", specialist_list[special - 1]);
    send_to_char(buf, ch);

  } else {
    send_to_char("Specialize to what!\r\n", ch);
  }
}

ACMD(do_deity)
{
  int i, special = 0, top, race = GET_RACE(ch);

  play_sound(SOUND_DEAD, ch, 0, 0, 0, 0, TO_CHAR);


  if (IS_AFFECTED(ch, AFF_CHARM)) {
    send_to_char("You may have been ordered to do this, so we will ignore you.\r\n", ch);
    return;
  }
  //one_argument(argument, arg);
  skip_spaces(&argument);

  switch (race) {
  case RACE_HUMAN:
    top = NUM_HUMAN;
    break;
  case RACE_HALFELF:
    top = NUM_HFELF;
    break;
  case RACE_ELF:
    top = NUM_ELF;
    break;
  case RACE_DWARF:
    top = NUM_DWARF;
    break;
  case RACE_HALFLING:
    top = NUM_HALFLING;
    break;
  case RACE_GNOME:
    top = NUM_GNOME;
    break;
  case RACE_HALFORC:
    top = NUM_HFORC;
    break;
  default:
    send_to_char("Sorry, you are of a godless variety!", ch);
    return;
  }
  if (!*argument) {
    if (GET_PRIEST_DEITY(ch)) {
      sprintf(buf, "You are worshiping %s.\r\n", deity_list[GET_PRIEST_DEITY(ch)].deity_name);
      send_to_char(buf, ch);
      return;
    }
    strcpy(buf, "&+YList of deities.&N\r\n");

    if (!IS_TRUSTED(ch)) {
      for (i = 0; i < top; i++) {
	sprintf(buf, "%s%-25s (%s)\r\n", buf,
		deity_list[race_gods[race][i]].deity_name,
		deity_list[race_gods[race][i]].deity_pantheon);
      }
    } else {
      for (i = 1; i < NUM_DEITY; i++) {
	sprintf(buf, "%s%-25s (%s)\r\n", buf, deity_list[i].deity_name,
		deity_list[i].deity_pantheon);
      }
    }
    page_string(ch->desc, buf, 1);
    return;
  }
  if (!strcasecmp(argument, "none")) {
    if ((GET_PRIEST_DEITY(ch) != 0) && !IS_TRUSTED(ch)) {
      act("$n screams as $s belief in $s god is destroyed.", TRUE, ch, NULL, NULL, TO_PROXIMITY2);
      act("You scream in pain as a part of your soul is ripped away.", TRUE, ch, NULL, NULL, TO_CHAR);
      gain_exp(ch, -(GET_EXP(ch) >> 1));
    }
    GET_PRIEST_DEITY(ch) = 0;
    GET_PRIEST_SPHERE_MAJOR(ch) = 0;
    GET_PRIEST_SPHERE_MINOR(ch) = 0;
    return;
  }
  if (GET_PRIEST_DEITY(ch) != 0) {
    send_to_char("You are already worshipping a god. To worship another god, you must first type deity none.\r\n", ch);
    return;
  }
  for (i = 0; i <= top; i++) {
    if (!strcasecmp(argument, deity_list[race_gods[race][i]].deity_name)) {
      if (GET_ALIGNMENT(ch) == ALIGN_NONE) {
	;
      } else if (deity_list[race_gods[race][i]].aligns[GET_ALIGNMENT(ch) - 1] == 1)
	special = race_gods[race][i];
      else {
	send_to_char("That god wouldn't want you as a follower.\r\n", ch);
	return;
      }
    }
  }
  if (special) {
    if ((GET_CLASS(ch) == CLASS_PALADIN) ||
	(IS_PRIEST(ch) && GET_CLASS(ch) != CLASS_DRUID)) {
      GET_PRIEST_SPHERE_MAJOR(ch) = deity_list[special].deity_major_sphere;
      GET_PRIEST_SPHERE_MINOR(ch) = deity_list[special].deity_minor_sphere;
    }
    if (GET_CLASS(ch) == CLASS_DRUID) {
      if (!((special == 17) || (special == 102))) {
	send_to_char("You cannot worship that god.\r\n", ch);
	return;
      }
      GET_PRIEST_SPHERE_MAJOR(ch) = deity_list[special].deity_major_sphere;
      GET_PRIEST_SPHERE_MINOR(ch) = deity_list[special].deity_minor_sphere;
    }
    GET_PRIEST_DEITY(ch) = special;

    sprintf(buf, "You start worshiping %s!\r\n", deity_list[special].deity_name);
    send_to_char(buf, ch);

  } else {
    send_to_char("Worship who?\r\n", ch);
  }
}
