
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>
#include "structs.h"
#include "interpreter.h"
#include "handler.h"
#include "comm.h"
#include "utils.h"
#include "db.h"
#include "olc.h"

extern int top_of_helpt;
extern struct help_index_element *help_table;
int find_help(char *keyword);

int hsort(const void *a, const void *b);

void free_help(struct help_index_element *help);

 /* function protos */
void hedit_disp_menu(struct descriptor_data *d);
void hedit_parse(struct descriptor_data *d, char *arg);
void hedit_setup_new(struct descriptor_data *d);
void hedit_setup_existing(struct descriptor_data *d, int real_num);
void hedit_save_to_disk(struct descriptor_data *d);
void hedit_save_internally(struct descriptor_data *d);
void hedit_delete(struct descriptor_data *d);
void index_boot(int mode);

 /*
  * Utils and exported functions.
  */

void hedit_setup_new(struct descriptor_data *d)
{
  CREATE(OLC_HELP(d), struct help_index_element, 1);
  OLC_HELP(d)->keyword = str_dup(OLC_STORAGE(d));
  sprintf(buf, "%s\r\n", OLC_STORAGE(d));
  OLC_HELP(d)->entry = str_dup(buf);
  OLC_HELP(d)->min_level = 0;
  OLC_HELP(d)->duplicate = 0;
  OLC_HELP(d)->save = TRUE;
  hedit_disp_menu(d);
  OLC_VAL(d) = 0;
}

/*------------------------------------------------------------------------*/

void hedit_setup_existing(struct descriptor_data *d, int real_num)
{
  CREATE(OLC_HELP(d), struct help_index_element, 1);
  OLC_HELP(d)->keyword = str_dup(help_table[real_num].keyword);
  OLC_HELP(d)->entry = str_dup(help_table[real_num].entry);
  OLC_HELP(d)->min_level = help_table[real_num].min_level;
  OLC_HELP(d)->duplicate = help_table[real_num].duplicate;
  OLC_HELP(d)->save = help_table[real_num].save;
  OLC_VAL(d) = 0;
  hedit_disp_menu(d);
}


void hedit_save_internally(struct descriptor_data *d)
{
  extern int top_of_helpt;
  extern struct help_index_element *help_table;

  struct help_index_element *new_help_table = NULL;
  int i;

  /* add a new help element into the list */
  if (OLC_ZNUM(d) > top_of_helpt) {
    CREATE(new_help_table, struct help_index_element, top_of_helpt + 2);
    for (i = 0; i <= top_of_helpt; i++)
      new_help_table[i] = help_table[i];
    new_help_table[++top_of_helpt] = *OLC_HELP(d);
    FREE(help_table);
    help_table = new_help_table;

    qsort(help_table, top_of_helpt + 1, sizeof(struct help_index_element), hsort);

  } else {
    i = find_help(OLC_HELP(d)->keyword);

    free_help(help_table + OLC_ZNUM(d));
    help_table[OLC_ZNUM(d)] = *OLC_HELP(d);
  }
  olc_add_to_save_list(HEDIT_PERMISSION, OLC_SAVE_HELP);

}


void hedit_delete(struct descriptor_data *d)
{
  extern int top_of_helpt;
  extern struct help_index_element *help_table;

  struct help_index_element *new_help_table = NULL;
  int i, found = FALSE;

  sprintf(buf, "Deleting help topic '%s'\r\n", OLC_STORAGE(d));
  send_to_char(buf, d->character);

  CREATE(new_help_table, struct help_index_element, top_of_helpt);

  for (i = 0; i <= top_of_helpt; i++) {
    if (!found) {
      if (!str_cmp(help_table[i].keyword, OLC_STORAGE(d))) {
	found = TRUE;
      } else {
	new_help_table[i] = help_table[i];
      }
    } else {
      new_help_table[i - 1] = help_table[i];
    }
  }

  FREE(help_table);
  help_table = new_help_table;
  top_of_helpt--;

  qsort(help_table, top_of_helpt + 1, sizeof(struct help_index_element), hsort);

/*  hedit_save_to_disk(d); */

}

/*------------------------------------------------------------------------*/

void hedit_save_to_disk(struct descriptor_data *d)
{
  FILE *fp;
  int i;

  if (!(fp = fopen(HELP_FILE, "w+"))) {
    sprintf(buf, "Can't open help file '%s'", HELP_FILE);
    perror(buf);
    raise(SIGSEGV);
  }
  for (i = 0; i <= top_of_helpt; i++) {
    if (help_table[i].duplicate > 0)
      continue;

    if (help_table[i].save == FALSE)
      continue;

    /*. Remove the '\r\n' sequences from entry . */
    strcpy(buf1, help_table[i].entry ? help_table[i].entry : "Empty");
    strip_string(buf1);

    sprintf(buf, "%s\n#%d\n",
	    buf1,
	    help_table[i].min_level);
    fputs(buf, fp);
  }

  fprintf(fp, "$\n");
  fclose(fp);
  olc_remove_from_save_list(HEDIT_PERMISSION, OLC_SAVE_HELP);
}

/*------------------------------------------------------------------------*/

 /* Menu functions */

 /* the main menu */
void hedit_disp_menu(struct descriptor_data *d)
{
  struct help_index_element *help;

  help = OLC_HELP(d);

  get_char_cols(d->character);

  sprintf(buf,
#if defined(CLEAR_SCREEN)
	  "[H[J"
#endif
	  "Keywords    : %s%s\r\n"
	  "%s1%s) Entry       :\r\n%s%s"
	  "%s2%s) Min Level   : %s%d\r\n"
	  "%sQ%s) Quit\r\n"
	  "Enter choice : ",
	  yel, help->keyword,
	  grn, nrm, yel, help->entry,
	  grn, nrm, yel, help->min_level,
	  grn, nrm
      );
  send_to_char(buf, d->character);

  OLC_MODE(d) = HEDIT_MAIN_MENU;
}

 /*
  * The main loop
  */

void hedit_parse(struct descriptor_data *d, char *arg)
{
  int i;

  switch (OLC_MODE(d)) {
  case HEDIT_CONFIRM_SAVESTRING:
    switch (*arg) {
    case 'y':
    case 'Y':
      hedit_save_internally(d);
      sprintf(buf, "OLC: %s edits help.", GET_NAME(d->character));
      mudlog(buf, CMP, LVL_IMPL, TRUE);
      /* do not free the strings.. just the structure */
      cleanup_olc(d, CLEANUP_STRUCTS);
      send_to_char("Help saved to memory.\r\n", d->character);
      break;
    case 'n':
    case 'N':
      /* free everything up, including strings etc */
      cleanup_olc(d, CLEANUP_ALL);
      break;
    default:
      send_to_char("Invalid choice!\r\nDo you wish to save this help internally? ", d->character);
      break;
    }
    return;			/* end of HEDIT_CONFIRM_SAVESTRING */

  case HEDIT_CONFIRM_DELETE:
    switch (*arg) {
    case 'y':
    case 'Y':
      hedit_delete(d);
      sprintf(buf, "OLC: %s delete help topic %s", GET_NAME(d->character),
	      OLC_STORAGE(d));
      mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(d->character)), TRUE);
      cleanup_olc(d, CLEANUP_STRUCTS);
      return;
    default:
      cleanup_olc(d, CLEANUP_ALL);
      return;
    }
    break;

  case HEDIT_CONFIRM_EDIT:
    switch (*arg) {
    case 'y':
    case 'Y':
      hedit_setup_existing(d, OLC_ZNUM(d));
      break;
    case 'q':
    case 'Q':
      cleanup_olc(d, CLEANUP_ALL);
      break;
    case 'n':
    case 'N':
      OLC_ZNUM(d)++;
      for (; (OLC_ZNUM(d) <= top_of_helpt); OLC_ZNUM(d)++)
	if (is_abbrev(OLC_STORAGE(d), help_table[OLC_ZNUM(d)].keyword))
	  break;
      if (OLC_ZNUM(d) > top_of_helpt) {
	if (find_help(OLC_STORAGE(d)) > NOTHING) {
	  cleanup_olc(d, CLEANUP_ALL);
	  break;
	}
	sprintf(buf, "Do you wish to add help on '%s'? ",
		OLC_STORAGE(d));
	send_to_char(buf, d->character);
	OLC_MODE(d) = HEDIT_CONFIRM_ADD;
      } else {
	sprintf(buf, "Do you wish to edit help on '%s'? ", help_table[OLC_ZNUM(d)].keyword);
	send_to_char(buf, d->character);
	OLC_MODE(d) = HEDIT_CONFIRM_EDIT;
      }
      break;
    default:
      sprintf(buf, "Invalid choice!\r\nDo you wish to edit help on '%s'? ", help_table[OLC_ZNUM(d)].keyword);
      send_to_char(buf, d->character);
      break;
    }
    return;

  case HEDIT_CONFIRM_ADD:
    switch (*arg) {
    case 'y':
    case 'Y':
      hedit_setup_new(d);
      break;
    case 'n':
    case 'N':
    case 'q':
    case 'Q':
      cleanup_olc(d, CLEANUP_ALL);
      break;
    default:
      sprintf(buf, "Invalid choice!\r\nDo you wish to add help on '%s'? ", OLC_STORAGE(d));
      send_to_char(buf, d->character);
      break;
    }
    return;

  case HEDIT_MAIN_MENU:
    switch (*arg) {
    case 'q':
    case 'Q':
      if (OLC_VAL(d)) {		/* Something was modified */
	send_to_char("Do you wish to save this help internally? ", d->character);
	OLC_MODE(d) = HEDIT_CONFIRM_SAVESTRING;
      } else
	cleanup_olc(d, CLEANUP_ALL);
      break;
    case '1':
      OLC_MODE(d) = HEDIT_ENTRY;
      SEND_TO_Q("\x1B[H\x1B[J", d);
      SEND_TO_Q("Enter the help info: (/s saves /h for help)\r\n\r\n", d);
      d->backstr = NULL;
      if (OLC_HELP(d)->entry) {
	SEND_TO_Q(OLC_HELP(d)->entry, d);
	d->backstr = str_dup(OLC_HELP(d)->entry);
      }
      d->str = &OLC_HELP(d)->entry;
      d->max_str = MAX_STRING_LENGTH;
      d->mail_to = 0;
      OLC_VAL(d) = 1;
      break;
    case '2':
      send_to_char("Enter the minimum level a player has to be to read this help: "
		   ,d->character);
      OLC_MODE(d) = HEDIT_MIN_LEVEL;
      return;
    default:
      hedit_disp_menu(d);
      break;
    }
    return;

  case HEDIT_ENTRY:
    /* should never get here */
    mudlog("SYSERR: Reached HEDIT_ENTRY in hedit_parse", BRF, LVL_BUILDER, TRUE);
    break;
  case HEDIT_MIN_LEVEL:
    if (*arg) {
      i = atoi(arg);
      if ((i < 0) && (i > LVL_IMPL)) {
	hedit_disp_menu(d);
	return;
      } else
	OLC_HELP(d)->min_level = i;
    } else {
      hedit_disp_menu(d);
      return;
    }
    break;
  default:
    /* we should never get here */
    break;
  }
  OLC_VAL(d) = 1;
  hedit_disp_menu(d);
}

int find_help(char *keyword)
{

  extern int top_of_helpt;
  extern struct help_index_element *help_table;

  int chk, bot, top, mid, minlen;

  bot = 0;
  top = top_of_helpt;
  minlen = strlen(keyword);

  for (;;) {
    mid = (bot + top) / 2;

    if (bot > top)
      return -1;
    else if (!(chk = strn_cmp(keyword, help_table[mid].keyword, minlen))) {
      /* trace backwards to find first matching entry. Thanks Jeff Fink! */
      while ((mid > 0) &&
       (!(chk = strn_cmp(keyword, help_table[mid - 1].keyword, minlen))))
	mid--;
      return mid;
    } else {
      if (chk > 0)
	bot = mid + 1;
      else
	top = mid - 1;
    }
  }
}

void free_help(struct help_index_element *help)
{
  if (help->keyword)
    FREE(help->keyword);
  if (help->entry)
    FREE(help->entry);
  memset(help, 0, sizeof(struct help_index_element));
}
