/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define ACMD(name)  \
   void name(struct char_data *ch, char *argument, int cmd, int subcmd)

#define CMD_NAME (complete_cmd_info[cmd].command)
#define CMD_IS(cmd_name) (!strcmp(cmd_name, complete_cmd_info[cmd].command))
#define IS_MOVE(cmdnum) (cmdnum >= 1 && cmdnum <= 10)

/* necessary for CMD_IS macro */
#ifndef __INTERPRETER_C__
extern struct command_info *complete_cmd_info;
#endif


void command_interpreter(struct char_data *ch, char *argument);
int search_block(char *arg, char **list, int exact);
char lower(char c);
char *one_argument(char *argument, char *first_arg);
char *one_word(char *argument, char *first_arg);
char *any_one_arg(char *argument, char *first_arg);
char *two_arguments(char *argument, char *first_arg, char *second_arg);
char *three_arguments(char *argument, char *first_arg, char *second_arg,
	char *third_arg);
char *any_two_arg(char *argument, char *first_arg, char *second_arg);
int fill_word(char *argument);
void half_chop(char *string, char *arg1, char *arg2);
void nanny(struct descriptor_data *d, char *arg);
int is_abbrev(char *arg1, char *arg2);
int is_number(char *str);
int find_command(char *command);
void skip_spaces(char **string);
char *delete_doubledollar(char *string);

/* for compatibility with 2.20: */
#define argument_interpreter(a, b, c) two_arguments(a, b, c)


struct command_info {
  char *command;
  char *sort_as;
  int minimum_position;
  byte in_battle;
  void (*command_pointer)
   (struct char_data * ch, char *argument, int cmd, int subcmd);
  sh_int minimum_level;
  int subcmd;
};

struct alias {
  char *alias;
  char *replacement;
  int type;
  struct alias *next;
};

#define ALIAS_SIMPLE	0
#define ALIAS_COMPLEX	1

#define ALIAS_SEP_CHAR	';'
#define ALIAS_VAR_CHAR	'$'
#define ALIAS_GLOB_CHAR	'*'

/*
 * SUBCOMMANDS
 *   You can define these however you want to, and the definitions of the
 *   subcommands are independent from function to function.
 */

/* directions */
#define SCMD_NORTH	1
#define SCMD_EAST	2
#define SCMD_SOUTH	3
#define SCMD_WEST	4
#define SCMD_UP		5
#define SCMD_DOWN	6
#define SCMD_NORTHEAST  7
#define SCMD_NORTHWEST  8
#define SCMD_SOUTHEAST  9
#define SCMD_SOUTHWEST  10
#define SCMD_ADVANCE    11
#define SCMD_RETREAT    12
#define SCMD_STEP       13
#define SCMD_CIRCLE	14
#define SCMD_ROLL	15

/* do_gen_ps */
#define SCMD_INFO       0
#define SCMD_HANDBOOK   1
#define SCMD_CREDITS    2
#define SCMD_NEWS       3
#define SCMD_WIZLIST    4
#define SCMD_POLICIES   5
#define SCMD_IMMLIST    6
#define SCMD_MOTD	7
#define SCMD_IMOTD	8
#define SCMD_CLEAR	9
#define SCMD_WHOAMI	10

/* do_mskillset */
#define SCMD_SKILLSET   0
#define SCMD_TEACH      1

/* do_gen_tog */
#define SCMD_NOHASSLE   0
#define SCMD_COMPACT    1
#define SCMD_NOTELL	2
#define SCMD_DEAF	3
#define SCMD_NOWIZ	4
#define SCMD_ROOMFLAGS	5
#define SCMD_NOREPEAT	6
#define SCMD_HOLYLIGHT	7
#define SCMD_SLOWNS	8
#define SCMD_PAGING	9
#define SCMD_MUDFTP     10
#define SCMD_IDENT	11
#define SCMD_PUMMEL	12

/* do_wizutil */
#define SCMD_REROLL	0
#define SCMD_PARDON     1
#define SCMD_NOTITLE    2
#define SCMD_SQUELCH    3
#define SCMD_FREEZE	4
#define SCMD_THAW	5
#define SCMD_UNAFFECT	6
#define SCMD_OLIST      7
#define SCMD_MLIST      8
#define SCMD_RLIST      9
#define SCMD_SLIST	10

/* do_shape */
#define SCMD_MOB	0
#define SCMD_OBJ	1

/* do_spec_com */
#define SCMD_WHISPER	0
#define SCMD_ASK	1

/* do_say */
#define SCMD_SAY	0
#define SCMD_SHOUT	1

/* do_mem */
#define SCMD_MEM	0
#define SCMD_PRAY	1

/* do_gen_com */
#define SCMD_PETITION   0
#define SCMD_PTELL	1

/* do_shutdown */
#define SCMD_SHUTDOW	0
#define SCMD_SHUTDOWN   1
#define SCMD_REBOOT	2

/* do_quit */
#define SCMD_QUI	0
#define SCMD_QUIT	1

/* do_date */
#define SCMD_DATE	0
#define SCMD_UPTIME	1

/* do_commands */
#define SCMD_COMMANDS	0
#define SCMD_SOCIALS	1
#define SCMD_WIZHELP	2

/* do_drop */
#define SCMD_DROP	0
#define SCMD_JUNK	1

/* do_gen_write */
#define SCMD_BUG	0
#define SCMD_TYPO	1
#define SCMD_IDEA	2

/* do_look */
#define SCMD_LOOK	0
#define SCMD_READ	1
#define SCMD_GLANCE     2

/* do_pour */
#define SCMD_POUR	0
#define SCMD_FILL	1

/* do_poof */
#define SCMD_POOFIN	0
#define SCMD_POOFOUT	1

/* do_hit */
#define SCMD_HIT	0
#define SCMD_MURDER	1

/* do_eat */
#define SCMD_EAT	0
#define SCMD_TASTE	1
#define SCMD_DRINK	2
#define SCMD_SIP	3

/* do_use */
#define SCMD_USE	0
#define SCMD_QUAFF	1
#define SCMD_RECITE	2

/* do_echo */
#define SCMD_ECHO	0
#define SCMD_EMOTE	1

/* do_gen_door */
#define SCMD_OPEN       1	/* keep 0 free, its hit from above */
#define SCMD_CLOSE      2
#define SCMD_UNLOCK     3
#define SCMD_LOCK       4
#define SCMD_PICK       5

/*. do_olc . */
#define SCMD_OLC_REDIT  0
#define SCMD_OLC_OEDIT  1
#define SCMD_OLC_ZEDIT  2
#define SCMD_OLC_MEDIT  3
#define SCMD_OLC_SEDIT  4
#define SCMD_OLC_TEDIT  5
#define SCMD_OLC_SAVEINFO  6
#define SCMD_OLC_AEDIT  7
#define SCMD_OLC_HEDIT  8
#define SCMD_OLC_QEDIT  9
#define SCMD_OLC_GEDIT  10
