/*****************************************************************/  
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File: limb.c                                                  */
/* Purpose: for all limb type functions                          */
/* Author: Tastone/Kanthas                                                                                                        */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "handler.h"
#include "comm.h"
#include "interpreter.h"
#include "db.h"

extern const char *race_types[];
extern struct index_data *mob_index;

extern struct index_data *obj_index;

//LIMB INFO
//value[7] = mobs vnum, or players id
//value[6] = owners race
//value[5] = 1 if pc, 0 otherwise


//limbs
#define LIMB_DEAD -1

//vorpal proc
#define LOCATION_HEAD 12


#define LIMB_HEAD 	0
#define LIMB_HAND 	1
#define LIMB_ARM  	2
#define LIMB_FOOT 	3
#define LIMB_LEG  	4
#define NUMBER_LIMBS 5

//vnums for body parts
//0 - head              3 - feet
//1 - hands             4 - legs
//2 - arms              5 - torso
const int Parts[6] =
{60, 61, 62, 63, 64, 65};

const char *limb_names[] =
{
  "head",
  "hand",
  "arm",
  "foot",
  "leg",
  "\n"
};


void RemoveLimb(struct char_data *ch, int location);

struct obj_data *KillLimb(struct char_data *ch, int location);
void MakeLimb(struct char_data *ch, struct obj_data *obj);
void AddObjName(struct char_data *ch, struct obj_data *obj);


void RemoveLimb(struct char_data *ch, int location) 
{
  put_obj_to_room(KillLimb(ch, location), ch->in_room,
		  GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
}				//RemoveLimb


struct obj_data *KillLimb(struct char_data *ch, int location)
{
  struct obj_data *obj = NULL, *ch_item = NULL;

  switch (location) {
  case LOCATION_VITALS:
  case LOCATION_BRAIN:
  case LOCATION_FACE:
  case LOCATION_HEAD:
    obj = read_object(Parts[0], VIRTUAL);
    MakeLimb(ch, obj);
    if (ch->equipment[WEAR_HEAD]) {
      ch_item = ch->equipment[WEAR_HEAD];
      obj_to_obj(unequip_char(ch, WEAR_HEAD), obj);
    }				//if

    if (ch->equipment[WEAR_FACE]) {
      ch_item = ch->equipment[WEAR_FACE];
      obj_to_obj(unequip_char(ch, WEAR_FACE), obj);
    }				//if

    if (ch->equipment[WEAR_EYES]) {
      ch_item = ch->equipment[WEAR_EYES];
      obj_to_obj(unequip_char(ch, WEAR_EYES), obj);
    }				//if

    if (ch->equipment[WEAR_EARRING_R]) {
      ch_item = ch->equipment[WEAR_EARRING_R];
      obj_to_obj(unequip_char(ch, WEAR_EARRING_R), obj);
    }				//if

    if (ch->equipment[WEAR_EARRING_L]) {
      ch_item = ch->equipment[WEAR_EARRING_L];
      obj_to_obj(unequip_char(ch, WEAR_EARRING_L), obj);
    }				//if

    if (ch->equipment[WEAR_NECK]) {
      ch_item = ch->equipment[WEAR_NECK];
      put_obj_to_room(unequip_char(ch, WEAR_NECK), ch->in_room,
		      GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    }				//if

    GET_HIT_HD(ch) = -11;
    break;
  case LOCATION_SHIELD_ARM:
    obj = read_object(Parts[2], VIRTUAL);
    MakeLimb(ch, obj);
    if (GET_HIT_SH(ch) != LIMB_DEAD)
      obj_to_obj(KillLimb(ch, LOCATION_SHIELD_HAND), obj);
    if (ch->equipment[WEAR_ARMS] && GET_HIT_WA(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_ARMS];
      obj_to_obj(unequip_char(ch, WEAR_ARMS), obj);
    }				//if

    if (ch->equipment[WEAR_WRIST_L]) {
      ch_item = ch->equipment[WEAR_WRIST_L];
      obj_to_obj(unequip_char(ch, WEAR_WRIST_L), obj);
    }				//if

    GET_HIT_SA(ch) = LIMB_DEAD;
    break;
  case LOCATION_SHIELD_HAND:
    obj = read_object(Parts[1], VIRTUAL);
    MakeLimb(ch, obj);
    if (ch->equipment[WEAR_HANDS] && GET_HIT_WH(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_HANDS];
      obj_to_obj(unequip_char(ch, WEAR_HANDS), obj);
    }				//if

    if (ch->equipment[WEAR_FINGER_L]) {
      ch_item = ch->equipment[WEAR_FINGER_L];
      obj_to_obj(unequip_char(ch, WEAR_FINGER_L), obj);
    }				//if

    if (ch->equipment[WEAR_SHIELD]) {
      ch_item = ch->equipment[WEAR_SHIELD];
      obj_to_obj(unequip_char(ch, WEAR_SHIELD), obj);
    }				//if

    else if (ch->equipment[WEAR_WIELD2]) {
      ch_item = ch->equipment[WEAR_WIELD2];
      obj_to_obj(unequip_char(ch, WEAR_WIELD2), obj);
    }				//if

    else if (ch->equipment[WEAR_HOLD]) {
      ch_item = ch->equipment[WEAR_HOLD];
      obj_to_obj(unequip_char(ch, WEAR_HOLD), obj);
    }				//if

    GET_HIT_SH(ch) = LIMB_DEAD;
    break;
  case LOCATION_WEAPON_HAND:
    obj = read_object(Parts[1], VIRTUAL);
    MakeLimb(ch, obj);
    if (ch->equipment[WEAR_HANDS] && GET_HIT_SH(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_HANDS];
      obj_to_obj(unequip_char(ch, WEAR_HANDS), obj);
    }				//if

    if (ch->equipment[WEAR_FINGER_R]) {
      ch_item = ch->equipment[WEAR_FINGER_R];
      obj_to_obj(unequip_char(ch, WEAR_FINGER_R), obj);
    }				//if

    if (ch->equipment[WEAR_WIELD]) {
      ch_item = ch->equipment[WEAR_WIELD];
      obj_to_obj(unequip_char(ch, WEAR_WIELD), obj);
    }				//if

    else if (ch->equipment[WEAR_HOLD2]) {
      ch_item = ch->equipment[WEAR_HOLD2];
      obj_to_obj(unequip_char(ch, WEAR_HOLD2), obj);
    }				//if

    GET_HIT_WH(ch) = LIMB_DEAD;
    break;
  case LOCATION_WEAPON_ARM:
    obj = read_object(Parts[2], VIRTUAL);
    MakeLimb(ch, obj);
    if (GET_HIT_WH(ch) != LIMB_DEAD)
      obj_to_obj(KillLimb(ch, LOCATION_WEAPON_HAND), obj);
    if (ch->equipment[WEAR_ARMS] && GET_HIT_SA(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_ARMS];
      obj_to_obj(unequip_char(ch, WEAR_ARMS), obj);
    }				//if

    if (ch->equipment[WEAR_WRIST_R]) {
      ch_item = ch->equipment[WEAR_WRIST_R];
      obj_to_obj(unequip_char(ch, WEAR_WRIST_R), obj);
    }				//if

    GET_HIT_WA(ch) = LIMB_DEAD;
    break;
  case LOCATION_TORSO:
    obj = read_object(Parts[5], VIRTUAL);
    MakeLimb(ch, obj);
    GET_HIT_TO(ch) = LIMB_DEAD;
    break;
  case LOCATION_RIGHT_LEG:
    obj = read_object(Parts[4], VIRTUAL);
    MakeLimb(ch, obj);
    if (GET_HIT_RF(ch) != LIMB_DEAD)
      obj_to_obj(KillLimb(ch, LOCATION_RIGHT_FOOT), obj);
    if (ch->equipment[WEAR_LEGS] && GET_HIT_LL(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_LEGS];
      obj_to_obj(unequip_char(ch, WEAR_LEGS), obj);
    }				//if

    GET_HIT_RL(ch) = LIMB_DEAD;
    break;
  case LOCATION_LEFT_LEG:
    obj = read_object(Parts[4], VIRTUAL);
    MakeLimb(ch, obj);
    if (GET_HIT_LF(ch) != LIMB_DEAD)
      obj_to_obj(KillLimb(ch, LOCATION_LEFT_FOOT), obj);
    if (ch->equipment[WEAR_LEGS] && GET_HIT_RL(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_LEGS];
      obj_to_obj(unequip_char(ch, WEAR_LEGS), obj);
    }				//if

    GET_HIT_LL(ch) = LIMB_DEAD;
    break;
  case LOCATION_RIGHT_FOOT:
    obj = read_object(Parts[3], VIRTUAL);
    MakeLimb(ch, obj);
    if (ch->equipment[WEAR_FEET] && GET_HIT_LF(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_FEET];
      obj_to_obj(unequip_char(ch, WEAR_FEET), obj);
    }				//if

    GET_HIT_RF(ch) = LIMB_DEAD;
    break;
  case LOCATION_LEFT_FOOT:
    obj = read_object(Parts[3], VIRTUAL);
    MakeLimb(ch, obj);
    if (ch->equipment[WEAR_FEET] && GET_HIT_RF(ch) == LIMB_DEAD) {
      ch_item = ch->equipment[WEAR_FEET];
      obj_to_obj(unequip_char(ch, WEAR_FEET), obj);
    }				//if

    GET_HIT_LF(ch) = LIMB_DEAD;
    break;
  default:
    break;
  }				//switch

  if (obj) {
    AddObjName(ch, obj);
  }				//if

  return obj;
}				//KillLimb


void MakeLimb(struct char_data *ch, struct obj_data *obj)
{
  extern int max_npc_corpse_time, max_pc_corpse_time;
  int _limb = GET_OBJ_VNUM(obj) - 60;
  float _wt = (float) GET_WEIGHT(ch) / 100.0;

  //determine weight of limbs
  switch (_limb) {
  case LIMB_HEAD:
    GET_OBJ_WEIGHT(obj) = _wt * 13;
    break;
  case LIMB_HAND:
    GET_OBJ_WEIGHT(obj) = _wt;
    break;
  case LIMB_ARM:
    GET_OBJ_WEIGHT(obj) = _wt * 5;
    break;
  case LIMB_FOOT:
    GET_OBJ_WEIGHT(obj) = _wt * 3;
    break;
  case LIMB_LEG:
    GET_OBJ_WEIGHT(obj) = _wt * 12;
    break;
  default:
    break;
  }				//switch

  GET_OBJ_TYPE(obj) = ITEM_CORPSE;
  GET_OBJ_VAL(obj, 0) = 0;	/* You can't store stuff in a corpse */

  //set decay rate
  if (IS_NPC(ch))
    GET_OBJ_TIMER(obj) = max_npc_corpse_time;
  else
    GET_OBJ_TIMER(obj) = max_pc_corpse_time;
}				//MakeLimb


void AddObjName(struct char_data *ch, struct obj_data *obj)
{
  int _race = GET_RACE(ch), _limb = GET_OBJ_VNUM(obj) - 60;
  struct extra_descr_data *temp;

  //determine the race of the limbs owner
  GET_OBJ_VAL(obj, 6) = GET_RACE(ch);

  //marking limb obj value[7] to contain the owners id/vnum
  if (IS_NPC(ch))
    GET_OBJ_VAL(obj, 7) = GET_MOB_VNUM(ch);
  else
    GET_OBJ_VAL(obj, 7) = GET_ID(ch);

  switch (_limb) {
  case LIMB_HEAD:
    CREATE(temp, struct extra_descr_data, 1);
    sprintf(buf, "This head's distinguishing features clearly resemble that of %s.\r\n"
	    "The eyes are wide open and the mouth is locked in a silent scream of death,\r\n"
	    "as blood oozes from the base of his skull.\r\n",
	    FirstWord(ch->player.name));
    temp->description = strdup(buf);
    sprintf(buf, "head");
    temp->keyword = strdup(buf);
    temp->next = obj->ex_description;
    obj->ex_description = temp;
  case LIMB_HAND:
  case LIMB_ARM:
  case LIMB_FOOT:
  case LIMB_LEG:
    sprintf(buf, "a %s %s lays here.", race_types[_race], limb_names[_limb]);
    GET_OBJ_LONG(obj) = strdup(buf);
    sprintf(buf, "a %s %s", race_types[_race], limb_names[_limb]);
    GET_OBJ_SHORT(obj) = strdup(buf);
    break;
  default:
    break;
  }				//switch

}				//AddObjName


