/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "comm.h"
#include "spells.h"
#include "utils.h"
#include "db.h"
#include "boards.h"
#include "shop.h"
#include "olc.h"
#include "handler.h"
#include "interpreter.h"
#include "dg_olc.h"

/*------------------------------------------------------------------------*/

/*
 * External variable declarations.
 */

extern struct obj_data *obj_proto;
extern struct index_data *obj_index;
extern struct obj_data *object_list;
extern int top_of_objt;
extern struct zone_data *zone_table;
extern int top_of_zone_table;
extern struct shop_data *shop_index;
extern int top_shop;
extern int num_of_cmds;
extern struct attack_hit_type attack_hit_text[];
extern char *furn_types[];
extern char *item_types[];
extern char *wear_bits[];
extern char *extra_bits[];
extern char *extra2_bits[];
extern char *anti_bits[];
extern char *drinks[];
extern char *apply_types[];
extern char *affected_bits[];
extern char *affected2_bits[];
extern char *affected3_bits[];
extern char *affected4_bits[];
extern char *protection_bits[];
extern char *detecting_bits[];
extern char *controlling_bits[];
extern char *container_bits[];
extern char *spells[];
extern char *position_types[];
extern char *activating_commands[];
extern struct board_info_type *board_info;
extern struct descriptor_data *descriptor_list;
extern char *material_names[];
extern char *craftsmanship_names[];
extern char *item_size_types[];
extern char *race_types[];
extern char *helmet_types[];
extern char *shield_types[];
extern char *armor_types[];
extern char *body_types[];
extern char *limb_types[];
extern char *extremity_types[];
extern char *weapons_types[];
extern void oedit_disp_commands_menu(struct descriptor_data *d);
extern char *kingdom_list[];
extern char *door_flags[];

/*------------------------------------------------------------------------*/

/*
 * Handy macros.
 */
#define S_PRODUCT(s, i) ((s)->producing[(i)])

/*------------------------------------------------------------------------*/

void oedit_disp_container_flags_menu(struct descriptor_data *d);
void oedit_disp_virtualdoor_flag_menu(struct descriptor_data *d);
void oedit_disp_extradesc_menu(struct descriptor_data *d);
void oedit_disp_weapon_menu(struct descriptor_data *d);
void oedit_disp_val1_menu(struct descriptor_data *d);
void oedit_disp_val2_menu(struct descriptor_data *d);
void oedit_disp_val3_menu(struct descriptor_data *d);
void oedit_disp_val4_menu(struct descriptor_data *d);
void oedit_disp_val5_menu(struct descriptor_data *d);
void oedit_disp_val6_menu(struct descriptor_data *d);
void oedit_disp_val7_menu(struct descriptor_data *d);
void oedit_disp_val8_menu(struct descriptor_data *d);
void oedit_disp_type_menu(struct descriptor_data *d);
void oedit_disp_extra_menu(struct descriptor_data *d);
void oedit_disp_extra2_menu(struct descriptor_data *d);
void oedit_disp_anti_menu(struct descriptor_data *d);
void oedit_disp_wear_menu(struct descriptor_data *d);
void oedit_disp_menu(struct descriptor_data *d);
void oedit_disp_material_menu(struct descriptor_data *d);
void oedit_disp_size_menu(struct descriptor_data *d);
void oedit_disp_coins_menu(struct descriptor_data *d);
void oedit_disp_craftsmanship_menu(struct descriptor_data *d);
void oedit_disp_race_menu(struct descriptor_data *d);
void oedit_disp_helmet_type_menu(struct descriptor_data *d);
void oedit_disp_body_type_menu(struct descriptor_data *d);
void oedit_disp_shield_type_menu(struct descriptor_data *d);
void oedit_disp_armor_type_menu(struct descriptor_data *d);
void oedit_disp_limb_type_menu(struct descriptor_data *d);
void oedit_disp_extremity_type_menu(struct descriptor_data *d);
void oedit_disp_weapon_type_menu(struct descriptor_data *d);
void oedit_disp_money_type(struct descriptor_data *d);
void oedit_disp_money_king(struct descriptor_data *d);
void oedit_disp_positions(struct descriptor_data *d);
void oedit_disp_furn_types(struct descriptor_data *d);
void oedit_parse(struct descriptor_data *d, char *arg);
void oedit_disp_spells_menu(struct descriptor_data *d);
void oedit_liquid_type(struct descriptor_data *d);
void oedit_setup_new(struct descriptor_data *d);
void oedit_setup_existing(struct descriptor_data *d, int real_num);
void oedit_save_to_disk(int zone);
void oedit_delete(struct descriptor_data *d);
void oedit_save_internally(struct descriptor_data *d);
void string_write(struct descriptor_data *d, char **writeto, size_t len, long mailto, void *data);

/*------------------------------------------------------------------------*\
  Utility and exported functions
\*------------------------------------------------------------------------*/

void oedit_setup_new(struct descriptor_data *d)
{
  CREATE(OLC_OBJ(d), struct obj_data, 1);

  clear_object(OLC_OBJ(d));
  OLC_OBJ(d)->name = str_dup("unfinished object");
  OLC_OBJ(d)->description = str_dup("An unfinished object is lying here.");
  OLC_OBJ(d)->short_description = str_dup("an unfinished object");
  GET_OBJ_ANTI(OLC_OBJ(d)) = ITEM_ALLOW_ALL;
  GET_OBJ_CRAFTSMANSHIP(OLC_OBJ(d)) = 7;
  GET_OBJ_WEAR(OLC_OBJ(d)) = ITEM_WEAR_TAKE + ITEM_WEAR_HOLD;
  OLC_VAL(d) = 0;
  GET_OBJ_SIZE(OLC_OBJ(d)) = ITEM_SIZE_MEDIUM;
  OLC_PROTO(d) = OLC_NUM(d);
  oedit_disp_menu(d);
}

/*------------------------------------------------------------------------*/

void oedit_setup_existing(struct descriptor_data *d, int real_num)
{
  struct extra_descr_data *this, *temp, *temp2;
  struct obj_data *obj;

  /*
   * Allocate object in memory.
   */
  CREATE(obj, struct obj_data, 1);

  clear_object(obj);
  *obj = obj_proto[real_num];

  /*
   * Copy all strings over.
   */
  obj->name = str_dup(obj_proto[real_num].name ? obj_proto[real_num].name : "undefined");
  obj->short_description = str_dup(obj_proto[real_num].short_description ?
		    obj_proto[real_num].short_description : "undefined");
  obj->description = str_dup(obj_proto[real_num].description ?
			  obj_proto[real_num].description : "undefined");
  obj->action_description = (obj_proto[real_num].action_description ?
		 str_dup(obj_proto[real_num].action_description) : NULL);

  /*
   * Extra descriptions if necessary.
   */
  if (obj_proto[real_num].ex_description) {
    CREATE(temp, struct extra_descr_data, 1);

    obj->ex_description = temp;
    for (this = obj_proto[real_num].ex_description; this; this = this->next) {
      temp->keyword = (this->keyword && *this->keyword) ? str_dup(this->keyword) : NULL;
      temp->description = (this->description && *this->description) ?
	  str_dup(this->description) : NULL;
      if (this->next) {
	CREATE(temp2, struct extra_descr_data, 1);
	temp->next = temp2;
	temp = temp2;
      } else
	temp->next = NULL;
    }
  }
  if (SCRIPT(obj))
    script_copy(obj, &obj_proto[real_num], OBJ_TRIGGER);

  /*
   * Attach new object to player's descriptor.
   */
  OLC_OBJ(d) = obj;
  OLC_PROTO(d) = GET_OBJ_ORIG_VNUM(OLC_OBJ(d));
  OLC_VAL(d) = 0;
  OLC_ITEM_TYPE(d) = OBJ_TRIGGER;
  dg_olc_script_copy(d);
  oedit_disp_menu(d);
}

/*------------------------------------------------------------------------*/

void oedit_delete(struct descriptor_data *d)
{
  int i, found = FALSE;
  int real_num, num_delete = 0;
  struct obj_data *obj, *new_obj_proto;
  struct index_data *new_obj_index;

  real_num = real_object(OLC_NUM(d));

  sprintf(buf, "Deleting obj (%d) %s\r\n", OLC_NUM(d), obj_proto[real_num].short_description);
  send_to_char(buf, d->character);

  for (obj = object_list; obj; obj = obj->next) {
    if (obj->item_number == real_num) {
      num_delete++;
      if (obj->carried_by)
	obj_from_char(obj);
      if (obj->worn_by) {
	unequip_char(obj->worn_by, obj->worn_on);
	affect_total(obj->worn_by);
	save_char(obj->worn_by, NOWHERE);
      }
      if (obj->in_obj)
	obj_from_obj(obj);
      if (obj->in_room)
	obj_from_room(obj);

      extract_obj(obj);

    }
  }

  CREATE(new_obj_index, struct index_data, top_of_objt);
  CREATE(new_obj_proto, struct obj_data, top_of_objt);

  for (i = 0; i <= top_of_objt; i++) {
    if (!found) {
      if (obj_index[i].virtual == OLC_NUM(d)) {
	found = TRUE;
      } else {
	new_obj_proto[i] = obj_proto[i];
	new_obj_index[i] = obj_index[i];
      }
    } else {
      new_obj_index[i - 1] = obj_index[i];
      new_obj_proto[i - 1] = obj_proto[i];
      new_obj_proto[i - 1].item_number = i - 1;
    }
  }

  FREE(obj_proto);
  FREE(obj_index);
  obj_proto = new_obj_proto;
  obj_index = new_obj_index;
  top_of_objt--;

  if (num_delete > 0) {
    sprintf(buf, "%d objects deleted from game.\r\n", num_delete);
    send_to_char(buf, d->character);
  }
  send_to_char("Saving file!\r\n", d->character);
  oedit_save_to_disk(OLC_ZNUM(d));
}

/*------------------------------------------------------------------------*/

#define ZCMD zone_table[zone].cmd[cmd_no]

void oedit_save_internally(struct descriptor_data *d)
{
  int i, shop, robj_num, found = FALSE, zone, cmd_no;
  struct extra_descr_data *this, *next_one;
  struct obj_data *obj, *swap, *new_obj_proto;
  struct index_data *new_obj_index;
  struct descriptor_data *dsc;

  /*
   * Write object to internal tables.
   */
  if ((robj_num = real_object(OLC_NUM(d))) > 0) {
    /*
     * We need to run through each and every object currently in the
     * game to see which ones are pointing to this prototype.
     * if object is pointing to this prototype, then we need to replace it
     * with the new one.
     */
    CREATE(swap, struct obj_data, 1);

    for (obj = object_list; obj; obj = obj->next) {
      if (obj->item_number == robj_num) {
	*swap = *obj;
	*obj = *OLC_OBJ(d);
	/*
	 * Copy game-time dependent variables over.
	 */
	obj->in_room = swap->in_room;
	obj->item_number = robj_num;
	obj->carried_by = swap->carried_by;
	obj->worn_by = swap->worn_by;
	obj->worn_on = swap->worn_on;
	obj->in_obj = swap->in_obj;
	obj->contains = swap->contains;
	obj->next_content = swap->next_content;
	obj->next = swap->next;
      }
    }
    free_obj(swap);
    /*
     * It is now safe to free the old prototype and write over it.
     */
    if (obj_proto[robj_num].name)
      FREE(obj_proto[robj_num].name);
    if (obj_proto[robj_num].description)
      FREE(obj_proto[robj_num].description);
    if (obj_proto[robj_num].short_description)
      FREE(obj_proto[robj_num].short_description);
    if (obj_proto[robj_num].action_description)
      FREE(obj_proto[robj_num].action_description);
    if (obj_proto[robj_num].ex_description)
      for (this = obj_proto[robj_num].ex_description; this; this = next_one) {
	next_one = this->next;
	if (this->keyword)
	  FREE(this->keyword);
	if (this->description)
	  FREE(this->description);
	FREE(this);
      }
    obj_proto[robj_num] = *OLC_OBJ(d);
    obj_proto[robj_num].item_number = robj_num;
    obj_proto[robj_num].proto_script = OLC_SCRIPT(d);
  } else {
    /*
     * It's a new object, we must build new tables to contain it.
     */
    CREATE(new_obj_index, struct index_data, top_of_objt + 2);
    CREATE(new_obj_proto, struct obj_data, top_of_objt + 2);

    /*
     * Start counting through both tables.
     */
    for (i = 0; i <= top_of_objt; i++) {
      /*
       * If we haven't found it.
       */
      if (!found) {
	/*
	 * Check if current virtual is bigger than our virtual number.
	 */
	if (obj_index[i].virtual > OLC_NUM(d)) {
	  found = TRUE;
	  robj_num = i;
	  OLC_OBJ(d)->item_number = robj_num;
	  new_obj_index[robj_num].virtual = OLC_NUM(d);
	  new_obj_index[robj_num].number = 0;
	  new_obj_index[robj_num].func = NULL;
	  new_obj_proto[robj_num] = *(OLC_OBJ(d));
	  new_obj_proto[robj_num].proto_script = OLC_SCRIPT(d);
	  new_obj_proto[robj_num].in_room = NOWHERE;
	  new_obj_proto[robj_num].obj_flags.orig_vnum = OLC_PROTO(d);
	  /*
	   * Copy over the mob that should be here.
	   */
	  new_obj_index[robj_num + 1] = obj_index[robj_num];
	  new_obj_proto[robj_num + 1] = obj_proto[robj_num];
	  new_obj_proto[robj_num + 1].item_number = robj_num + 1;
	} else {
	  /*
	   * Just copy from old to new, no number change.
	   */
	  new_obj_proto[i] = obj_proto[i];
	  new_obj_index[i] = obj_index[i];
	}
      } else {
	/*
	 * We HAVE already found it, therefore copy to object + 1 
	 */
	new_obj_index[i + 1] = obj_index[i];
	new_obj_proto[i + 1] = obj_proto[i];
	new_obj_proto[i + 1].item_number = i + 1;
      }
    }
    if (!found) {
      robj_num = i;
      OLC_OBJ(d)->item_number = robj_num;
      new_obj_index[robj_num].virtual = OLC_NUM(d);
      new_obj_index[robj_num].number = 0;
      new_obj_index[robj_num].func = NULL;
      new_obj_proto[robj_num] = *(OLC_OBJ(d));
      new_obj_proto[robj_num].proto_script = OLC_SCRIPT(d);
      new_obj_proto[robj_num].in_room = NOWHERE;
      new_obj_proto[robj_num].obj_flags.orig_vnum = OLC_PROTO(d);
    }
    /*
     * Free and replace old tables.
     */
    FREE(obj_proto);
    FREE(obj_index);
    obj_proto = new_obj_proto;
    obj_index = new_obj_index;
    top_of_objt++;

    /*
     * Renumber live objects.
     */
    for (obj = object_list; obj; obj = obj->next)
      if (GET_OBJ_RNUM(obj) >= robj_num)
	GET_OBJ_RNUM(obj)++;

    /*
     * Renumber zone table.
     */
    for (zone = 0; zone <= top_of_zone_table; zone++)
      for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++)
	switch (ZCMD.command) {
	case 'P':
	  if (ZCMD.arg3 >= robj_num)
	    ZCMD.arg3++;
	  /*
	   * No break here - drop into next case.
	   */
	case 'O':
	case 'G':
	case 'E':
	  if (ZCMD.arg1 >= robj_num)
	    ZCMD.arg1++;
	  break;
	case 'R':
	  if (ZCMD.arg2 >= robj_num)
	    ZCMD.arg2++;
	  break;
	}

    /*
     * Renumber shop produce.
     */
    for (shop = 0; shop < top_shop; shop++)
      for (i = 0; SHOP_PRODUCT(shop, i) != -1; i++)
	if (SHOP_PRODUCT(shop, i) >= robj_num)
	  SHOP_PRODUCT(shop, i)++;

    /*
     * Renumber produce in shops being edited.
     */
    for (dsc = descriptor_list; dsc; dsc = dsc->next)
      if (dsc->connected == CON_SEDIT)
	for (i = 0; S_PRODUCT(OLC_SHOP(dsc), i) != -1; i++)
	  if (S_PRODUCT(OLC_SHOP(dsc), i) >= robj_num)
	    S_PRODUCT(OLC_SHOP(dsc), i)++;

  }
  olc_add_to_save_list(zone_table[OLC_ZNUM(d)].number, OLC_SAVE_OBJ);
}

/*------------------------------------------------------------------------*/

void oedit_save_to_disk(int zone_num)
{
  int counter, counter2, realcounter;
  FILE *fp;
  struct obj_data *obj;
  struct extra_descr_data *ex_desc;

  sprintf(buf, "%s/%d.new", OBJ_PREFIX, zone_table[zone_num].number);
  if (!(fp = fopen(buf, "w+"))) {
    mudlog("SYSERR: OLC: Cannot open objects file!", BRF, LVL_BUILDER, TRUE);
    return;
  }
  /*
   * Start running through all objects in this zone.
   */
  for (counter = zone_table[zone_num].number * 100;
       counter <= zone_table[zone_num].top; counter++) {
    if ((realcounter = real_object(counter)) >= 0) {
      if ((obj = (obj_proto + realcounter))->action_description) {
	strcpy(buf1, obj->action_description);
	strip_string(buf1);
      } else
	*buf1 = '\0';

      fprintf(fp,
	      "#%d\n"
	      "%s~\n"
	      "%s~\n"
	      "%s~\n"
	      "%s~\n"
	      "%d %d %d %d %d\n"
	      "%d %d %d %d %d %d %d %d\n"
	      "%d %d %d %d %d %d %d\n"
	      "%7.2f %d %d\n",

	      GET_OBJ_VNUM(obj),
	      (obj->name && *obj->name) ? obj->name : "undefined",
	      (obj->short_description && *obj->short_description) ?
	      obj->short_description : "undefined",
	      (obj->description && *obj->description) ?
	      obj->description : "undefined",
	      buf1, (int) GET_OBJ_TYPE(obj), (int) GET_OBJ_EXTRA(obj), (int) GET_OBJ_WEAR(obj),
	      (int) GET_OBJ_EXTRA2(obj), (int) GET_OBJ_ANTI(obj),
	   GET_OBJ_VAL(obj, 0), GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2),
	   GET_OBJ_VAL(obj, 3), GET_OBJ_VAL(obj, 4), GET_OBJ_VAL(obj, 5),
	      GET_OBJ_VAL(obj, 6), GET_OBJ_VAL(obj, 7),
	   GET_OBJ_SIZE(obj), GET_OBJ_SPACE(obj), GET_OBJ_CONDITION(obj),
	      GET_OBJ_STATUS(obj), GET_OBJ_MATERIAL(obj), GET_OBJ_CRAFTSMANSHIP(obj),
	      GET_OBJ_RACE(obj),
	   GET_OBJ_WEIGHT(obj), GET_OBJ_COST(obj), GET_OBJ_ORIG_VNUM(obj)
	  );

      script_save_to_disk(fp, obj, OBJ_TRIGGER);

      /*
       * Do we have extra descriptions? 
       */
      if (obj->ex_description) {	/* Yes, save them too. */
	for (ex_desc = obj->ex_description; ex_desc; ex_desc = ex_desc->next) {
	  /*
	   * Sanity check to prevent nasty protection faults.
	   */
	  if (!*ex_desc->keyword || !*ex_desc->description) {
	    mudlog("SYSERR: OLC: oedit_save_to_disk: Corrupt ex_desc!", BRF, LVL_BUILDER, TRUE);
	    continue;
	  }
	  strcpy(buf1, ex_desc->description);
	  strip_string(buf1);
	  fprintf(fp, "E\n"
		  "%s~\n"
		  "%s~\n", ex_desc->keyword, buf1);
	}
      }
      /*
       * Do we have affects? 
       */
      for (counter2 = 0; counter2 < MAX_OBJ_AFFECT; counter2++)
	if (obj->affected[counter2].modifier)
	  fprintf(fp, "A\n"
		  "%d %d\n", obj->affected[counter2].location,
		  obj->affected[counter2].modifier);
      /* trapped item? */
      if (obj->obj_flags.trap_charge && obj->obj_flags.trap_eff)
	fprintf(fp, "X\n"
		"%d %d %d %d %d\n", obj->obj_flags.trap_eff,
		obj->obj_flags.trap_dam, obj->obj_flags.trap_charge,
		obj->obj_flags.trap_level, 0 /* future addition */ );
      /* Do we have perm affects? */
      if (obj->obj_flags.bitvector)
	fprintf(fp, "B1\n"
		"%ld\n", obj->obj_flags.bitvector);
      if (obj->obj_flags.bitvector2)
	fprintf(fp, "B2\n"
		"%ld\n", obj->obj_flags.bitvector2);
      if (obj->obj_flags.bitvector3)
	fprintf(fp, "B3\n"
		"%ld\n", obj->obj_flags.bitvector3);
      if (obj->obj_flags.bitvector4)
	fprintf(fp, "B4\n"
		"%ld\n", obj->obj_flags.bitvector4);
      if (obj->obj_flags.protection)
	fprintf(fp, "P0\n"
		"%ld\n", obj->obj_flags.protection);
      if (obj->obj_flags.detecting)
	fprintf(fp, "D0\n"
		"%ld\n", obj->obj_flags.detecting);
      if (obj->obj_flags.protection)
	fprintf(fp, "C0\n"
		"%ld\n", obj->obj_flags.controlling);
    }
  }

  /*
   * Write the final line, close the file.
   */
  fprintf(fp, "$~\n");
  fclose(fp);
  sprintf(buf2, "%s/%d.obj", OBJ_PREFIX, zone_table[zone_num].number);
  /*
   * We're fubar'd if we crash between the two lines below.
   */
  remove(buf2);
  rename(buf, buf2);

  olc_remove_from_save_list(zone_table[zone_num].number, OLC_SAVE_OBJ);
}

/**************************************************************************
 Menu functions 
 **************************************************************************/

/*
 * For container flags.
 */
void oedit_disp_container_flags_menu(struct descriptor_data *d)
{
  get_char_cols(d->character);
  sprintbit(GET_OBJ_VAL(OLC_OBJ(d), 1), container_bits, buf1);

  send_to_char("[H[J", d->character);

  sprintf(buf,
	  "%s1%s) CLOSEABLE\r\n"
	  "%s2%s) PICKPROOF\r\n"
	  "%s3%s) CLOSED\r\n"
	  "%s4%s) LOCKED\r\n"
	  "Container flags: %s%s%s\r\n"
	  "Enter flag, 0 to quit : ",
	  grn, nrm, grn, nrm, grn, nrm, grn, nrm, cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

void oedit_disp_money_king(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char(CLS(d), d->character);

  for (counter = 0; counter < NUM_KINGDOMS; counter++) {
    sprintf(buf, " %s%2d%s)%s%-20.20s %s", grn, counter, nrm, yel,
	    kingdom_list[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintf(buf, "\r\n%sEnter coin's kingdom : ", nrm);
  send_to_char(buf, d->character);
}

void oedit_disp_money_type(struct descriptor_data *d)
{
  get_char_cols(d->character);
  sprintf(buf,
	  "%s0%s) Copper\r\n"
	  "%s1%s) Silver\r\n"
	  "%s2%s) Electrum\r\n"
	  "%s3%s) Gold\r\n"
	  "%s4%s) Platinum\r\n"
	  "Enter type : ",
	  grn, nrm, grn, nrm, grn, nrm, grn, nrm, grn, nrm);
  send_to_char(buf, d->character);
}

/*
 * For extra descriptions.
 */
void oedit_disp_extradesc_menu(struct descriptor_data *d)
{
  struct extra_descr_data *extra_desc = OLC_DESC(d);

  strcpy(buf1, !extra_desc->next ? "<Not set>\r\n" : "Set.");

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  sprintf(buf,
	  "Extra desc menu\r\n"
	  "%s1%s) Keyword: %s%s\r\n"
	  "%s2%s) Description:\r\n%s%s\r\n"
	  "%s3%s) Goto next description: %s\r\n"
	  "%s0%s) Quit\r\n"
	  "Enter choice : ",

	  grn, nrm, yel, (extra_desc->keyword && *extra_desc->keyword) ? extra_desc->keyword : "<NONE>",
	  grn, nrm, yel, (extra_desc->description && *extra_desc->description) ? extra_desc->description : "<NONE>",
	  grn, nrm, buf1, grn, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_EXTRADESC_MENU;
}

/*
 * Ask for *which* apply to edit.
 */
void oedit_disp_prompt_apply_menu(struct descriptor_data *d)
{
  int counter;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < MAX_OBJ_AFFECT; counter++) {
    if (OLC_OBJ(d)->affected[counter].modifier) {
      sprinttype(OLC_OBJ(d)->affected[counter].location, apply_types, buf2);
      sprintf(buf, " %s%d%s) %+d to %s\r\n", grn, counter + 1, nrm,
	      OLC_OBJ(d)->affected[counter].modifier, buf2);
      send_to_char(buf, d->character);
    } else {
      sprintf(buf, " %s%d%s) None.\r\n", grn, counter + 1, nrm);
      send_to_char(buf, d->character);
    }
  }
  send_to_char("\r\nEnter affection to modify (0 to quit) : ", d->character);
  OLC_MODE(d) = OEDIT_PROMPT_APPLY;
}

/*
 * Ask for liquid type.
 */
void oedit_liquid_type(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_LIQ_TYPES; counter++) {
    sprintf(buf, " %s%2d%s) %s%-20.20s %s", grn, counter, nrm, yel,
	    drinks[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintf(buf, "\r\n%sEnter drink type : ", nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_VALUE_3;
}

/*
 * Display positions. (sitting, standing, etc)
 */
void oedit_disp_positions(struct descriptor_data *d)
{
  int i;

  get_char_cols(d->character);


  send_to_char("[H[J", d->character);

  for (i = 0; *position_types[i] != '\n'; i++) {
    sprintf(buf, "%s%2d%s) %s\r\n", grn, i, nrm, position_types[i]);
    send_to_char(buf, d->character);
  }
  send_to_char("Enter minimum position number : ", d->character);
}
/*
 * The actual apply to set.
 */
void oedit_disp_apply_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_APPLIES; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    apply_types[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter apply type (0 is no apply) : ", d->character);
  OLC_MODE(d) = OEDIT_APPLY;
}

/* affected_bits */
void oedit_disp_affected_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_AFF_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    affected_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector), affected_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_AFFECTED_BITS;
}


/* affected2_bits */
void oedit_disp_affected2_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_AFF2_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    affected2_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector2), affected2_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff2 flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_AFFECTED_BITS2;
}

/* affected3_bits */
void oedit_disp_affected3_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_AFF3_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    affected3_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector3), affected3_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff3 flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_AFFECTED_BITS3;
}

/* affected4_bits */
void oedit_disp_affected4_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_AFF4_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    affected4_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector4), affected4_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter aff4 flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_AFFECTED_BITS4;
}

void oedit_disp_protection_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_PROT_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    protection_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector), affected_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter protection flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_PROTECTION;
}

void oedit_disp_detecting_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_PROT_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    detecting_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector), affected_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter detection flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_DETECTING;
}

void oedit_disp_controlling_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_PROT_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    controlling_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit((OLC_OBJ(d)->obj_flags.bitvector), affected_bits, buf1);
  sprintf(buf, "\r\nCurrent flags   : %s%s%s\r\nEnter controlling flags (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_CONTROLLING;
}

/*
 * Weapon type.
 */
void oedit_disp_weapon_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ATTACK_TYPES; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    attack_hit_text[counter].singular,
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter weapon type : ", d->character);
}


/*
 * Material.
 */
void oedit_disp_material_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_MATERIALS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    material_names[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter material : ", d->character);
}

/*
 * Race.
 */
void oedit_disp_race_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < 20; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    race_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter race : ", d->character);
}

/*
 * Weapon type.
 */
void oedit_disp_weapon_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < 35; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    weapons_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter weapon type : ", d->character);
}

/*
 * Helmet type.
 */
void oedit_disp_helmet_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("^[[H^[[J", d->character);

  for (counter = 0; counter < 6; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    helmet_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter helmet type : ", d->character);
}
/*
 * Body type.
 */
void oedit_disp_body_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  send_to_char("^[[H^[[J", d->character);

  get_char_cols(d->character);
  for (counter = 0; counter < 8; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    body_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter armor type : ", d->character);
}

/*
 * Shield type
 */
void oedit_disp_shield_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;


  send_to_char("^[[H^[[J", d->character);

  get_char_cols(d->character);
  for (counter = 0; counter < 5; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    shield_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter shield type : ", d->character);
}

/*
 * Armor Type
 */
void oedit_disp_armor_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("^[[H^[[J", d->character);

  for (counter = 0; counter < 6; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    armor_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter armor type : ", d->character);
}

/*
 * Limb type.
 */
void oedit_disp_limb_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("^[[H^[[J", d->character);

  for (counter = 0; counter < 7; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    limb_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter limb type : ", d->character);
}

/*
 * Extremity type.
 */
void oedit_disp_extremity_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("^[[H^[[J", d->character);

  for (counter = 0; counter < 7; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    extremity_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter extremity type : ", d->character);
}


/*
 * Size.
 */
void oedit_disp_size_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < 11; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    item_size_types[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter size : ", d->character);
}

/*
 * Craftsmanship.
 */
void oedit_disp_craftsmanship_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < 16; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    craftsmanship_names[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter craftsmanship : ", d->character);
}

/*
 * Spell type.
 */
void oedit_disp_spells_menu(struct descriptor_data *d)
{
  int counter, columns = 0;
  char Gbuf[80], Gbuf1[80];

  *buf = '\0';
  get_char_cols(d->character);
  send_to_char("[H[J", d->character);

  for (counter = 0; counter < 8; counter++) {
    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_SCROLL:
      if (counter == 0) {
	strcpy(Gbuf, "Level: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else {
	strcpy(Gbuf, "Spell: ");
	sprintf(Gbuf1, "%s", spells[GET_OBJ_VAL(OLC_OBJ(d), counter)]);
      }
      break;
    case ITEM_WAND:
    case ITEM_STAFF:
      if (counter == 0) {
	strcpy(Gbuf, "Level: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else if (counter == 1) {
	strcpy(Gbuf, "Max Charge: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else if (counter == 2) {
	strcpy(Gbuf, "Cur Charge: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else if (counter == 3) {
	strcpy(Gbuf, "Spell: ");
	sprintf(Gbuf1, "%s", spells[GET_OBJ_VAL(OLC_OBJ(d), counter)]);
      } else {
	sprintf(Gbuf, "%d:", counter);
	strcpy(Gbuf1, "Unused");
      }
      break;
    case ITEM_POTION:
      if (counter == 0) {
	strcpy(Gbuf, "Level: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else if (counter == 1) {
	strcpy(Gbuf, "Spell: ");
	sprintf(Gbuf1, "%s", spells[GET_OBJ_VAL(OLC_OBJ(d), counter)]);
      } else if (counter == 2) {
	strcpy(Gbuf, "Doses: ");
	sprintf(Gbuf1, "%d", GET_OBJ_VAL(OLC_OBJ(d), counter));
      } else {
	sprintf(Gbuf, "%d:", counter);
	strcpy(Gbuf1, "Unused");
      }
      break;
    default:
      sprintf(Gbuf, "%d:", counter);
      strcpy(Gbuf1, "Unused");
      break;
    }
    sprintf(buf + strlen(buf), "%s%-12s%s) %s%-20.20s %s", grn, Gbuf, nrm, yel,
	    Gbuf1, !(++columns % 2) ? "\r\n&n" : "&n");
  }
  send_to_char(buf, d->character);

  sprintf(buf2, "\r\n%sEnter spells name (0 for none) : ", nrm);
  send_to_char(buf2, d->character);
}


/*
 * Object value #1
 */
void oedit_disp_val1_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_1;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_LIGHT:
    send_to_char("Illumination level (1-distant star, 100-sun): ", d->character);
    break;
  case ITEM_HOLY_SYMBOL:
    send_to_char("Type (1-Holy symbol, 2-misteltoe : ", d->character);
    break;
  case ITEM_TREASURE:
    send_to_char("Type (0-other, 1-gemstone, 2-jewelry : ", d->character);
    break;
  case ITEM_SCROLL:
  case ITEM_WAND:
  case ITEM_STAFF:
  case ITEM_POTION:
    send_to_char("Spell level : ", d->character);
    break;
  case ITEM_SHIP:
    send_to_char("Ship's Room vnum: ", d->character);
    break;
  case ITEM_WEAPON:
    oedit_disp_val2_menu(d);
    break;
  case ITEM_SCABBARD:
    send_to_char("Scabbard's weapon's DB vnum: ", d->character);
    break;
  case ITEM_ARMOR:
    if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_BODY)) {
      if (IS_SET(GET_OBJ_EXTRA(OLC_OBJ(d)), ITEM_WHOLE_BODY))
	oedit_disp_armor_type_menu(d);
      else
	oedit_disp_body_type_menu(d);
    } else if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_ARMS) || CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_LEGS))
      oedit_disp_limb_type_menu(d);
    else if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_HEAD))
      oedit_disp_helmet_type_menu(d);
    else if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_FEET) || CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_HANDS))
      oedit_disp_extremity_type_menu(d);
    else
      oedit_disp_menu(d);
    break;
  case ITEM_SHIELD:
    oedit_disp_shield_type_menu(d);
    break;
  case ITEM_STORAGE:
  case ITEM_CONTAINER:
    send_to_char("Max weight to contain : ", d->character);
    break;
  case ITEM_DRINKCON:
  case ITEM_FOUNTAIN:
    send_to_char("Max drink units : ", d->character);
    break;
  case ITEM_FOOD:
    send_to_char("Hours to fill stomach : ", d->character);
    break;
  case ITEM_MONEY:
    send_to_char("Number of coins : ", d->character);
    break;
  case ITEM_NOTE:
    send_to_char("Language note is in : ", d->character);
    break;
  case ITEM_BOARD:
    send_to_char("Enter the minimum level to read this board: ", d->character);
    break;
  case ITEM_FIREWEAPON:
    send_to_char("Range (in coordinates): ", d->character);
    break;
  case ITEM_TRAP:
    oedit_disp_spells_menu(d);
    break;
  case ITEM_TELEPORT:
  case ITEM_PIT:
  case ITEM_DOOR:
  case ITEM_MAP_CONNECTOR:
    send_to_char("Target room: ", d->character);
    break;
  case ITEM_QUIVER:
    send_to_char("Max. capacity: ", d->character);
    break;
  case ITEM_PICK:
    send_to_char("+% chance to pick: ", d->character);
    break;
  case ITEM_INSTRUMENT:
    send_to_char("Type: ", d->character);
    break;
  case ITEM_FURNITURE:
    oedit_disp_positions(d);
    break;
  case ITEM_MISSILE:
  case ITEM_KEY:
    oedit_disp_val2_menu(d);
    break;
  case ITEM_BOOK:
    oedit_disp_val3_menu(d);
    break;
  case ITEM_SWITCH:
    oedit_disp_commands_menu(d);
    break;
  default:
    oedit_disp_menu(d);
  }
}

/*
 * Object value #2
 */
void oedit_disp_val2_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_2;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_LIGHT:
    send_to_char("Shape (1-sphere, 2-cone, 3-line) : ", d->character);
    break;
  case ITEM_TREASURE:
    send_to_char("Value %1-100 (-1 being totally random): ", d->character);
    break;
  case ITEM_SCROLL:
  case ITEM_POTION:
    oedit_disp_spells_menu(d);
    break;
  case ITEM_WAND:
  case ITEM_STAFF:
    send_to_char("Max number of charges : ", d->character);
    break;
  case ITEM_SHIP:
    send_to_char("Ship Number: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_MISSILE:
    send_to_char("Number of damage dice (SM): ", d->character);
    break;
  case ITEM_FOOD:
    /*
     * Values 2 and 3 are unused, jump to 4...Odd.
     */
    oedit_disp_val4_menu(d);
    break;
  case ITEM_STORAGE:
  case ITEM_CONTAINER:
  case ITEM_QUIVER:
    /*
     * These are flags, needs a bit of special handling.
     */
    oedit_disp_container_flags_menu(d);
    break;
  case ITEM_DRINKCON:
  case ITEM_FOUNTAIN:
    send_to_char("Initial drink units : ", d->character);
    break;
  case ITEM_BOARD:
    send_to_char("Minimum level to write: ", d->character);
    break;
  case ITEM_FIREWEAPON:
    send_to_char("Rate of fire: ", d->character);
    break;
  case ITEM_TRAP:
    send_to_char("Damage: ", d->character);
    break;
  case ITEM_TELEPORT:
    send_to_char("Command: ", d->character);
    break;
  case ITEM_PICK:
  case ITEM_INSTRUMENT:
  case ITEM_KEY:
    send_to_char("% chance to break: ", d->character);
    break;
  case ITEM_FURNITURE:
    send_to_char("Capacity :", d->character);
    break;
  case ITEM_MONEY:
    oedit_disp_money_type(d);
    break;
  case ITEM_SWITCH:
    send_to_char("Which room is the blocked door in? : ", d->character);
    break;
  case ITEM_PIT:
    send_to_char("Auto open(0) or weight activated(1)? : ", d->character);
    break;
  case ITEM_DOOR:
    send_to_char("X cord they will be sent to? : ", d->character);
    break;
  case ITEM_MAP_CONNECTOR:
    send_to_char("Direction this object connects?\n0 = north, 1 = south, 3 = west, 4 = east, 5 = up, 6 = down : ", d->character);
    break;
  case ITEM_ARMOR:
    send_to_char("Armor Class value? : ", d->character);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object value #3
 */
void oedit_disp_val3_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_3;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_LIGHT:
    send_to_char("Number of hours (0 = burnt, -1 is infinite) : ", d->character);
    break;
  case ITEM_SCROLL:
    oedit_disp_spells_menu(d);
    break;
  case ITEM_POTION:
    send_to_char("Number of doses? (1-6 [-1 random]) : ", d->character);
    break;
  case ITEM_WAND:
  case ITEM_STAFF:
    send_to_char("Number of charges remaining : ", d->character);
    break;
  case ITEM_SHIP:
    send_to_char("Ship Weight 0-255: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_MISSILE:
    send_to_char("Size of damage dice (SM): ", d->character);
    break;
  case ITEM_CONTAINER:
  case ITEM_STORAGE:
    send_to_char("Vnum of key to open container (-1 for no key) : ", d->character);
    break;
  case ITEM_DRINKCON:
  case ITEM_FOUNTAIN:
    oedit_liquid_type(d);
    break;
  case ITEM_BOARD:
    send_to_char("Minimum level to remove messages: ", d->character);
    break;
  case ITEM_BOOK:
    send_to_char("Number of pages: ", d->character);
    break;
  case ITEM_TELEPORT:
    send_to_char("# charges: ", d->character);
    break;
  case ITEM_QUIVER:
    send_to_char("Missile type (1st vnum, 5 counted from there): ", d->character);
    break;
  case ITEM_INSTRUMENT:
    send_to_char("Minimum level: ", d->character);
    break;
  case ITEM_FIREWEAPON:
    oedit_disp_val4_menu(d);
    break;
  case ITEM_SWITCH:
    send_to_char("Which exit direction will this activate? : ", d->character);
    break;
  case ITEM_MONEY:
    oedit_disp_money_king(d);
    break;
  case ITEM_PIT:
    send_to_char("Weight before it opens? : ", d->character);
    break;
  case ITEM_DOOR:
    send_to_char("Y cord they will be sent to? : ", d->character);
    break;
  case ITEM_FURNITURE:
    send_to_char("Regen gain :", d->character);
    break;
  case ITEM_MAP_CONNECTOR:
    send_to_char("Offset to north/south coords? (0 for none) :", d->character);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object value #4
 */
void oedit_disp_val4_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_4;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_SCROLL:
  case ITEM_WAND:
  case ITEM_STAFF:
    oedit_disp_spells_menu(d);
    break;
  case ITEM_SHIP:
    send_to_char("Enter a value, but it wont be used: ", d->character);
    break;
  case ITEM_WEAPON:
    oedit_disp_weapon_menu(d);
    break;
  case ITEM_DRINKCON:
  case ITEM_FOUNTAIN:
  case ITEM_FOOD:
    send_to_char("Poisoned (0 = not poison) : ", d->character);
    break;
  case ITEM_BOARD:
    Board_save_board(GET_OBJ_VNUM(OLC_OBJ(d)));
    break;
  case ITEM_FIREWEAPON:
  case ITEM_MISSILE:
    send_to_char("Missile type: ", d->character);
    break;
  case ITEM_QUIVER:
    send_to_char("Current #: ", d->character);
    break;
  case ITEM_SWITCH:
    send_to_char("0 = Door moves, 1 = switch moves : ", d->character);
    break;
  case ITEM_DOOR:
    send_to_char("Z cord they will be sent to? : ", d->character);
    break;
  case ITEM_FURNITURE:
    send_to_char("Char use (-1 = Mob only) :", d->character);
    break;
  case ITEM_MAP_CONNECTOR:
    send_to_char("Offset to east/west coords? (0 for none) :", d->character);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}


/*
 * Object value #5
 */
void oedit_disp_val5_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_5;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_SHIP:
    send_to_char("Enter a value, but it wont be used: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_MISSILE:
    send_to_char("Number of damage dice (L): ", d->character);
    break;
  case ITEM_CONTAINER:
  case ITEM_STORAGE:
    send_to_char("Max space: ", d->character);
    break;
  case ITEM_SCROLL:
    oedit_disp_spells_menu(d);
    break;
  case ITEM_FIREWEAPON:
    oedit_disp_val7_menu(d);
    break;
  case ITEM_DOOR:
    oedit_disp_virtualdoor_flag_menu(d);
    break;
  case ITEM_FURNITURE:
    oedit_disp_furn_types(d);
    break;
  case ITEM_MAP_CONNECTOR:
    send_to_char("Offset to up/down coords? (0 for none) :", d->character);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object value #6
 */
void oedit_disp_val6_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_6;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_SHIP:
    send_to_char("Enter a value, but it wont be used: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_MISSILE:
    send_to_char("Size of damage dice (L): ", d->character);
    break;
  case ITEM_SCROLL:
    oedit_disp_spells_menu(d);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object value #7
 */
void oedit_disp_val7_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_7;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_SHIP:
    send_to_char("Enter a value, but it wont be used: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_MISSILE:
  case ITEM_FIREWEAPON:
    send_to_char("Speed: ", d->character);
    break;
  case ITEM_SCROLL:
    oedit_disp_spells_menu(d);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object value #8
 */
void oedit_disp_val8_menu(struct descriptor_data *d)
{
  OLC_MODE(d) = OEDIT_VALUE_8;
  switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
  case ITEM_SHIP:
    send_to_char("Enter a value, but it wont be used: ", d->character);
    break;
  case ITEM_WEAPON:
  case ITEM_FIREWEAPON:
    oedit_disp_weapon_type_menu(d);
    break;
  default:
    OLC_VAL(d) = 1;
    oedit_disp_menu(d);
  }
}

/*
 * Object type.
 */
void oedit_disp_type_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ITEM_TYPES; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    item_types[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter object type : ", d->character);
}

/*
 * Object extra flags.
 */
void oedit_disp_extra_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ITEM_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    extra_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(GET_OBJ_EXTRA(OLC_OBJ(d)), extra_bits, buf1);
  sprintf(buf, "\r\nObject flags: %s%s%s\r\n"
	  "Enter object extra flag (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*
 * Object extra2 flags.
 */
void oedit_disp_extra2_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ITEM2_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    extra2_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(GET_OBJ_EXTRA2(OLC_OBJ(d)), extra2_bits, buf1);
  sprintf(buf, "\r\nObject extra2 flags: %s%s%s\r\n"
	  "Enter object extra2 flag (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*
 * Object anti flags.
 */
void oedit_disp_anti_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ANTI_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    anti_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(GET_OBJ_ANTI(OLC_OBJ(d)), anti_bits, buf1);
  sprintf(buf, "\r\nObject anti: %s%s%s\r\n"
	  "Enter object anti (0 to quit) : ",
	  cyn, buf1, nrm);
  send_to_char(buf, d->character);
}


/*
 * Object wear flags.
 */
void oedit_disp_wear_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ITEM_WEARS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    wear_bits[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintbit(GET_OBJ_WEAR(OLC_OBJ(d)), wear_bits, buf1);
  sprintf(buf, "\r\nWear flags: %s%s%s\r\n"
	  "Enter wear flag, 0 to quit : ", cyn, buf1, nrm);
  send_to_char(buf, d->character);
}

/*
 * Display main menu.
 */
void oedit_disp_menu(struct descriptor_data *d)
{
  struct obj_data *obj;

  obj = OLC_OBJ(d);
  get_char_cols(d->character);

  /*
   * Build buffers for first part of menu.
   */
  sprinttype(GET_OBJ_TYPE(obj), item_types, buf1);
  sprintbit(GET_OBJ_EXTRA(obj), extra_bits, buf2);

  /*
   * Build first part of menu.
   */
  sprintf(buf,

	  "[H[J"
	  "-- Item number : [%s%d%s]\r\n"
	  "-- Copied from : [%s%d%s]\r\n"
	  "%s1%s) Namelist : %s%s\r\n"
	  "%s2%s) S-Desc   : %s%s\r\n"
	  "%s4%s) L-Desc   :-\r\n%s%s\r\n"
	  "%s5%s) A-Desc   :-\r\n%s%s"
	  "%s6%s) Type          : %s%s\r\n"
	  "%s7%s) Extra flags   : %s%s\r\n",

	  cyn, OLC_NUM(d), nrm, cyn, OLC_PROTO(d), nrm,
      grn, nrm, nrm, (obj->name && *obj->name) ? obj->name : "undefined",
	  grn, nrm, nrm, (obj->short_description && *obj->short_description) ? obj->short_description : "undefined",
	  grn, nrm, nrm, (obj->description && *obj->description) ? obj->description : "undefined",
	  grn, nrm, nrm, (obj->action_description && *obj->action_description) ? obj->action_description : "<not set>\r\n",
	  grn, nrm, cyn, buf1,
	  grn, nrm, cyn, buf2
      );
  /*
   * Send first part.
   */
  send_to_char(buf, d->character);

  sprintbit(GET_OBJ_EXTRA2(obj), extra2_bits, buf1);

  sprintf(buf,
	  "%s8%s) Extra2 flags  : %s%s\r\n",
	  grn, nrm, cyn, buf1
      );
  /*
   * Send second part.
   */
  send_to_char(buf, d->character);

  sprintbit(GET_OBJ_ANTI(obj), anti_bits, buf1);

  sprintf(buf,
	  "%s9%s) Anti flags    : %s%s\r\n",
	  grn, nrm, cyn, buf1
      );
  /*
   * Send second part.
   */
  send_to_char(buf, d->character);

  /*
   * Build third part of menu.
   */
  sprintbit(GET_OBJ_WEAR(obj), wear_bits, buf1);
  sprintf(buf,
	  "%sA%s) Wear flags    : %s%s\r\n"
	  "%sB%s) Weight        : %s%7.2f\r\n"
	  "%sC%s) Size          : %s%s\r\n"
	  "%sD%s) Condition     : %s%d\r\n"
	  "%sE%s) Status        : %s%d\r\n"
	  "%sF%s) Material      : %s%s\r\n"
	  "%sG%s) Craftsmanship : %s%s\r\n"
	  "%sH%s) Race          : %s%s\r\n"
	  "%sI%s) Cost (copper) : %s%d\r\n"
	  "%sJ%s) Timer         : %s%d\r\n"
	  "%sK%s) Values        : %s%d %d %d %d %d %d %d %d\r\n"
	  "%sL%s) Applies menu\r\n"
	  "%sM%s) Affected Bits I   %sN%s) Affected Bits II\r\n"
	  "%sO%s) Affected Bits III %sP%s) Affected Bits menu\r\n"
	  "%sR%s) Extra descriptions menu\r\n"
	  "%sS%s) AI Script     : %s%s\r\n"
	  "%s+%s) Edit Next         %s-%s) Edit Previous\r\n"
	  "%sQ%s) Quit\r\n"
	  "Enter choice : ",
	  grn, nrm, cyn, buf1,
	  grn, nrm, cyn, GET_OBJ_WEIGHT(obj),
	  grn, nrm, cyn, item_size_types[(int) GET_OBJ_SIZE(obj)],
	  grn, nrm, cyn, GET_OBJ_CONDITION(obj),
	  grn, nrm, cyn, GET_OBJ_STATUS(obj),
	  grn, nrm, cyn, material_names[(int) GET_OBJ_MATERIAL(obj)],
    grn, nrm, cyn, craftsmanship_names[(int) GET_OBJ_CRAFTSMANSHIP(obj)],
	  grn, nrm, cyn, race_types[(int) GET_OBJ_RACE(obj)],
	  grn, nrm, cyn, GET_OBJ_COST(obj),
	  grn, nrm, cyn, GET_OBJ_TIMER(obj),
	  grn, nrm, cyn, GET_OBJ_VAL(obj, 0),
	  GET_OBJ_VAL(obj, 1),
	  GET_OBJ_VAL(obj, 2),
	  GET_OBJ_VAL(obj, 3),
	  GET_OBJ_VAL(obj, 4),
	  GET_OBJ_VAL(obj, 5),
	  GET_OBJ_VAL(obj, 6),
	  GET_OBJ_VAL(obj, 7),
	  grn, nrm, grn, nrm, grn, nrm, grn, nrm, grn, nrm, grn, nrm,
	  grn, nrm, cyn, obj->proto_script ? "Set." : "Not Set.",
	  grn, nrm, grn, nrm,
	  grn, nrm);
  send_to_char(buf, d->character);
  OLC_MODE(d) = OEDIT_MAIN_MENU;
}

/***************************************************************************
 main loop (of sorts).. basically interpreter throws all input to here
 ***************************************************************************/

void oedit_parse(struct descriptor_data *d, char *arg)
{
  int number, max_val, min_val, real_num, counter;

  switch (OLC_MODE(d)) {

  case OEDIT_CONFIRM_DELETE:
    switch (*arg) {
    case 'y':
    case 'Y':
      oedit_delete(d);
      sprintf(buf, "OLC: %s deleted obj %d", GET_NAME(d->character), OLC_NUM(d));
      mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(d->character)), TRUE);
      cleanup_olc(d, CLEANUP_STRUCTS);
      return;
    default:
      cleanup_olc(d, CLEANUP_ALL);
      return;
    }
    break;

  case OEDIT_CONFIRM_SAVESTRING:
    switch (*arg) {
    case 'y':
    case 'Y':
      send_to_char("Saving object to memory.\r\n", d->character);
      oedit_save_internally(d);
      sprintf(buf, "OLC: %s edits obj %d", GET_NAME(d->character), OLC_NUM(d));
      mudlog(buf, CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(d->character)), TRUE);
      cleanup_olc(d, CLEANUP_STRUCTS);
      return;
    case 'n':
    case 'N':
      /*
       * Cleanup all.
       */
      cleanup_olc(d, CLEANUP_ALL);
      return;
    default:
      send_to_char("Invalid choice!\r\n", d->character);
      send_to_char("Do you wish to save this object internally?\r\n", d->character);
      return;
    }

  case OEDIT_MAIN_MENU:
    /*
     * Throw us out to whichever edit mode based on user input.
     */
    switch (*arg) {
    case 'q':
    case 'Q':
      if (OLC_VAL(d)) {		/* Something has been modified. */
	send_to_char("Do you wish to save this object internally? : ", d->character);
	OLC_MODE(d) = OEDIT_CONFIRM_SAVESTRING;
      } else
	cleanup_olc(d, CLEANUP_ALL);
      return;
    case '+':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      number = OLC_NUM(d) + 1;
      oedit_save_internally(d);
      cleanup_olc(d, CLEANUP_ALL);
      OLC_NUM(d) = number;
      if ((real_num = real_object(number)) >= 0)
	oedit_setup_existing(d, real_num);
      else
	oedit_setup_new(d);
      STATE(d) = CON_OEDIT;
      break;
    case '-':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      number = OLC_NUM(d) - 1;
      oedit_save_internally(d);
      cleanup_olc(d, CLEANUP_ALL);
      OLC_NUM(d) = number;
      if ((real_num = real_object(number)) >= 0)
	oedit_setup_existing(d, real_num);
      else
	oedit_setup_new(d);
      STATE(d) = CON_OEDIT;
      break;
    case '1':
      send_to_char("Enter namelist : ", d->character);
      OLC_MODE(d) = OEDIT_EDIT_NAMELIST;
      break;
    case '2':
      send_to_char("Enter short desc : ", d->character);
      OLC_MODE(d) = OEDIT_SHORTDESC;
      break;
    case '4':
      send_to_char("Enter long desc :-\r\n| ", d->character);
      OLC_MODE(d) = OEDIT_LONGDESC;
      break;
    case '5':
      OLC_MODE(d) = OEDIT_ACTDESC;
      SEND_TO_Q("Enter action description (Never used or saved): (/s saves /h for help)\r\n\r\n", d);
      d->backstr = NULL;
      if (OLC_OBJ(d)->action_description) {
	SEND_TO_Q(OLC_OBJ(d)->action_description, d);
	d->backstr = str_dup(OLC_OBJ(d)->action_description);
      }
#ifndef MUDFTP
      d->str = &OLC_OBJ(d)->action_description;
      d->max_str = MAX_MESSAGE_LENGTH;
      d->mail_to = 0;
#else
      string_write(d, &OLC_OBJ(d)->action_description, MAX_MESSAGE_LENGTH, 0, NULL);
#endif
      OLC_VAL(d) = 1;
      break;
    case '6':
      oedit_disp_type_menu(d);
      OLC_MODE(d) = OEDIT_TYPE;
      break;
    case '7':
      oedit_disp_extra_menu(d);
      OLC_MODE(d) = OEDIT_EXTRAS;
      break;
    case '8':
      oedit_disp_extra2_menu(d);
      OLC_MODE(d) = OEDIT_EXTRAS2;
      break;
    case '9':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_anti_menu(d);
      OLC_MODE(d) = OEDIT_ANTI;
      break;
    case 'a':
    case 'A':
      oedit_disp_wear_menu(d);
      OLC_MODE(d) = OEDIT_WEAR;
      break;
    case 'b':
    case 'B':
      send_to_char("Enter weight : ", d->character);
      OLC_MODE(d) = OEDIT_WEIGHT;
      break;
    case 'c':
    case 'C':
      oedit_disp_size_menu(d);
      OLC_MODE(d) = OEDIT_SIZE;
      break;
    case 'd':
    case 'D':
      send_to_char("Enter condition : ", d->character);
      OLC_MODE(d) = OEDIT_CONDITION;
      break;
    case 'e':
    case 'E':
      send_to_char("Enter status : ", d->character);
      OLC_MODE(d) = OEDIT_STATUS;
      break;
    case 'f':
    case 'F':
      oedit_disp_material_menu(d);
      OLC_MODE(d) = OEDIT_MATERIAL;
      break;
    case 'g':
    case 'G':
      oedit_disp_craftsmanship_menu(d);
      OLC_MODE(d) = OEDIT_CRAFTSMANSHIP;
      break;
    case 'h':
    case 'H':
      oedit_disp_race_menu(d);
      OLC_MODE(d) = OEDIT_RACE;
      break;
    case 'i':
    case 'I':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      send_to_char("Enter cost (copper): ", d->character);
      OLC_MODE(d) = OEDIT_COST;
      break;
    case 'j':
    case 'J':
      send_to_char("Enter timer : ", d->character);
      OLC_MODE(d) = OEDIT_TIMER;
      break;
    case 'k':
    case 'K':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	vlogf("%s attempted to edit an items values!", GET_NAME(d->character));
	return;
      }
      /*
       * Clear any old values  
       */
      GET_OBJ_VAL(OLC_OBJ(d), 0) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 1) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 2) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 3) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 4) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 5) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 6) = 0;
      GET_OBJ_VAL(OLC_OBJ(d), 7) = 0;
      oedit_disp_val1_menu(d);
      break;
    case 'l':
    case 'L':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_prompt_apply_menu(d);
      break;
    case 'm':
    case 'M':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_affected_menu(d);
      break;
    case 'n':
    case 'N':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_affected2_menu(d);
      break;
    case 'o':
    case 'O':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_affected3_menu(d);
      break;
    case 'p':
    case 'P':
      if (GET_LEVEL(d->character) < LVL_GOD5) {
	oedit_disp_menu(d);
	return;
      }
      oedit_disp_affected4_menu(d);
      break;
    case 'r':
    case 'R':
      /*
       * If extra descriptions don't exist.
       */
      if (!OLC_OBJ(d)->ex_description) {
	CREATE(OLC_OBJ(d)->ex_description, struct extra_descr_data, 1);
	OLC_OBJ(d)->ex_description->keyword = str_dup(OLC_OBJ(d)->name);
	OLC_OBJ(d)->ex_description->next = NULL;
      }
      OLC_DESC(d) = OLC_OBJ(d)->ex_description;
      oedit_disp_extradesc_menu(d);
      break;
    case 's':
    case 'S':
      OLC_SCRIPT_EDIT_MODE(d) = SCRIPT_MAIN_MENU;
      dg_script_menu(d);
      return;
    default:
      oedit_disp_menu(d);
      break;
    }
    return;			/*
				 * end of OEDIT_MAIN_MENU 
				 */

  case OLC_SCRIPT_EDIT:
    if (dg_script_edit_parse(d, arg))
      return;
    break;

  case OEDIT_EDIT_NAMELIST:
    if (OLC_OBJ(d)->name)
      FREE(OLC_OBJ(d)->name);
    OLC_OBJ(d)->name = str_dup((arg && *arg) ? arg : "undefined");
    break;

  case OEDIT_SHORTDESC:
    if (OLC_OBJ(d)->short_description)
      FREE(OLC_OBJ(d)->short_description);
    OLC_OBJ(d)->short_description = str_dup((arg && *arg) ? arg : "undefined");
    break;

  case OEDIT_LONGDESC:
    if (OLC_OBJ(d)->description)
      FREE(OLC_OBJ(d)->description);
    OLC_OBJ(d)->description = str_dup((arg && *arg) ? arg : "undefined");
    break;

  case OEDIT_TYPE:
    number = atoi(arg);
    if ((number < 1) || (number >= NUM_ITEM_TYPES)) {
      send_to_char("Invalid choice, try again : ", d->character);
      return;
    } else
      GET_OBJ_TYPE(OLC_OBJ(d)) = number;
    break;

  case OEDIT_EXTRAS:
    number = atoi(arg);
    if ((number < 0) || (number > NUM_ITEM_FLAGS)) {
      oedit_disp_extra_menu(d);
      return;
    } else if (number == 0)
      break;
    else {
      TOGGLE_BIT(GET_OBJ_EXTRA(OLC_OBJ(d)), 1 << (number - 1));
      oedit_disp_extra_menu(d);
      return;
    }

  case OEDIT_EXTRAS2:
    number = atoi(arg);
    if ((number < 0) || (number > NUM_ITEM2_FLAGS)) {
      oedit_disp_extra2_menu(d);
      return;
    } else if (number == 0)
      break;
    else {
      TOGGLE_BIT(GET_OBJ_EXTRA2(OLC_OBJ(d)), 1 << (number - 1));
      oedit_disp_extra2_menu(d);
      return;
    }

  case OEDIT_ANTI:
    number = atoi(arg);
    if ((number < 0) || (number > NUM_ANTI_FLAGS)) {
      oedit_disp_anti_menu(d);
      return;
    } else if (number == 0)
      break;
    else {
      TOGGLE_BIT(GET_OBJ_ANTI(OLC_OBJ(d)), 1 << (number - 1));
      oedit_disp_anti_menu(d);
      return;
    }

  case OEDIT_WEAR:
    number = atoi(arg);
    if ((number < 0) || (number > NUM_ITEM_WEARS)) {
      send_to_char("That's not a valid choice!\r\n", d->character);
      oedit_disp_wear_menu(d);
      return;
    } else if (number == 0)	/* Quit. */
      break;
    else {
      TOGGLE_BIT(GET_OBJ_WEAR(OLC_OBJ(d)), 1 << (number - 1));
      oedit_disp_wear_menu(d);
      return;
    }

  case OEDIT_WEIGHT:
    GET_OBJ_WEIGHT(OLC_OBJ(d)) = atof(arg);
    break;

  case OEDIT_SIZE:
    GET_OBJ_SIZE(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_SPACE:
    GET_OBJ_SPACE(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_CONDITION:
    GET_OBJ_CONDITION(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_STATUS:
    GET_OBJ_STATUS(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_MATERIAL:
    number = atoi(arg);
    if ((number < 1) || (number >= NUM_MATERIALS)) {
      send_to_char("Invalid choice, try again : ", d->character);
      return;
    } else
      GET_OBJ_MATERIAL(OLC_OBJ(d)) = number;
    break;

  case OEDIT_CRAFTSMANSHIP:
    GET_OBJ_CRAFTSMANSHIP(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_RACE:
    GET_OBJ_RACE(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_COST:
    GET_OBJ_COST(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_TIMER:
    GET_OBJ_TIMER(OLC_OBJ(d)) = atoi(arg);
    break;

  case OEDIT_VALUE_1:
    /*
     * Lucky, I don't need to check any of these for out of range values.
     * Hmm, I'm not so sure - Rv  
     * Putting check for armor types here - Granor
     * And shield types -Granor
     */
    number = atoi(arg);
    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_ARMOR:
      if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_BODY)) {
	if (IS_SET(GET_OBJ_EXTRA(OLC_OBJ(d)), ITEM_WHOLE_BODY) && !IN_RANGE(0, number, 5))
	  oedit_disp_armor_type_menu(d);
	else if (!IN_RANGE(0, number, 7))
	  oedit_disp_body_type_menu(d);
	else {
	  GET_OBJ_VAL(OLC_OBJ(d), 0) = number;
	  oedit_disp_val2_menu(d);
	}
      } else if (!IN_RANGE(0, number, 6) &&
		 (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_ARMS) || CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_LEGS)))
	oedit_disp_limb_type_menu(d);
      else if (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_HEAD) && !IN_RANGE(0, number, 5))
	oedit_disp_helmet_type_menu(d);
      else if (!IN_RANGE(0, number, 6) &&
	       (CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_FEET) || CAN_WEAR(OLC_OBJ(d), ITEM_WEAR_HANDS)))
	oedit_disp_extremity_type_menu(d);
      else {
	GET_OBJ_VAL(OLC_OBJ(d), 0) = number;
	oedit_disp_val2_menu(d);
      }
      break;
    case ITEM_SHIELD:
      if (!IN_RANGE(0, number, 4)) {
	oedit_disp_shield_type_menu(d);
	break;
      } else;
    default:
      GET_OBJ_VAL(OLC_OBJ(d), 0) = number;
      /*
       * proceed to menu 2
       */
      oedit_disp_val2_menu(d);
    }
    return;
  case OEDIT_VALUE_2:
    /*
     * Here, I do need to check for out of range values.
     */
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_LIGHT:
      if (number < 1 || number > 3)
	oedit_disp_val2_menu(d);
      else {
	GET_OBJ_VAL(OLC_OBJ(d), 1) = number;
	oedit_disp_val3_menu(d);
      }
      break;
    case ITEM_SCROLL:
    case ITEM_POTION:
      if (number < 0 || number >= NUM_SPELLS)
	oedit_disp_val2_menu(d);
      else {
	GET_OBJ_VAL(OLC_OBJ(d), 1) = number;
	oedit_disp_val3_menu(d);
      }
      break;
    case ITEM_CONTAINER:
    case ITEM_STORAGE:
      /*
       * Needs some special handling since we are dealing with flag values
       * here.
       */
      if (number < 0 || number > 4)
	oedit_disp_container_flags_menu(d);
      else if (number != 0) {
	TOGGLE_BIT(GET_OBJ_VAL(OLC_OBJ(d), 1), 1 << (number - 1));
	OLC_VAL(d) = 1;
	oedit_disp_val2_menu(d);
      } else
	oedit_disp_val3_menu(d);
      break;
    case ITEM_ARMOR:
      GET_OBJ_VAL(OLC_OBJ(d), 1) = number;
      oedit_disp_menu(d);
      break;
    default:
      GET_OBJ_VAL(OLC_OBJ(d), 1) = number;
      oedit_disp_val3_menu(d);
    }
    return;

  case OEDIT_VALUE_3:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);
    /*
     * Quick'n'easy error checking.
     */
    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_SCROLL:
      min_val = 0;
      max_val = NUM_SPELLS - 1;
      break;
    case ITEM_POTION:
      if (number == -1)
	number = dice(1, 6);
      else if (number < 1 || number >= 6)
	oedit_disp_val2_menu(d);
      else {
	for (counter = 1; counter <= number; counter++)
	  GET_OBJ_VAL(OLC_OBJ(d), counter) = GET_OBJ_VAL(OLC_OBJ(d), 1);
	oedit_disp_menu(d);
      }
      break;
    case ITEM_WEAPON:
      min_val = 1;
      max_val = 50;
    case ITEM_WAND:
    case ITEM_STAFF:
      min_val = 0;
      max_val = 20;
      break;
    case ITEM_DRINKCON:
    case ITEM_FOUNTAIN:
      min_val = 0;
      max_val = NUM_LIQ_TYPES - 1;
      break;
    case ITEM_SWITCH:
      min_val = 0;
      max_val = NUM_OF_DIRS;
      break;
    default:
      min_val = -32000;
      max_val = 32000;
    }
    GET_OBJ_VAL(OLC_OBJ(d), 2) = MAX(min_val, MIN(number, max_val));
    oedit_disp_val4_menu(d);
    return;

  case OEDIT_VALUE_4:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_SCROLL:
      min_val = 0;
      max_val = NUM_SPELLS - 1;
      break;
    case ITEM_WAND:
    case ITEM_STAFF:
      min_val = 1;
      max_val = NUM_SPELLS - 1;
      break;
    case ITEM_WEAPON:
      min_val = 0;
      max_val = NUM_ATTACK_TYPES - 1;
      break;
    default:
      min_val = -32000;
      max_val = 32000;
      break;
    }
    GET_OBJ_VAL(OLC_OBJ(d), 3) = MAX(min_val, MIN(number, max_val));
    oedit_disp_val5_menu(d);
    return;

  case OEDIT_VALUE_5:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    case ITEM_DOOR:
      if (number != 0) {
	TOGGLE_BIT(GET_OBJ_VAL(OLC_OBJ(d), 4), 1 << (number - 1));
	OLC_VAL(d) = 1;
	oedit_disp_virtualdoor_flag_menu(d);
      } else
	oedit_disp_val6_menu(d);
      break;
    default:
      min_val = -32000;
      max_val = 32000;
      break;
    }
    oedit_disp_val6_menu(d);
    return;

  case OEDIT_VALUE_6:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    default:
      min_val = -32000;
      max_val = 32000;
      break;
    }
    GET_OBJ_VAL(OLC_OBJ(d), 5) = MAX(min_val, MIN(number, max_val));
    oedit_disp_val7_menu(d);
    return;

  case OEDIT_VALUE_7:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    default:
      min_val = -32000;
      max_val = 32000;
      break;
    }
    GET_OBJ_VAL(OLC_OBJ(d), 6) = MAX(min_val, MIN(number, max_val));
    oedit_disp_val8_menu(d);
    return;

  case OEDIT_VALUE_8:
    if (isdigit(*arg))
      number = atoi(arg);
    else
      number = find_spells_num(arg);

    switch (GET_OBJ_TYPE(OLC_OBJ(d))) {
    default:
      min_val = -32000;
      max_val = 32000;
      break;
    }
    GET_OBJ_VAL(OLC_OBJ(d), 7) = MAX(min_val, MIN(number, max_val));
    break;

  case OEDIT_AFFECTED_BITS:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_AFF_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.bitvector, 1 << (number - 1));
    oedit_disp_affected_menu(d);
    return;

  case OEDIT_AFFECTED_BITS2:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_AFF2_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.bitvector2, 1 << (number - 1));
    oedit_disp_affected2_menu(d);
    return;

  case OEDIT_AFFECTED_BITS3:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_AFF3_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.bitvector3, 1 << (number - 1));
    oedit_disp_affected3_menu(d);
    return;

  case OEDIT_AFFECTED_BITS4:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_AFF4_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.bitvector4, 1 << (number - 1));
    oedit_disp_affected4_menu(d);
    return;

  case OEDIT_PROTECTION:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_PROT_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.protection, 1 << (number - 1));
    oedit_disp_protection_menu(d);
    return;

  case OEDIT_DETECTING:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_DETE_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.detecting, 1 << (number - 1));
    oedit_disp_detecting_menu(d);
    return;

  case OEDIT_CONTROLLING:
    if ((number = atoi(arg)) == 0)
      break;
    else if (!((number < 0) || (number > NUM_CONT_FLAGS)))
      TOGGLE_BIT(OLC_OBJ(d)->obj_flags.controlling, 1 << (number - 1));
    oedit_disp_controlling_menu(d);
    return;

  case OEDIT_PROMPT_APPLY:
    if ((number = atoi(arg)) == 0)
      break;
    else if (number < 0 || number > MAX_OBJ_AFFECT) {
      oedit_disp_prompt_apply_menu(d);
      return;
    }
    OLC_VAL(d) = number - 1;
    OLC_MODE(d) = OEDIT_APPLY;
    oedit_disp_apply_menu(d);
    return;

  case OEDIT_APPLY:
    if ((number = atoi(arg)) == 0) {
      OLC_OBJ(d)->affected[OLC_VAL(d)].location = 0;
      OLC_OBJ(d)->affected[OLC_VAL(d)].modifier = 0;
      oedit_disp_prompt_apply_menu(d);
    } else if (number < 0 || number >= NUM_APPLIES)
      oedit_disp_apply_menu(d);
    else {
      OLC_OBJ(d)->affected[OLC_VAL(d)].location = number;
      send_to_char("Modifier : ", d->character);
      OLC_MODE(d) = OEDIT_APPLYMOD;
    }
    return;

  case OEDIT_APPLYMOD:
    OLC_OBJ(d)->affected[OLC_VAL(d)].modifier = atoi(arg);
    oedit_disp_prompt_apply_menu(d);
    return;

  case OEDIT_EXTRADESC_KEY:
    if (OLC_DESC(d)->keyword)
      FREE(OLC_DESC(d)->keyword);
    OLC_DESC(d)->keyword = str_dup((arg && *arg) ? arg : "undefined");
    oedit_disp_extradesc_menu(d);
    return;

  case OEDIT_EXTRADESC_MENU:
    switch ((number = atoi(arg))) {
    case 0:
      if (!OLC_DESC(d)->keyword || !OLC_DESC(d)->description) {
	struct extra_descr_data **tmp_desc;

	if (OLC_DESC(d)->keyword)
	  FREE(OLC_DESC(d)->keyword);
	if (OLC_DESC(d)->description)
	  FREE(OLC_DESC(d)->description);

	/*
	 * Clean up pointers  
	 */
	for (tmp_desc = &(OLC_OBJ(d)->ex_description); *tmp_desc;
	     tmp_desc = &((*tmp_desc)->next)) {
	  if (*tmp_desc == OLC_DESC(d)) {
	    *tmp_desc = NULL;
	    break;
	  }
	}
	FREE(OLC_DESC(d));
      }
      break;

    case 1:
      OLC_MODE(d) = OEDIT_EXTRADESC_KEY;
      send_to_char("Enter keywords, separated by spaces :-\r\n| ", d->character);
      return;

    case 2:
      OLC_MODE(d) = OEDIT_EXTRADESC_DESCRIPTION;
      SEND_TO_Q("Enter the extra description: (/s saves /h for help)\r\n\r\n", d);
      d->backstr = NULL;
      if (OLC_DESC(d)->description) {
	SEND_TO_Q(OLC_DESC(d)->description, d);
	d->backstr = str_dup(OLC_DESC(d)->description);
      }
      d->str = &OLC_DESC(d)->description;
      d->max_str = MAX_MESSAGE_LENGTH;
      d->mail_to = 0;
      OLC_VAL(d) = 1;
      return;

    case 3:
      /*
       * Only go to the next description if this one is finished.
       */
      if (OLC_DESC(d)->keyword && OLC_DESC(d)->description) {
	struct extra_descr_data *new_extra;

	if (OLC_DESC(d)->next)
	  OLC_DESC(d) = OLC_DESC(d)->next;
	else {			/* Make new extra description and attach at end. */
	  CREATE(new_extra, struct extra_descr_data, 1);
	  OLC_DESC(d)->next = new_extra;
	  OLC_DESC(d) = OLC_DESC(d)->next;
	}
      }
      /*
       * No break - drop into default case.
       */
    default:
      oedit_disp_extradesc_menu(d);
      return;
    }
    break;
  default:
    mudlog("SYSERR: OLC: Reached default case in oedit_parse()!", BRF, LVL_BUILDER, TRUE);
    send_to_char("Oops...\r\n", d->character);
    break;
  }

  /*
   * If we get here, we have changed something.  
   */
  OLC_VAL(d) = 1;
  oedit_disp_menu(d);
}


/* command numbers */
void oedit_disp_commands_menu(struct descriptor_data *d)
{
  int columns = 0, counter;

  *buf = '\0';
  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter < NUM_ACTIVATING_COMMANDS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter, nrm,
	    activating_commands[counter],
	    !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  send_to_char("\r\nEnter command choice : ", d->character);
}

ACMD(do_copyo)
{
  extern void olc_add_to_save_list(int zone, byte type);
  int fmob = 0, tmob = 0, mobnum;

  two_arguments(argument, buf1, buf2);

  fmob = atoi(buf1);
  tmob = atoi(buf2);

  mobnum = real_object(fmob);
  if (mobnum < 1) {
    send_to_char("Error finding from_object", ch);
    return;
  }
  CREATE(ch->desc->olc, struct olc_data, 1);
  STATE(ch->desc) = CON_OEDIT;
  oedit_setup_existing(ch->desc, mobnum);
  OLC_NUM(ch->desc) = tmob;
  if (tmob <= 5000)
    OLC_PROTO(ch->desc) = tmob;
  else
    OLC_PROTO(ch->desc) = fmob;
  OLC_VAL(ch->desc) = 1;	/* So code knows its changed */
  olc_add_to_save_list(tmob / 100, OLC_SAVE_OBJ);
}

void oedit_disp_furn_types(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char(CLS(d), d->character);

  for (counter = 0; counter < NUM_FURN_TYPES; counter++) {
    sprintf(buf, " %s%2d%s)%s%-20.20s %s", grn, counter + 1, nrm, yel,
	    furn_types[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintf(buf, "\r\n%sEnter furniture type : ", nrm);
  send_to_char(buf, d->character);
}

void oedit_disp_virtualdoor_flag_menu(struct descriptor_data *d)
{
  int counter, columns = 0;

  get_char_cols(d->character);

  send_to_char("[H[J", d->character);

  for (counter = 0; counter <= NUM_DOOR_FLAGS; counter++) {
    sprintf(buf, "%s%2d%s) %-20.20s %s", grn, counter + 1, nrm,
	    door_flags[counter], !(++columns % 2) ? "\r\n" : "");
    send_to_char(buf, d->character);
  }
  sprintf(buf, "\r\nDoor flags: %s%s%s\r\n"
	  "Enter door flags, 0 to quit : ", cyn, buf1, nrm);
  send_to_char(buf, d->character);
}
