/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "comm.h"
#include "spells.h"
#include "utils.h"
#include "db.h"
#include "shop.h"
#include "olc.h"
#include "handler.h"

extern char *motd;
extern char *char_abils;
extern char *class_display[];
extern char *alignment_display[];
extern int top_of_p_table;
extern struct player_index_element *player_table;
extern long top_idnum;

/*
 * Set this to 1 for debugging logs in medit_save_internally.
 */
#if 0
#define DEBUG
#endif

/*
 * Set this to 1 as a last resort to save mobiles.
 */
#if 0
#define I_CRASH
#endif

/*-------------------------------------------------------------------*/
void init_language(struct char_data *ch);
void init_spells(struct char_data *ch);
void init_skills(struct char_data *ch);
void set_weight(struct char_data *ch);
void set_height(struct char_data *ch);
void generate_desc(struct char_data *ch);

/* choice o is illithid, and is not shown right now. Re-add if we
 * truly wish
 */
const char *race_menu = "\r\nSelect a race:\r\n"
"a: Human\tb: Half-Elf\tc: Elf\r\n"
"d: Dwarf\te: Halfling\tf: Gnome\r\n"
"g: Half-Orc\th: Barbarian\ti: Drow\r\n"
"j: Duergar\tk: Ogre\t\tl: Troll\r\n"
"m: Orc\t\tn: Goblin\r\n";

const char *pts_menu = "\r\nWhich attribute do you wish to add to?\r\n"
"a)Strength\r\nb)Dexterity\r\nc)Constitution"
"\r\nd)Intelligence\r\ne)Wisdom\r\nf)Charisma\r\n";

int abil_ok_class(struct char_data *ch, int class)
{
  int ok = 0;
  switch (class) {
  case CLASS_CLERIC:
    if (ch->real_abils.wis > 8)
      ok = 1;
    break;
  case CLASS_DRUID:
    if ((ch->real_abils.wis > 11) && (ch->real_abils.cha > 14))
      ok = 1;
    break;
  case CLASS_FIGHTER:
    // Anyone can be a fighter, even wimps
    //if ((ch->real_abils.str > 8) && (ch->real_abils.con > 6))
    ok = 1;
    break;
  case CLASS_PALADIN:
    if ((ch->real_abils.str > 11)
	&& (ch->real_abils.wis > 12) && (ch->real_abils.con > 8)
	&& (ch->real_abils.cha > 16))
      ok = 1;
    break;
  case CLASS_RANGER:
  case CLASS_MULTI_C_R:
    if ((ch->real_abils.str > 12) && (ch->real_abils.wis > 13)
	&& (ch->real_abils.dex > 12) && (ch->real_abils.con > 13))
      ok = 1;
    break;
  case CLASS_THIEF:
    if (ch->real_abils.dex > 8)
      ok = 1;
    break;
  case CLASS_MAGIC_USER:
    if (ch->real_abils.intel > 8)
      ok = 1;
    break;
  case CLASS_ILLUSIONIST:
    if ((ch->real_abils.intel > 8) && (ch->real_abils.dex > 15))
      ok = 1;
    break;
  case CLASS_BARD:
    if ((ch->real_abils.dex > 11) && (ch->real_abils.intel > 11)
	&& (ch->real_abils.cha > 14))
      ok = 1;
    break;
  case CLASS_MULTI_C_M:
    if ((ch->real_abils.wis > 8) && (ch->real_abils.intel > 8))
      ok = 1;
    break;
  case CLASS_MULTI_C_F:
    if ((ch->real_abils.wis > 8) && (ch->real_abils.str > 8) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  case CLASS_MULTI_F_M:
    if ((ch->real_abils.str > 8) && (ch->real_abils.intel > 8) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  case CLASS_MULTI_F_T:
    if ((ch->real_abils.dex > 8) && (ch->real_abils.str > 8) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  case CLASS_MULTI_M_T:
    if ((ch->real_abils.dex > 8) && (ch->real_abils.intel > 8))
      ok = 1;
    break;
  case CLASS_MULTI_F_M_T:
    if ((ch->real_abils.dex > 8) && (ch->real_abils.intel > 8)
	&& (ch->real_abils.str > 8) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  case CLASS_MULTI_C_T:
    if ((ch->real_abils.dex > 8) && (ch->real_abils.wis > 8))
      ok = 1;
    break;
  case CLASS_MULTI_C_I:
    if ((ch->real_abils.wis > 8) && (ch->real_abils.intel > 8) && (ch->real_abils.dex > 15))
      ok = 1;
    break;
  case CLASS_MULTI_F_I:
    if ((ch->real_abils.str > 8) && (ch->real_abils.intel > 8) && (ch->real_abils.dex > 15) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  case CLASS_MULTI_I_T:
    if ((ch->real_abils.intel > 8) && (ch->real_abils.dex > 15))
      ok = 1;
    break;
  case CLASS_MULTI_C_F_M:
    if ((ch->real_abils.str > 8) && (ch->real_abils.intel > 8) && (ch->real_abils.wis > 8) && (ch->real_abils.con > 6))
      ok = 1;
    break;
  default:
    ok = 1;
    break;
  }
  return ((ok) ? 1 : 0);
}

void display_classes(struct descriptor_data *d)
{
  int x;
  extern int class_ok_race[NUM_RACES][NUM_CLASSES];

  *buf = '\0';

  SEND_TO_Q("\r\n   Class selection menu \r\n  Please choose a class\r\n--------------------\r\n", d);
  for (x = 0; x < NUM_CLASSES; x++)
    if (class_ok_race[(int) GET_RACE(d->character) - 1][x])
      send_to_char(class_display[x], d->character);
  send_to_char("\nClass: ", d->character);
}

void display_alignments(struct descriptor_data *d)
{
  int x;
  extern int alignment_ok_class[NUM_CLASSES][NUM_ALIGNMENT];
  extern int alignment_ok_race[NUM_PC_RACES][NUM_ALIGNMENT];

  SEND_TO_Q("\r\n   Alignment selection menu \r\n Please choose an alignment\r\n--------------------\r\n", d);
  for (x = 0; x < NUM_ALIGNMENT; x++)
    if (alignment_ok_class[(int) GET_CLASS(d->character)][x])
      if (alignment_ok_race[(int) GET_RACE(d->character)][x])
	send_to_char(alignment_display[x], d->character);
  send_to_char("\nAlignment: ", d->character);
}

/*
 * Roll the 6 stats for a character... each stat is made of the sum of
 * the best 3 out of 4 rolls of a 6-sided die.  Each class then decides
 * which priority will be given for the best to worst stats.
 */
void roll_real_abils(struct char_data *ch)
{
  int i, j, k, temp;
  ubyte table[6];
  ubyte rolls[4];

  for (i = 0; i < 6; i++)
    table[i] = 0;

  for (i = 0; i < 6; i++) {

    for (j = 0; j < 4; j++)
      rolls[j] = number(3, 6);

    temp = rolls[0] + rolls[1] + rolls[2] + rolls[3] -
	MIN(rolls[0], MIN(rolls[1], MIN(rolls[2], rolls[3])));

    for (k = 0; k < 6; k++)
      if (table[k] < temp) {
	temp ^= table[k];
	table[k] ^= temp;
	temp ^= table[k];
      }
  }

  ch->real_abils.str_add = 0;

  switch (GET_CLASS(ch)) {
  case CLASS_PALADIN:
    ch->real_abils.str = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.cha = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.dex = table[4];
    ch->real_abils.intel = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_RANGER:
    ch->real_abils.str = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.wis = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.dex = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_BARD:
    ch->real_abils.cha = table[0];
    ch->real_abils.dex = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.intel = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.con = table[5];
    break;
  case CLASS_MULTI_C_M:
    ch->real_abils.intel = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.dex = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MULTI_F_M:
    ch->real_abils.str = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_F_T:
    ch->real_abils.str = table[0];
    ch->real_abils.dex = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.wis = table[3];
    ch->real_abils.intel = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_M_T:
    ch->real_abils.intel = table[0];
    ch->real_abils.dex = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.str = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MULTI_F_M_T:
    ch->real_abils.str = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.dex = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_C_R:
    ch->real_abils.str = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.intel = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.con = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MULTI_C_F:
    ch->real_abils.str = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.intel = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_I_T:
    ch->real_abils.dex = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MULTI_C_F_M:
    ch->real_abils.str = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.intel = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.dex = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_C_T:
    ch->real_abils.dex = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.intel = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MULTI_F_I:
    ch->real_abils.str = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  case CLASS_MULTI_C_I:
    ch->real_abils.wis = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.con = table[3];
    ch->real_abils.dex = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_MAGIC_USER:
  case CLASS_ILLUSIONIST:
    ch->real_abils.intel = table[0];
    ch->real_abils.wis = table[1];
    ch->real_abils.dex = table[2];
    ch->real_abils.str = table[3];
    ch->real_abils.con = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_CLERIC:
  case CLASS_DRUID:
    ch->real_abils.wis = table[0];
    ch->real_abils.intel = table[1];
    ch->real_abils.str = table[2];
    ch->real_abils.dex = table[3];
    ch->real_abils.con = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_THIEF:
    ch->real_abils.dex = table[0];
    ch->real_abils.str = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.intel = table[3];
    ch->real_abils.wis = table[4];
    ch->real_abils.cha = table[5];
    break;
  case CLASS_FIGHTER:
    ch->real_abils.str = table[0];
    ch->real_abils.dex = table[1];
    ch->real_abils.con = table[2];
    ch->real_abils.wis = table[3];
    ch->real_abils.intel = table[4];
    ch->real_abils.cha = table[5];
    if (ch->real_abils.str == 18)
      ch->real_abils.str_add = number(0, 100);
    break;
  }
  ch->aff_abils = ch->real_abils;

  if (!abil_ok_class(ch, GET_CLASS(ch)))
    roll_real_abils(ch);
}

void set_racial_mod(struct char_data *ch)
{
  switch (GET_RACE(ch)) {
  case RACE_ELF:
  case RACE_P_DROW:
    ch->real_abils.intel += 1;
    ch->real_abils.dex += 1;
    ch->real_abils.con -= 1;
    break;
  case RACE_GNOME:
    ch->real_abils.intel += 1;
    ch->real_abils.wis -= 1;
    break;
  case RACE_DWARF:
  case RACE_P_DUERGAR:
    ch->real_abils.con += 1;
    ch->real_abils.cha -= 1;
    if (ch->real_abils.dex > 17)
      ch->real_abils.dex = 17;
    if (ch->real_abils.cha > 17)
      ch->real_abils.cha = 17;
    break;
  case RACE_HALFLING:
    ch->real_abils.dex += 1;
    ch->real_abils.str -= 1;
    if (ch->real_abils.wis > 17)
      ch->real_abils.wis = 17;
    ch->real_abils.str_add = 0;
    break;
  case RACE_HALFORC:
    ch->real_abils.con += 1;
    ch->real_abils.str += 1;
    ch->real_abils.cha -= 2;
    if (ch->real_abils.str > 18)
      ch->real_abils.str = 18;
    if (ch->real_abils.cha > 16)
      ch->real_abils.cha = 16;
  case RACE_BARBARIAN:
    ch->real_abils.con += 1;
    ch->real_abils.str += 1;
    ch->real_abils.wis -= 1;
    ch->real_abils.intel -= 1;
    break;
  case RACE_P_TROLL:
  case RACE_P_OGRE:
    ch->real_abils.con += 2;
    ch->real_abils.str = 19;
    ch->real_abils.wis -= 3;
    ch->real_abils.intel -= 2;
    ch->real_abils.cha -= 2;
    break;
  case RACE_P_ILLITHID:
    ch->real_abils.intel = 20;
    ch->real_abils.con -= 2;
    ch->real_abils.cha = 3;
    break;
  case RACE_P_ORC:
  case RACE_P_GOBLIN:
    ch->real_abils.dex += 1;
    ch->real_abils.str -= 1;
    if (ch->real_abils.wis > 15)
      ch->real_abils.wis = 15;
    ch->real_abils.str_add = 0;
    break;
  default:
    break;
  }

  ch->aff_abils = ch->real_abils;
}



/* initialize a new character only if class is set */
void init_char(struct char_data *ch)
{
  int i;

  /* *** if this is our first player --- he be God *** */

/*  if (top_of_p_table == 0) {
   GET_EXP(ch) = 7000000;
   GET_LEVEL(ch) = LVL_IMPL;
   GET_TOT_LEVEL(ch) = LVL_IMPL * NUM_CLASSES;

   ch->real_abils.intel = 25;
   ch->real_abils.wis = 25;
   ch->real_abils.dex = 25;
   ch->real_abils.str = 25;
   ch->real_abils.str_add = 0;
   ch->real_abils.con = 25;
   ch->real_abils.cha = 25;


   ch->points.max_hit = 500;
   } else {

   GET_EXP(ch) = 7000000;
   GET_LEVEL(ch) = LVL_IMPL;
   GET_TOT_LEVEL(ch) = LVL_IMPL * NUM_CLASSES;

   ch->points.max_hit = 500;

   } */
  set_title(ch, NULL);

  ch->player.short_descr = NULL;
  ch->player.long_descr = NULL;
  ch->player.description = NULL;

  ch->num_completed_quests = -1;
  ch->completed_quests = NULL;
  ch->current_quest = -1;

  ch->player.hometown = 1;

  ch->player.time.birth = time(0);
  ch->player.time.played = 0;
  ch->player.time.logon = time(0);

  for (i = 0; i < MAX_TONGUE; i++)
    GET_TALK(ch, i) = 0;


  ch->points.hit = GET_MAX_HIT(ch);
  ch->points.armor = 10;

  player_table[top_of_p_table].id = GET_IDNUM(ch) = ++top_idnum;

  ch->char_specials.saved.affected_by = 0;
  ch->char_specials.saved.affected_by2 = 0;
  ch->char_specials.saved.affected_by3 = 0;
  ch->char_specials.saved.affected_by4 = 0;

  for (i = 0; i < 5; i++)
    GET_SAVE(ch, i) = 0;

  for (i = 0; i < 3; i++)
    GET_COND(ch, i) = (GET_LEVEL(ch) == LVL_IMPL ? -1 : 24);

  switch (GET_CLASS(ch)) {
  case CLASS_FIGHTER:
  case CLASS_PALADIN:
  case CLASS_RANGER:
  case CLASS_MULTI_F_M:
  case CLASS_MULTI_F_T:
  case CLASS_MULTI_F_M_T:
  case CLASS_MULTI_C_R:
  case CLASS_MULTI_C_F_M:
  case CLASS_MULTI_F_I:
    if (GET_RACE(ch) != RACE_HALFLING) {
      if (ch->real_abils.str == 18)
	ch->real_abils.str_add = number(1, 100);
    }
    break;
  }

  set_height(ch);
  set_weight(ch);
  init_spells(ch);
  init_skills(ch);
  init_language(ch);

  GET_DOM_HAND(ch) = ((number(0, 100) < 75) ? HAND_RIGHT : HAND_LEFT);

  GET_LOADROOM(ch) = NOWHERE;
}

void set_height(struct char_data *ch)
{
  switch (ch->player.race) {
  case RACE_HUMAN:
  case RACE_BARBARIAN:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 60 + number(2, 20);
    else
      ch->player.height = 59 + number(2, 20);
    break;
  case RACE_HALFORC:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 60 + number(1, 12);
    else
      ch->player.height = 58 + number(1, 12);
    break;
  case RACE_HALFELF:
  case RACE_P_ILLITHID:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 60 + number(2, 12);
    else
      ch->player.height = 58 + number(2, 12);
    break;
  case RACE_ELF:
  case RACE_P_DROW:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 55 + number(1, 10);
    else
      ch->player.height = 50 + number(1, 10);
    break;
  case RACE_HALFLING:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 32 + number(2, 16);
    else
      ch->player.height = 30 + number(2, 16);
    break;
  case RACE_DWARF:
  case RACE_P_DUERGAR:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 43 + number(1, 10);
    else
      ch->player.height = 41 + number(1, 10);
    break;
  case RACE_GNOME:
  case RACE_GOBLIN:
  case RACE_ORC:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 38 + number(1, 6);
    else
      ch->player.height = 36 + number(1, 6);
    break;
  case RACE_P_TROLL:
  case RACE_P_OGRE:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 85 + number(2, 20);
    else
      ch->player.height = 80 + number(2, 20);
  default:
    if (ch->player.sex == SEX_MALE)
      ch->player.height = 60 + number(2, 20);
    else
      ch->player.height = 59 + number(2, 20);
  }
}

/*A note about weight. The first number value is sort of a fat factor, or
   the general body tone of the character. Some people are bigboned, others
   are not. The constitution of a character effects this (ie. are they
   healthy, anorexic, etc etc.). This is the weight of the character (in
   grams) per centimeter of height. We then divide by 1000 to get weight in
   kilograms. Yes, this is a lot of work for one lousy stat, but I never
   liked having someone with a 6 con and all of 5 foot nothing weighing
   300 lbs. Sure its possible, but adventurers are supposed to be fit. */

void set_weight(struct char_data *ch)
{
  switch (ch->player.race) {
  case RACE_HUMAN:
  case RACE_BARBARIAN:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(250, 500) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_HALFORC:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(250, 500) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_HALFELF:
  case RACE_P_ILLITHID:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(300, 550) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(200, 450) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_ELF:
  case RACE_P_DROW:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(150, 400) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(100, 300) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_HALFLING:
  case RACE_P_ORC:
  case RACE_P_GOBLIN:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(450, 700) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(400, 650) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_DWARF:
  case RACE_P_DUERGAR:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(600, 800) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(550, 750) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_GNOME:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    break;
  case RACE_P_OGRE:
  case RACE_P_TROLL:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(850, 1100) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(750, 1000) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
  default:
    if (ch->player.sex == SEX_MALE)
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
    else
      ch->player.weight =
	  ((number(350, 600) + 15 * (GET_CON(ch) - 9)) *
	   ch->player.height) / 1000;
  }
  ch->player.weight *= 2.5;
  ch->player.weight *= 2.2;
}


void pedit_parse(struct descriptor_data *d, char *arg)
{
  int load_result;
  int parse_class(struct char_data *ch, char arg);
  int parse_alignment(struct char_data *ch, char arg);
  int parse_race(char arg);
  void disp_char_sheet(struct char_data *ch);

  switch (OLC_MODE(d)) {
  case PEDIT_QSEX:		/* query sex of new user         */
    switch (*arg) {
    case 'm':
    case 'M':
      d->character->player.sex = SEX_MALE;
      break;
    case 'f':
    case 'F':
      d->character->player.sex = SEX_FEMALE;
      break;
    default:
      SEND_TO_Q("That is not a sex..\r\n"
		"What IS your sex? ", d);
      return;
      break;
    }

    break;

  case PEDIT_QRACE:
    load_result = parse_race(*arg);
    if (load_result == RACE_NONE) {
      SEND_TO_Q("\r\nThat's not a race.\r\n", d);
      SEND_TO_Q(race_menu, d);
      SEND_TO_Q("\r\nRace: ", d);
      return;
    } else
      GET_RACE(d->character) = load_result;

    break;

  case PEDIT_QCLASS:
    load_result = parse_class(d->character, *arg);
    if (load_result == CLASS_UNDEFINED) {
      SEND_TO_Q("\r\nThat's not a class.\r\nClass: ", d);
      return;
    } else
      GET_CLASS(d->character) = load_result;

    if (GET_PFILEPOS(d->character) < 0)
      GET_PFILEPOS(d->character) = create_entry(GET_NAME(d->character));
    init_char(d->character);
    SET_BIT(MULTI_FLAGS(d->character), (1 << (int) GET_CLASS(d->character)));

    break;

  case PEDIT_QEXDESC:
    /* handled in modify.c */
    break;

  case PEDIT_INTROD1:
    switch (*arg) {
    case 'y':
    case 'Y':
      generate_desc(d->character);
      break;
    case 'n':
    case 'N':
      OLC_MODE(d) = PEDIT_INTROD3;
      break;
    default:
      break;
    }
    break;
  case PEDIT_INTROD2:
    switch (*arg) {
    case 'y':
    case 'Y':
      generate_desc(d->character);
      break;
    case 'n':
    case 'N':
      OLC_MODE(d) = PEDIT_INTROD3;
      break;
    default:
      break;
    }
    break;

  case PEDIT_INTROD3:
    break;

  case PEDIT_ALIGNMENT:
    load_result = parse_alignment(d->character, *arg);
    if (load_result == ALIGN_NONE) {
      SEND_TO_Q("\r\nThat's not an alignment.\r\nAlignment: ", d);
      return;
    } else {
      SET_ALIGNMENT(d->character, load_result);
      generate_desc(d->character);
    }
    break;

  default:
    /*
     * We should never get here.
     */
    cleanup_olc(d, CLEANUP_ALL);
    mudlog("SYSERR: OLC: pedit_parse(): Reached default case!", BRF,
	   LVL_BUILDER, TRUE);
    send_to_char("Oops...\r\n", d->character);
    break;
  }

/*
 * END OF CASE 
 * If we get here, we have probably changed something, and now want to
 * return to main menu.  Use OLC_VAL as a 'has changed' flag  
 */

  send_to_char("[H[J", d->character);
//  disp_char_sheet(d->character);

  switch (OLC_MODE(d)) {

  case PEDIT_QSEX:		/* query sex of new user         */
    SEND_TO_Q("\r\nWhich race do you wish to be?\r\n", d);
    SEND_TO_Q(race_menu, d);
    SEND_TO_Q("\r\nRace: ", d);
    OLC_MODE(d) = PEDIT_QRACE;
    break;

  case PEDIT_QRACE:
    /*** Skip the description for now, move on to class ***
    SEND_TO_Q("\r\nEnter a brief visual description of yourself:\r\n", d);
    SEND_TO_Q("(/s saves /h for help)\r\n", d);
    d->backstr = NULL;
    d->str = &d->character->player.description;
    d->max_str = EXDSCR_LENGTH;
    OLC_MODE(d) = PEDIT_QEXDESC;
    break;
    */
    display_classes(d);
    OLC_MODE(d) = PEDIT_QCLASS;
    break;

  case PEDIT_QEXDESC:
    display_classes(d);
    OLC_MODE(d) = PEDIT_QCLASS;
    break;

  case PEDIT_QCLASS:
    /* class has been chosen, so roll some appropriate stats */
    roll_real_abils(d->character);
    /* adjust them for their race */
    set_racial_mod(d->character);
    /* and carry garth */
    display_alignments(d);
    OLC_MODE(d) = PEDIT_ALIGNMENT;
    break;
  case PEDIT_ALIGNMENT:
    /* create a player_special structure */
    if (d->character->player_specials == NULL)
      CREATE(d->character->player_specials, struct player_special_data, 1);
    generate_desc(d->character);
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q(d->character->player.short_descr, d);
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q("\r\nA short description has been generated for you.\r\n", d);
    SEND_TO_Q("\r\nYou may try for a better one 2 more times. Do you wish to reroll? ", d);
    OLC_MODE(d) = PEDIT_INTROD1;
    break;
  case PEDIT_INTROD1:
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q(d->character->player.short_descr, d);
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q("\r\nYou now have 1 more roll. Do you wish to reroll? ", d);
    OLC_MODE(d) = PEDIT_INTROD2;
    break;
  case PEDIT_INTROD2:
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q(d->character->player.short_descr, d);
    SEND_TO_Q("\r\n", d);
    SEND_TO_Q("\r\nYou have no more rolls left.\r\n", d);
    OLC_MODE(d) = PEDIT_INTROD3;
    break;
  case PEDIT_INTROD3:
    save_char(d->character, NOWHERE);
    SEND_TO_Q(motd, d);
    SEND_TO_Q("\r\n\n*** PRESS RETURN: ", d);
    FREE(d->olc);
    STATE(d) = CON_RMOTD;

    sprintf(buf, "%s [%s] new player.", GET_NAME(d->character), d->host);
    mudlog(buf, NRM, LVL_IMMORT, TRUE);
    break;
  default:
    /*
     * We should never get here.
     */
    cleanup_olc(d, CLEANUP_ALL);
    mudlog("SYSERR: OLC: pedit_parse(): Reached default case!", BRF,
	   LVL_BUILDER, TRUE);
    send_to_char("Oops...\r\n", d->character);
    break;
  }

}
