/* ************************************************************************
   *   File: gha_procs.c                                                     *
   *  Usage: Special object procs for GreyHawk Adventures                    *
   *                                                                         *
   *  1998 Bradley Hartin, AKA Vrallis                                       *
   ************************************************************************ */

#include "conf.h"
#include "sysdep.h"
#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"

/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;

void ASSIGNOBJ(int obj, SPECIAL(fname));

void assign_vrallis_procs(void)
{
  SPECIAL(robe_useful_items);

  ASSIGNOBJ(3032, robe_useful_items);
}


#define RUI_TEN(val, ten) \
               ((val - (val % ten) - ((val / (ten * 10)) * ten * 10)) / ten)

SPECIAL(robe_useful_items)
{
  int bits;
  int r, i;
  char buf2[MAX_STRING_LENGTH];
  struct char_data *dummy;
  struct obj_data *temp_obj;
  struct obj_data *object;
  struct obj_data *obj;
  struct rui_items {
    char *desc;
    int val;
    int ten;
  };

  struct rui_items rui[] =
  {
    {"dagger", 1, 1},		/* vnum 1004   */
    {"lantern", 1, 10},		/* vnum 402    */
    {"rope", 1, 100},		/* NA          */
    {"backpack", 1, 1000},	/* vnum 801    */
    {"bag of gold", 2, 1},	/* NA          */
    {"coffer", 2, 10},		/* NA          */
    {"gems", 2, 100},		/* vnum 489-496 */
    {"mule", 2, 1000},		/* NA          */
    {"potion of extra healing", 3, 1},	/* NA          */
    {"rowboat", 3, 10},		/* NA          */
    {"scroll", 3, 100},		/* NA          */
    {"war dogs", 3, 1000}	/* NA          */
  };

  if (!ch || (!CMD_IS("tear") && !CMD_IS("examine") &&
	      !CMD_IS("look") && !CMD_IS("wear")))
    return FALSE;

  if (!*argument)
    return FALSE;

  if (IS_NPC(ch))
    return FALSE;

  obj = me;

  bits = generic_find(argument, FIND_OBJ_EQUIP | FIND_OBJ_INV, ch,
		      &dummy, &object);

  if (obj != object)		/* we aren't the target */
    return FALSE;

  if (CMD_IS("wear")) {

    if (obj->obj_flags.value[0] == 815)
      return FALSE;
    /* is robe initialized? */
    /* new robe, let's fix it up and make it unique */

    obj->obj_flags.value[0] = 815;	/* 'magic' number */
    obj->obj_flags.value[1] = 2222;	/* two each of stock items */
    obj->obj_flags.value[2] = 0;
    obj->obj_flags.value[3] = 0;

    for (i = 0; i < number(2, 9); i++) {
      r = number(4, 11);
      obj->obj_flags.value[rui[r].val] += rui[r].ten;
    }

    act("A multitude of familiar shapes form on the patches of $p&n.",
	FALSE, ch, obj, 0, TO_CHAR);

    return FALSE;
  } else {

    /* robe has to be worn to use it */
    bits = generic_find(argument, FIND_OBJ_EQUIP, ch, &dummy, &object);
    if (obj != object)
      return FALSE;

    if (CMD_IS("look") || CMD_IS("examine")) {
      /* let's tell the owner what patches remain */
      sprintf(buf,
	  "You examine the robe carefully, and notice a multitude of\r\n"
	 "patches depicting familiar items.  Remembering the legends\r\n"
	      "of this robe, you diligently take note of the patches.\r\n"
	      "Remaining patches:\r\n");

      for (i = 0; i < 12; i++)
	if (RUI_TEN(obj->obj_flags.value[rui[i].val], rui[i].ten))
	  sprintf(buf, "%s(%d) %s\r\n", buf,
		  RUI_TEN(obj->obj_flags.value[rui[i].val], rui[i].ten),
		  rui[i].desc);

      send_to_char(buf, ch);
      return TRUE;
    } else {

      two_arguments(argument, buf, buf2);

      if (!strcmp(buf2, "dagger")) {
	if (RUI_TEN(obj->obj_flags.value[1], 1) < 1) {
	  act("You search $p&n, but are unable to find a patch like that.",
	      FALSE, ch, obj, 0, TO_CHAR);
	  return TRUE;
	}
	obj->obj_flags.value[1] -= 1;
	temp_obj = read_object(1004, VIRTUAL);
	act("You tear a patch off $p&n, which turns into a dagger!",
	    FALSE, ch, obj, 0, TO_CHAR);
	act("$n tears a patch off $p&n, which turns into a dagger!.",
	    FALSE, ch, obj, 0, TO_PROXIMITY);
	put_obj_to_room(temp_obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	return TRUE;
      } else if (!strcmp(buf2, "lantern")) {
	if (RUI_TEN(obj->obj_flags.value[1], 10) < 1) {
	  act("You search $p&n, but are unable to find a patch like that.",
	      FALSE, ch, obj, 0, TO_CHAR);
	  return TRUE;
	}
	obj->obj_flags.value[1] -= 10;
	temp_obj = read_object(402, VIRTUAL);
	act("You tear a patch off $p&n, which turns into a lantern!",
	    FALSE, ch, obj, 0, TO_CHAR);
	act("$n tears a patch off $p&n, which turns into a lantern!.",
	    FALSE, ch, obj, 0, TO_PROXIMITY);
	put_obj_to_room(temp_obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	return TRUE;
      } else if (!strcmp(buf2, "backpack")) {
	if (RUI_TEN(obj->obj_flags.value[1], 1000) < 1) {
	  act("You search $p&n, but are unable to find a patch like that.",
	      FALSE, ch, obj, 0, TO_CHAR);
	  return TRUE;
	}
	obj->obj_flags.value[1] -= 1000;
	temp_obj = read_object(801, VIRTUAL);
	act("You tear a patch off $p&n, which turns into a backpack!",
	    FALSE, ch, obj, 0, TO_CHAR);
	act("$n tears a patch off $p&n, which turns into a backpack!.",
	    FALSE, ch, obj, 0, TO_PROXIMITY);
	put_obj_to_room(temp_obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	return TRUE;
      } else if (!strcmp(buf2, "gems")) {
	if (RUI_TEN(obj->obj_flags.value[2], 100) < 1) {
	  act("You search $p&n, but are unable to find a patch like that.",
	      FALSE, ch, obj, 0, TO_CHAR);
	  return TRUE;
	}
	obj->obj_flags.value[2] -= 100;
	act("You tear a patch off $p&n, which turns into a pile of gems!",
	    FALSE, ch, obj, 0, TO_CHAR);
	act("$n tears a patch off $p&n, which turns into a pile of gems!.",
	    FALSE, ch, obj, 0, TO_PROXIMITY);
	for (i = 0; i < 10; i++) {
	  temp_obj = read_object(number(489, 496), VIRTUAL);
	  put_obj_to_room(temp_obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
	}
	return TRUE;
      } else {
	act("You tear a patch off $p&n, but nothing happens.",
	    FALSE, ch, obj, 0, TO_CHAR);
	act("$n tears a patch off $p&n, but nothing happens.",
	    FALSE, ch, obj, 0, TO_PROXIMITY);
      }
      return TRUE;
    }
  }
  send_to_char("Serious error in robe_useful_items, please inform a god.", ch);
  return FALSE;
}
